@@ -168,14 +168,14 @@ start_queue_timer (GstAlsaMidiSrc * alsamidisrc)
 
   ret = snd_seq_start_queue (alsamidisrc->seq, alsamidisrc->queue, NULL);
   if (ret < 0) {
-    GST_ERROR_OBJECT (alsamidisrc, "Timer event output error: %s\n",
+    GST_ERROR_OBJECT (alsamidisrc, "Timer event output error: %s",
         snd_strerror (ret));
     return ret;
   }
 
   ret = snd_seq_drain_output (alsamidisrc->seq);
   if (ret < 0)
-    GST_ERROR_OBJECT (alsamidisrc, "Drain output error: %s\n",
+    GST_ERROR_OBJECT (alsamidisrc, "Drain output error: %s",
         snd_strerror (ret));
 
   return ret;
@@ -198,18 +198,16 @@ schedule_next_tick (GstAlsaMidiSrc * alsamidisrc)
   GST_TIME_TO_TIMESPEC (alsamidisrc->tick * MIDI_TICK_PERIOD_MS * GST_MSECOND,
       time);
 
-  snd_seq_ev_schedule_real (&ev, alsamidisrc->queue, SND_SEQ_TIME_MODE_ABS,
-      &time);
+  snd_seq_ev_schedule_real (&ev, alsamidisrc->queue, 0, &time);
 
   ret = snd_seq_event_output (alsamidisrc->seq, &ev);
   if (ret < 0)
-    GST_ERROR_OBJECT (alsamidisrc, "Event output error: %s\n",
+    GST_ERROR_OBJECT (alsamidisrc, "Event output error: %s",
         snd_strerror (ret));
 
   ret = snd_seq_drain_output (alsamidisrc->seq);
   if (ret < 0)
-    GST_ERROR_OBJECT (alsamidisrc, "Event drain error: %s\n",
-        snd_strerror (ret));
+    GST_ERROR_OBJECT (alsamidisrc, "Event drain error: %s", snd_strerror (ret));
 }
 
 static int
@@ -227,7 +225,7 @@ create_port (GstAlsaMidiSrc * alsamidisrc)
 
   ret = snd_seq_alloc_named_queue (alsamidisrc->seq, DEFAULT_CLIENT_NAME);
   if (ret < 0) {
-    GST_ERROR_OBJECT (alsamidisrc, "Cannot allocate queue: %s\n",
+    GST_ERROR_OBJECT (alsamidisrc, "Cannot allocate queue: %s",
         snd_strerror (ret));
     return ret;
   }
