@@ -515,6 +515,7 @@ _src_smpte_fill_bound_fbo (gpointer impl)
 {
   struct SrcSMPTE *src = impl;
   gint attr_color_position = -1;
+  GstGLBaseSrc *bsrc = GST_GL_BASE_SRC (src->base.base.src);
 
   src->base.n_attributes = 2;
   if (src->base.shader)
@@ -535,7 +536,7 @@ _src_smpte_fill_bound_fbo (gpointer impl)
   src->base.index_offset = (N_QUADS - 1) * 6 * sizeof (gushort);
   gst_gl_shader_use (src->snow_shader);
   gst_gl_shader_set_uniform_1f (src->snow_shader, "time",
-      (gfloat) src->base.base.src->running_time / GST_SECOND);
+      (gfloat) bsrc->running_time / GST_SECOND);
   if (!_src_shader_fill_bound_fbo (impl))
     return FALSE;
   src->attr_snow_position = src->base.attributes[0].location;
@@ -863,13 +864,14 @@ static gboolean
 _src_snow_fill_bound_fbo (gpointer impl)
 {
   struct SrcShader *src = impl;
+  GstGLBaseSrc *bsrc = GST_GL_BASE_SRC (src->base.src);
 
   g_return_val_if_fail (src->base.context, FALSE);
   g_return_val_if_fail (src->shader, FALSE);
 
   gst_gl_shader_use (src->shader);
   gst_gl_shader_set_uniform_1f (src->shader, "time",
-      (gfloat) src->base.src->running_time / GST_SECOND);
+      (gfloat) bsrc->running_time / GST_SECOND);
 
   return _src_shader_fill_bound_fbo (impl);
 }
@@ -1007,13 +1009,14 @@ static gboolean
 _src_mandelbrot_fill_bound_fbo (gpointer impl)
 {
   struct SrcShader *src = impl;
+  GstGLBaseSrc *bsrc = GST_GL_BASE_SRC (src->base.src);
 
   g_return_val_if_fail (src->base.context, FALSE);
   g_return_val_if_fail (src->shader, FALSE);
 
   gst_gl_shader_use (src->shader);
   gst_gl_shader_set_uniform_1f (src->shader, "time",
-      (gfloat) src->base.src->running_time / GST_SECOND);
+      (gfloat) bsrc->running_time / GST_SECOND);
 
   return _src_shader_fill_bound_fbo (impl);
 }
