@@ -19,23 +19,24 @@
 
 /**
  * SECTION:element-vorbisenc
+ * @title: vorbisenc
  * @see_also: vorbisdec, oggmux
  *
  * This element encodes raw float audio into a Vorbis stream.
- * <ulink url="http://www.vorbis.com/">Vorbis</ulink> is a royalty-free
- * audio codec maintained by the <ulink url="http://www.xiph.org/">Xiph.org
- * Foundation</ulink>.
+ * [Vorbis](http://www.vorbis.com/) is a royalty-free audio codec maintained by
+ * the [Xiph.org Foundation](http://www.xiph.org/).
  *
- * <refsect2>
- * <title>Example pipelines</title>
+ * ## Example pipelines
  * |[
  * gst-launch-1.0 -v audiotestsrc wave=sine num-buffers=100 ! audioconvert ! vorbisenc ! oggmux ! filesink location=sine.ogg
- * ]| Encode a test sine signal to Ogg/Vorbis.  Note that the resulting file
+ * ]|
+ *  Encode a test sine signal to Ogg/Vorbis.  Note that the resulting file
  * will be really small because a sine signal compresses very well.
  * |[
  * gst-launch-1.0 -v autoaudiosrc ! audioconvert ! vorbisenc ! oggmux ! filesink location=alsasrc.ogg
- * ]| Record from a sound card and encode to Ogg/Vorbis.
- * </refsect2>
+ * ]|
+ *  Record from a sound card and encode to Ogg/Vorbis.
+ *
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -306,6 +307,7 @@ gst_vorbis_enc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
   vorbisenc->frequency = GST_AUDIO_INFO_RATE (info);
 
   /* if re-configured, we were drained and cleared already */
+  vorbisenc->header_sent = FALSE;
   if (!gst_vorbis_enc_setup (vorbisenc))
     return FALSE;
 
@@ -904,7 +906,7 @@ gst_vorbis_enc_handle_frame (GstAudioEncoder * enc, GstBuffer * buffer)
   if (!buffer)
     return gst_vorbis_enc_clear (vorbisenc);
 
-  gst_buffer_map (buffer, &map, GST_MAP_WRITE);
+  gst_buffer_map (buffer, &map, GST_MAP_READ);
 
   /* data to encode */
   size = map.size / (vorbisenc->channels * sizeof (float));
