@@ -43,18 +43,6 @@ typedef enum {
   GST_AUDIO_FLAG_UNPOSITIONED      = (1 << 0)
 } GstAudioFlags;
 
-/**
- * GstAudioLayout:
- * @GST_AUDIO_LAYOUT_INTERLEAVED: interleaved audio
- * @GST_AUDIO_LAYOUT_NON_INTERLEAVED: non-interleaved audio
- *
- * Layout of the audio samples for the different channels.
- */
-typedef enum {
-  GST_AUDIO_LAYOUT_INTERLEAVED = 0,
-  GST_AUDIO_LAYOUT_NON_INTERLEAVED
-} GstAudioLayout;
-
 /**
  * GstAudioInfo:
  * @finfo: the format info of the audio
@@ -84,7 +72,9 @@ struct _GstAudioInfo {
   gpointer _gst_reserved[GST_PADDING];
 };
 
-GType gst_audio_info_get_type        (void);
+#define GST_TYPE_AUDIO_INFO                  (gst_audio_info_get_type ())
+GST_AUDIO_API
+GType gst_audio_info_get_type                (void);
 
 #define GST_AUDIO_INFO_IS_VALID(i)           ((i)->finfo != NULL && (i)->rate > 0 && (i)->channels > 0 && (i)->bpf > 0)
 
@@ -111,25 +101,40 @@ GType gst_audio_info_get_type        (void);
 #define GST_AUDIO_INFO_BPF(info)             ((info)->bpf)
 #define GST_AUDIO_INFO_POSITION(info,c)      ((info)->position[c])
 
+GST_AUDIO_API
 GstAudioInfo * gst_audio_info_new         (void);
+
+GST_AUDIO_API
 void           gst_audio_info_init        (GstAudioInfo *info);
+
+GST_AUDIO_API
 GstAudioInfo * gst_audio_info_copy        (const GstAudioInfo *info);
+
+GST_AUDIO_API
 void           gst_audio_info_free        (GstAudioInfo *info);
 
+GST_AUDIO_API
 void           gst_audio_info_set_format  (GstAudioInfo *info, GstAudioFormat format,
                                            gint rate, gint channels,
                                            const GstAudioChannelPosition *position);
 
+GST_AUDIO_API
 gboolean       gst_audio_info_from_caps   (GstAudioInfo *info, const GstCaps *caps);
+
+GST_AUDIO_API
 GstCaps *      gst_audio_info_to_caps     (const GstAudioInfo *info);
 
+GST_AUDIO_API
 gboolean       gst_audio_info_convert     (const GstAudioInfo * info,
                                            GstFormat src_fmt, gint64 src_val,
                                            GstFormat dest_fmt, gint64 * dest_val);
 
+GST_AUDIO_API
 gboolean       gst_audio_info_is_equal    (const GstAudioInfo *info,
                                            const GstAudioInfo *other);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstAudioInfo, gst_audio_info_free)
+
 G_END_DECLS
 
 #endif /* __GST_AUDIO_INFO_H__ */
