@@ -60,6 +60,20 @@
  * have the same sample rate as either the downstream requirement,
  * or the first configured pad, or a combination of both (when
  * downstream specifies a range or a set of acceptable rates).
+ *
+ * The #GstAggregator::samples-selected signal is provided with some
+ * additional information about the output buffer:
+ * - "offset"  G_TYPE_UINT64   Offset in samples since segment start
+ *   for the position that is next to be filled in the output buffer.
+ * - "frames"  G_TYPE_UINT   Number of frames per output buffer.
+ *
+ * In addition the gst_aggregator_peek_next_sample() function returns
+ * additional information in the info #GstStructure of the returned sample:
+ * - "output-offset"  G_TYPE_UINT64   Sample offset in output segment relative to
+ *   the output segment's start where the current position of this input
+ *   buffer would be placed
+ * - "position"  G_TYPE_UINT   current position in the input buffer in samples
+ * - "size"  G_TYPE_UINT   size of the input buffer in samples
  */
 
 
@@ -86,11 +100,9 @@ struct _GstAudioAggregatorPadPrivate
   guint position, size;         /* position in the input buffer and size of the
                                    input buffer in number of samples */
 
-  GstBuffer *input_buffer;
-
   guint64 output_offset;        /* Sample offset in output segment relative to
-                                   pad.segment.start that position refers to
-                                   in the current buffer. */
+                                   srcpad.segment.start where the current position
+                                   of this input_buffer would be placed. */
 
   guint64 next_offset;          /* Next expected sample offset relative to
                                    pad.segment.start */
@@ -125,7 +137,6 @@ gst_audio_aggregator_pad_finalize (GObject * object)
   GstAudioAggregatorPad *pad = (GstAudioAggregatorPad *) object;
 
   gst_buffer_replace (&pad->priv->buffer, NULL);
-  gst_buffer_replace (&pad->priv->input_buffer, NULL);
 
   G_OBJECT_CLASS (gst_audio_aggregator_pad_parent_class)->finalize (object);
 }
@@ -148,7 +159,6 @@ gst_audio_aggregator_pad_init (GstAudioAggregatorPad * pad)
   gst_audio_info_init (&pad->info);
 
   pad->priv->buffer = NULL;
-  pad->priv->input_buffer = NULL;
   pad->priv->position = 0;
   pad->priv->size = 0;
   pad->priv->output_offset = -1;
@@ -168,7 +178,6 @@ gst_audio_aggregator_pad_flush_pad (GstAggregatorPad * aggpad,
   pad->priv->output_offset = pad->priv->next_offset = -1;
   pad->priv->discont_time = GST_CLOCK_TIME_NONE;
   gst_buffer_replace (&pad->priv->buffer, NULL);
-  gst_buffer_replace (&pad->priv->input_buffer, NULL);
   GST_OBJECT_UNLOCK (aggpad);
 
   return GST_FLOW_OK;
@@ -210,8 +219,8 @@ gst_audio_aggregator_convert_pad_update_converter (GstAudioAggregatorConvertPad
     aaggcpad->priv->converter =
         gst_audio_converter_new (GST_AUDIO_CONVERTER_FLAG_NONE,
         in_info, out_info,
-        aaggcpad->priv->converter_config ? gst_structure_copy (aaggcpad->
-            priv->converter_config) : NULL);
+        aaggcpad->priv->converter_config ? gst_structure_copy (aaggcpad->priv->
+            converter_config) : NULL);
   }
 
   aaggcpad->priv->converter_config_changed = FALSE;
@@ -368,10 +377,17 @@ struct _GstAudioAggregatorPrivate
 
   /* All three properties are unprotected, can't be modified while streaming */
   /* Size in frames that is output per buffer */
-  GstClockTime output_buffer_duration;
   GstClockTime alignment_threshold;
   GstClockTime discont_wait;
 
+  gint output_buffer_duration_n;
+  gint output_buffer_duration_d;
+
+  guint samples_per_buffer;
+  guint error_per_buffer;
+  guint accumulated_error;
+  guint current_blocksize;
+
   /* Protected by srcpad stream clock */
   /* Output buffer starting at offset containing blocksize frames (calculated
    * from output_buffer_duration) */
@@ -382,6 +398,10 @@ struct _GstAudioAggregatorPrivate
 
   /* Sample offset starting from 0 at aggregator.segment.start */
   gint64 offset;
+
+  /* info structure passed to selected-samples signal, must only be accessed
+   * from the aggregate thread */
+  GstStructure *selected_samples_info;
 };
 
 #define GST_AUDIO_AGGREGATOR_LOCK(self)   g_mutex_lock (&(self)->priv->mutex);
@@ -420,10 +440,14 @@ gst_audio_aggregator_update_src_caps (GstAggregator * agg,
     GstCaps * caps, GstCaps ** ret);
 static GstCaps *gst_audio_aggregator_fixate_src_caps (GstAggregator * agg,
     GstCaps * caps);
+static GstSample *gst_audio_aggregator_peek_next_sample (GstAggregator * agg,
+    GstAggregatorPad * aggpad);
 
 #define DEFAULT_OUTPUT_BUFFER_DURATION (10 * GST_MSECOND)
 #define DEFAULT_ALIGNMENT_THRESHOLD   (40 * GST_MSECOND)
 #define DEFAULT_DISCONT_WAIT (1 * GST_SECOND)
+#define DEFAULT_OUTPUT_BUFFER_DURATION_N (1)
+#define DEFAULT_OUTPUT_BUFFER_DURATION_D (100)
 
 enum
 {
@@ -431,6 +455,7 @@ enum
   PROP_OUTPUT_BUFFER_DURATION,
   PROP_ALIGNMENT_THRESHOLD,
   PROP_DISCONT_WAIT,
+  PROP_OUTPUT_BUFFER_DURATION_FRACTION,
 };
 
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GstAudioAggregator, gst_audio_aggregator,
@@ -448,6 +473,79 @@ gst_audio_aggregator_convert_buffer (GstAudioAggregator * aagg, GstPad * pad,
   return klass->convert_buffer (aaggpad, in_info, out_info, buffer);
 }
 
+static void
+gst_audio_aggregator_translate_output_buffer_duration (GstAudioAggregator *
+    aagg, GstClockTime duration)
+{
+  gint gcd;
+
+  aagg->priv->output_buffer_duration_n = duration;
+  aagg->priv->output_buffer_duration_d = GST_SECOND;
+
+  gcd = gst_util_greatest_common_divisor (aagg->priv->output_buffer_duration_n,
+      aagg->priv->output_buffer_duration_d);
+
+  if (gcd) {
+    aagg->priv->output_buffer_duration_n /= gcd;
+    aagg->priv->output_buffer_duration_d /= gcd;
+  }
+}
+
+static gboolean
+gst_audio_aggregator_update_samples_per_buffer (GstAudioAggregator * aagg)
+{
+  gboolean ret = TRUE;
+  GstAudioAggregatorPad *srcpad =
+      GST_AUDIO_AGGREGATOR_PAD (GST_AGGREGATOR_SRC_PAD (aagg));
+
+  if (!srcpad->info.finfo
+      || GST_AUDIO_INFO_FORMAT (&srcpad->info) == GST_AUDIO_FORMAT_UNKNOWN) {
+    ret = FALSE;
+    goto out;
+  }
+
+  aagg->priv->samples_per_buffer =
+      (((guint64) GST_AUDIO_INFO_RATE (&srcpad->info)) *
+      aagg->priv->output_buffer_duration_n) /
+      aagg->priv->output_buffer_duration_d;
+
+  if (aagg->priv->samples_per_buffer == 0) {
+    ret = FALSE;
+    goto out;
+  }
+
+  aagg->priv->error_per_buffer =
+      (((guint64) GST_AUDIO_INFO_RATE (&srcpad->info)) *
+      aagg->priv->output_buffer_duration_n) %
+      aagg->priv->output_buffer_duration_d;
+  aagg->priv->accumulated_error = 0;
+
+  GST_DEBUG_OBJECT (aagg, "Buffer duration: %u/%u",
+      aagg->priv->output_buffer_duration_n,
+      aagg->priv->output_buffer_duration_d);
+  GST_DEBUG_OBJECT (aagg, "Samples per buffer: %u (error: %u/%u)",
+      aagg->priv->samples_per_buffer, aagg->priv->error_per_buffer,
+      aagg->priv->output_buffer_duration_d);
+
+out:
+  return ret;
+}
+
+static void
+gst_audio_aggregator_recalculate_latency (GstAudioAggregator * aagg)
+{
+  guint64 latency = gst_util_uint64_scale_int (GST_SECOND,
+      aagg->priv->output_buffer_duration_n,
+      aagg->priv->output_buffer_duration_d);
+
+  gst_aggregator_set_latency (GST_AGGREGATOR (aagg), latency, latency);
+
+  GST_OBJECT_LOCK (aagg);
+  /* Force recalculating in aggregate */
+  aagg->priv->samples_per_buffer = 0;
+  GST_OBJECT_UNLOCK (aagg);
+}
+
 static void
 gst_audio_aggregator_class_init (GstAudioAggregatorClass * klass)
 {
@@ -477,6 +575,7 @@ gst_audio_aggregator_class_init (GstAudioAggregatorClass * klass)
   gstaggregator_class->fixate_src_caps = gst_audio_aggregator_fixate_src_caps;
   gstaggregator_class->negotiated_src_caps =
       gst_audio_aggregator_negotiated_src_caps;
+  gstaggregator_class->peek_next_sample = gst_audio_aggregator_peek_next_sample;
 
   klass->create_output_buffer = gst_audio_aggregator_create_output_buffer;
 
@@ -489,6 +588,23 @@ gst_audio_aggregator_class_init (GstAudioAggregatorClass * klass)
           G_MAXUINT64, DEFAULT_OUTPUT_BUFFER_DURATION,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstAudioAggregator:output-buffer-duration-fraction:
+   *
+   * Output block size in nanoseconds, expressed as a fraction.
+   *
+   * Since: 1.18
+   */
+  g_object_class_install_property (gobject_class,
+      PROP_OUTPUT_BUFFER_DURATION_FRACTION,
+      gst_param_spec_fraction ("output-buffer-duration-fraction",
+          "Output buffer duration fraction",
+          "Output block size in nanoseconds, expressed as a fraction", 1,
+          G_MAXINT, G_MAXINT, 1, DEFAULT_OUTPUT_BUFFER_DURATION_N,
+          DEFAULT_OUTPUT_BUFFER_DURATION_D,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
+          GST_PARAM_MUTABLE_READY));
+
   g_object_class_install_property (gobject_class, PROP_ALIGNMENT_THRESHOLD,
       g_param_spec_uint64 ("alignment-threshold", "Alignment Threshold",
           "Timestamp alignment threshold in nanoseconds", 0,
@@ -510,14 +626,17 @@ gst_audio_aggregator_init (GstAudioAggregator * aagg)
 
   g_mutex_init (&aagg->priv->mutex);
 
-  aagg->priv->output_buffer_duration = DEFAULT_OUTPUT_BUFFER_DURATION;
   aagg->priv->alignment_threshold = DEFAULT_ALIGNMENT_THRESHOLD;
   aagg->priv->discont_wait = DEFAULT_DISCONT_WAIT;
 
+  gst_audio_aggregator_translate_output_buffer_duration (aagg,
+      DEFAULT_OUTPUT_BUFFER_DURATION);
+  gst_audio_aggregator_recalculate_latency (aagg);
+
   aagg->current_caps = NULL;
 
-  gst_aggregator_set_latency (GST_AGGREGATOR (aagg),
-      aagg->priv->output_buffer_duration, aagg->priv->output_buffer_duration);
+  aagg->priv->selected_samples_info =
+      gst_structure_new_empty ("GstAudioAggregatorSelectedSamplesInfo");
 }
 
 static void
@@ -527,6 +646,8 @@ gst_audio_aggregator_dispose (GObject * object)
 
   gst_caps_replace (&aagg->current_caps, NULL);
 
+  gst_clear_structure (&aagg->priv->selected_samples_info);
+
   g_mutex_clear (&aagg->priv->mutex);
 
   G_OBJECT_CLASS (gst_audio_aggregator_parent_class)->dispose (object);
@@ -540,10 +661,10 @@ gst_audio_aggregator_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_OUTPUT_BUFFER_DURATION:
-      aagg->priv->output_buffer_duration = g_value_get_uint64 (value);
-      gst_aggregator_set_latency (GST_AGGREGATOR (aagg),
-          aagg->priv->output_buffer_duration,
-          aagg->priv->output_buffer_duration);
+      gst_audio_aggregator_translate_output_buffer_duration (aagg,
+          g_value_get_uint64 (value));
+      g_object_notify (object, "output-buffer-duration-fraction");
+      gst_audio_aggregator_recalculate_latency (aagg);
       break;
     case PROP_ALIGNMENT_THRESHOLD:
       aagg->priv->alignment_threshold = g_value_get_uint64 (value);
@@ -551,6 +672,14 @@ gst_audio_aggregator_set_property (GObject * object, guint prop_id,
     case PROP_DISCONT_WAIT:
       aagg->priv->discont_wait = g_value_get_uint64 (value);
       break;
+    case PROP_OUTPUT_BUFFER_DURATION_FRACTION:
+      aagg->priv->output_buffer_duration_n =
+          gst_value_get_fraction_numerator (value);
+      aagg->priv->output_buffer_duration_d =
+          gst_value_get_fraction_denominator (value);
+      g_object_notify (object, "output-buffer-duration");
+      gst_audio_aggregator_recalculate_latency (aagg);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -565,7 +694,9 @@ gst_audio_aggregator_get_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_OUTPUT_BUFFER_DURATION:
-      g_value_set_uint64 (value, aagg->priv->output_buffer_duration);
+      g_value_set_uint64 (value, gst_util_uint64_scale_int (GST_SECOND,
+              aagg->priv->output_buffer_duration_n,
+              aagg->priv->output_buffer_duration_d));
       break;
     case PROP_ALIGNMENT_THRESHOLD:
       g_value_set_uint64 (value, aagg->priv->alignment_threshold);
@@ -573,6 +704,10 @@ gst_audio_aggregator_get_property (GObject * object, guint prop_id,
     case PROP_DISCONT_WAIT:
       g_value_set_uint64 (value, aagg->priv->discont_wait);
       break;
+    case PROP_OUTPUT_BUFFER_DURATION_FRACTION:
+      gst_value_set_fraction (value, aagg->priv->output_buffer_duration_n,
+          aagg->priv->output_buffer_duration_d);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -760,7 +895,8 @@ gst_audio_aggregator_sink_setcaps (GstAudioAggregatorPad * aaggpad,
   gboolean downstream_supports_rate = TRUE;
 
   if (!gst_audio_info_from_caps (&info, caps)) {
-    GST_WARNING_OBJECT (agg, "Rejecting invalid caps: %" GST_PTR_FORMAT, caps);
+    GST_WARNING_OBJECT (aaggpad, "Rejecting invalid caps: %" GST_PTR_FORMAT,
+        caps);
     return FALSE;
   }
 
@@ -793,6 +929,10 @@ gst_audio_aggregator_sink_setcaps (GstAudioAggregatorPad * aaggpad,
 
   if (!downstream_supports_rate || (first_configured_pad
           && info.rate != first_configured_pad->info.rate)) {
+    GST_WARNING_OBJECT (aaggpad,
+        "Sample rate %d can't be configured (downstream supported: %d, configured rate: %d)",
+        info.rate, downstream_supports_rate,
+        first_configured_pad ? first_configured_pad->info.rate : 0);
     gst_pad_push_event (GST_PAD (aaggpad), gst_event_new_reconfigure ());
     ret = FALSE;
   } else {
@@ -926,7 +1066,7 @@ gst_audio_aggregator_update_converters (GstAudioAggregator * aagg,
     if (aaggpad->priv->buffer) {
       GstBuffer *new_converted_buffer =
           gst_audio_aggregator_convert_buffer (aagg, GST_PAD (aaggpad),
-          old_info, new_info, aaggpad->priv->input_buffer);
+          old_info, new_info, aaggpad->priv->buffer);
       gst_buffer_replace (&aaggpad->priv->buffer, new_converted_buffer);
       gst_buffer_unref (new_converted_buffer);
     }
@@ -959,13 +1099,16 @@ gst_audio_aggregator_negotiated_src_caps (GstAggregator * agg, GstCaps * caps)
     GST_INFO_OBJECT (aagg, "setting caps to %" GST_PTR_FORMAT, caps);
     gst_caps_replace (&aagg->current_caps, caps);
 
+    if (old_info.rate != info.rate)
+      aagg->priv->offset = -1;
+
     memcpy (&srcpad->info, &info, sizeof (info));
 
     gst_audio_aggregator_update_converters (aagg, &info, &old_info);
 
     if (srcpad_klass->update_conversion_info)
-      srcpad_klass->
-          update_conversion_info (GST_AUDIO_AGGREGATOR_PAD (agg->srcpad));
+      srcpad_klass->update_conversion_info (GST_AUDIO_AGGREGATOR_PAD (agg->
+              srcpad));
 
     if (aagg->priv->current_buffer) {
       GstBuffer *converted;
@@ -976,6 +1119,9 @@ gst_audio_aggregator_negotiated_src_caps (GstAggregator * agg, GstCaps * caps)
       gst_buffer_unref (aagg->priv->current_buffer);
       aagg->priv->current_buffer = converted;
     }
+
+    /* Force recalculating in aggregate */
+    aagg->priv->samples_per_buffer = 0;
   }
 
   GST_OBJECT_UNLOCK (aagg);
@@ -1182,7 +1328,7 @@ gst_audio_aggregator_sink_query (GstAggregator * agg, GstAggregatorPad * aggpad,
  *
  * We don't do synchronized mixing so this really depends on where the
  * streams where punched in and what their relative offsets are against
- * eachother which we can get from the first timestamps we see.
+ * each other which we can get from the first timestamps we see.
  *
  * When we add a new stream (or remove a stream) the duration might
  * also become invalid again and we need to post a new DURATION
@@ -1285,8 +1431,8 @@ gst_audio_aggregator_src_query (GstAggregator * agg, GstQuery * query)
       switch (format) {
         case GST_FORMAT_TIME:
           gst_query_set_position (query, format,
-              gst_segment_to_stream_time (&GST_AGGREGATOR_PAD (agg->
-                      srcpad)->segment, GST_FORMAT_TIME,
+              gst_segment_to_stream_time (&GST_AGGREGATOR_PAD (agg->srcpad)->
+                  segment, GST_FORMAT_TIME,
                   GST_AGGREGATOR_PAD (agg->srcpad)->segment.position));
           res = TRUE;
           break;
@@ -1352,6 +1498,7 @@ gst_audio_aggregator_reset (GstAudioAggregator * aagg)
   gst_audio_info_init (&GST_AUDIO_AGGREGATOR_PAD (agg->srcpad)->info);
   gst_caps_replace (&aagg->current_caps, NULL);
   gst_buffer_replace (&aagg->priv->current_buffer, NULL);
+  aagg->priv->accumulated_error = 0;
   GST_OBJECT_UNLOCK (aagg);
   GST_AUDIO_AGGREGATOR_UNLOCK (aagg);
 }
@@ -1385,6 +1532,7 @@ gst_audio_aggregator_flush (GstAggregator * agg)
   GST_OBJECT_LOCK (aagg);
   GST_AGGREGATOR_PAD (agg->srcpad)->segment.position = -1;
   aagg->priv->offset = -1;
+  aagg->priv->accumulated_error = 0;
   gst_buffer_replace (&aagg->priv->current_buffer, NULL);
   GST_OBJECT_UNLOCK (aagg);
   GST_AUDIO_AGGREGATOR_UNLOCK (aagg);
@@ -1661,7 +1809,6 @@ gst_audio_aggregator_mix_buffer (GstAudioAggregator * aagg,
     pad->priv->position = pad->priv->size;
 
     gst_buffer_replace (&pad->priv->buffer, NULL);
-    gst_buffer_replace (&pad->priv->input_buffer, NULL);
     return FALSE;
   }
 
@@ -1691,7 +1838,6 @@ gst_audio_aggregator_mix_buffer (GstAudioAggregator * aagg,
   if (pad->priv->position == pad->priv->size) {
     /* Buffer done, drop it */
     gst_buffer_replace (&pad->priv->buffer, NULL);
-    gst_buffer_replace (&pad->priv->input_buffer, NULL);
     GST_LOG_OBJECT (pad, "Finished mixing buffer, waiting for next");
     return FALSE;
   }
@@ -1752,6 +1898,33 @@ sync_pad_values (GstElement * aagg, GstPad * pad, gpointer user_data)
   return TRUE;
 }
 
+static GstSample *
+gst_audio_aggregator_peek_next_sample (GstAggregator * agg,
+    GstAggregatorPad * aggpad)
+{
+  GstAudioAggregator *aagg = GST_AUDIO_AGGREGATOR (agg);
+  GstAudioAggregatorPad *pad = GST_AUDIO_AGGREGATOR_PAD (aggpad);
+  GstSample *sample = NULL;
+
+  if (pad->priv->buffer && pad->priv->output_offset >= aagg->priv->offset
+      && pad->priv->output_offset <
+      aagg->priv->offset + aagg->priv->samples_per_buffer) {
+    GstCaps *caps = gst_pad_get_current_caps (GST_PAD (aggpad));
+    GstStructure *info =
+        gst_structure_new ("GstAudioAggregatorPadNextSampleInfo",
+        "output-offset", G_TYPE_UINT64, pad->priv->output_offset,
+        "position", G_TYPE_UINT, pad->priv->position,
+        "size", G_TYPE_UINT, pad->priv->size,
+        NULL);
+
+    sample = gst_sample_new (pad->priv->buffer, caps, &aggpad->segment, info);
+    gst_caps_unref (caps);
+    gst_structure_free (info);
+  }
+
+  return sample;
+}
+
 static GstFlowReturn
 gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
 {
@@ -1769,7 +1942,7 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
    *        the running time.
    *
    * 2) If the current pad's offset/offset_end overlaps with the output
-   *    offset/offset_end, mix it at the appropiate position in the output
+   *    offset/offset_end, mix it at the appropriate position in the output
    *    buffer and advance the pad's position. Remember if this pad needs
    *    a new buffer to advance behind the output offset_end.
    *
@@ -1803,6 +1976,15 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
   GST_AUDIO_AGGREGATOR_LOCK (aagg);
   GST_OBJECT_LOCK (agg);
 
+  if (aagg->priv->samples_per_buffer == 0) {
+    if (!gst_audio_aggregator_update_samples_per_buffer (aagg)) {
+      GST_ERROR_OBJECT (aagg,
+          "Failed to calculate the number of samples per buffer");
+      GST_OBJECT_UNLOCK (agg);
+      goto not_negotiated;
+    }
+  }
+
   /* Update position from the segment start/stop if needed */
   if (agg_segment->position == -1) {
     if (agg_segment->rate > 0.0)
@@ -1811,16 +1993,31 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
       agg_segment->position = agg_segment->stop;
   }
 
+  rate = GST_AUDIO_INFO_RATE (&srcpad->info);
+  bpf = GST_AUDIO_INFO_BPF (&srcpad->info);
+
   if (G_UNLIKELY (srcpad->info.finfo->format == GST_AUDIO_FORMAT_UNKNOWN)) {
     if (timeout) {
+      GstClockTime output_buffer_duration;
       GST_DEBUG_OBJECT (aagg,
           "Got timeout before receiving any caps, don't output anything");
 
+      blocksize = aagg->priv->samples_per_buffer;
+      if (aagg->priv->error_per_buffer + aagg->priv->accumulated_error >=
+          aagg->priv->output_buffer_duration_d)
+        blocksize += 1;
+      aagg->priv->accumulated_error =
+          (aagg->priv->accumulated_error +
+          aagg->priv->error_per_buffer) % aagg->priv->output_buffer_duration_d;
+
+      output_buffer_duration =
+          gst_util_uint64_scale (blocksize, GST_SECOND, rate);
+
       /* Advance position */
       if (agg_segment->rate > 0.0)
-        agg_segment->position += aagg->priv->output_buffer_duration;
-      else if (agg_segment->position > aagg->priv->output_buffer_duration)
-        agg_segment->position -= aagg->priv->output_buffer_duration;
+        agg_segment->position += output_buffer_duration;
+      else if (agg_segment->position > output_buffer_duration)
+        agg_segment->position -= output_buffer_duration;
       else
         agg_segment->position = 0;
 
@@ -1833,9 +2030,6 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
     }
   }
 
-  rate = GST_AUDIO_INFO_RATE (&srcpad->info);
-  bpf = GST_AUDIO_INFO_BPF (&srcpad->info);
-
   if (aagg->priv->offset == -1) {
     aagg->priv->offset =
         gst_util_uint64_scale (agg_segment->position - agg_segment->start, rate,
@@ -1844,9 +2038,29 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
         aagg->priv->offset);
   }
 
-  blocksize = gst_util_uint64_scale (aagg->priv->output_buffer_duration,
-      rate, GST_SECOND);
-  blocksize = MAX (1, blocksize);
+  if (aagg->priv->current_buffer == NULL) {
+    blocksize = aagg->priv->samples_per_buffer;
+
+    if (aagg->priv->error_per_buffer + aagg->priv->accumulated_error >=
+        aagg->priv->output_buffer_duration_d)
+      blocksize += 1;
+
+    aagg->priv->current_blocksize = blocksize;
+
+    aagg->priv->accumulated_error =
+        (aagg->priv->accumulated_error +
+        aagg->priv->error_per_buffer) % aagg->priv->output_buffer_duration_d;
+
+    GST_OBJECT_UNLOCK (agg);
+    aagg->priv->current_buffer =
+        GST_AUDIO_AGGREGATOR_GET_CLASS (aagg)->create_output_buffer (aagg,
+        blocksize);
+    /* Be careful, some things could have changed ? */
+    GST_OBJECT_LOCK (agg);
+    GST_BUFFER_FLAG_SET (aagg->priv->current_buffer, GST_BUFFER_FLAG_GAP);
+  } else {
+    blocksize = aagg->priv->current_blocksize;
+  }
 
   /* FIXME: Reverse mixing does not work at all yet */
   if (agg_segment->rate > 0.0) {
@@ -1860,15 +2074,6 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
       agg_segment->start + gst_util_uint64_scale (next_offset, GST_SECOND,
       rate);
 
-  if (aagg->priv->current_buffer == NULL) {
-    GST_OBJECT_UNLOCK (agg);
-    aagg->priv->current_buffer =
-        GST_AUDIO_AGGREGATOR_GET_CLASS (aagg)->create_output_buffer (aagg,
-        blocksize);
-    /* Be careful, some things could have changed ? */
-    GST_OBJECT_LOCK (agg);
-    GST_BUFFER_FLAG_SET (aagg->priv->current_buffer, GST_BUFFER_FLAG_GAP);
-  }
   outbuf = aagg->priv->current_buffer;
 
   GST_LOG_OBJECT (agg,
@@ -1880,14 +2085,15 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
     GstAudioAggregatorPad *pad = (GstAudioAggregatorPad *) iter->data;
     GstAggregatorPad *aggpad = (GstAggregatorPad *) iter->data;
     gboolean pad_eos = gst_aggregator_pad_is_eos (aggpad);
+    GstBuffer *input_buffer;
 
     if (!pad_eos)
       is_eos = FALSE;
 
-    pad->priv->input_buffer = gst_aggregator_pad_peek_buffer (aggpad);
+    input_buffer = gst_aggregator_pad_peek_buffer (aggpad);
 
     GST_OBJECT_LOCK (pad);
-    if (!pad->priv->input_buffer) {
+    if (!input_buffer) {
       if (timeout) {
         if (pad->priv->output_offset < next_offset) {
           gint64 diff = next_offset - pad->priv->output_offset;
@@ -1912,24 +2118,21 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
       if (GST_AUDIO_AGGREGATOR_PAD_GET_CLASS (pad)->convert_buffer)
         pad->priv->buffer =
             gst_audio_aggregator_convert_buffer
-            (aagg, GST_PAD (pad), &pad->info, &srcpad->info,
-            pad->priv->input_buffer);
+            (aagg, GST_PAD (pad), &pad->info, &srcpad->info, input_buffer);
       else
-        pad->priv->buffer = gst_buffer_ref (pad->priv->input_buffer);
+        pad->priv->buffer = gst_buffer_ref (input_buffer);
 
       if (!gst_audio_aggregator_fill_buffer (aagg, pad)) {
         gst_buffer_replace (&pad->priv->buffer, NULL);
-        gst_buffer_replace (&pad->priv->input_buffer, NULL);
-        pad->priv->buffer = NULL;
+        gst_buffer_unref (input_buffer);
         dropped = TRUE;
         GST_OBJECT_UNLOCK (pad);
 
         gst_aggregator_pad_drop_buffer (aggpad);
         continue;
       }
-    } else {
-      gst_buffer_unref (pad->priv->input_buffer);
     }
+    gst_buffer_unref (input_buffer);
 
     if (!pad->priv->buffer && !dropped && pad_eos) {
       GST_DEBUG_OBJECT (aggpad, "Pad is in EOS state");
@@ -1957,7 +2160,6 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
                     GST_AUDIO_INFO_RATE (&srcpad->info))), pad->priv->buffer);
         /* Buffer done, drop it */
         gst_buffer_replace (&pad->priv->buffer, NULL);
-        gst_buffer_replace (&pad->priv->input_buffer, NULL);
         dropped = TRUE;
         GST_OBJECT_UNLOCK (pad);
         gst_aggregator_pad_drop_buffer (aggpad);
@@ -1966,8 +2168,27 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
     }
 
     g_assert (pad->priv->buffer);
+    GST_OBJECT_UNLOCK (pad);
+  }
+  GST_OBJECT_UNLOCK (agg);
 
-    if (pad->priv->output_offset >= aagg->priv->offset
+  {
+    gst_structure_set (aagg->priv->selected_samples_info, "offset",
+        G_TYPE_UINT64, aagg->priv->offset, "frames", G_TYPE_UINT, blocksize,
+        NULL);
+    gst_aggregator_selected_samples (agg, agg_segment->position,
+        GST_CLOCK_TIME_NONE, next_timestamp - agg_segment->position,
+        aagg->priv->selected_samples_info);
+  }
+
+  GST_OBJECT_LOCK (agg);
+  for (iter = element->sinkpads; iter; iter = iter->next) {
+    GstAudioAggregatorPad *pad = (GstAudioAggregatorPad *) iter->data;
+    GstAggregatorPad *aggpad = (GstAggregatorPad *) iter->data;
+
+    GST_OBJECT_LOCK (pad);
+
+    if (pad->priv->buffer && pad->priv->output_offset >= aagg->priv->offset
         && pad->priv->output_offset < aagg->priv->offset + blocksize) {
       gboolean drop_buf;
 
