@@ -1284,7 +1284,7 @@ gst_audio_base_sink_custom_slaving (GstAudioBaseSink * sink,
   etime = etime > cexternal ? etime - cexternal : 0;
   itime = itime > cinternal ? itime - cinternal : 0;
 
-  /* don't do any skewing unless the callback explicitely requests one */
+  /* don't do any skewing unless the callback explicitly requests one */
   requested_skew = 0;
 
   if (sink->priv->custom_slaving_callback != NULL) {
@@ -1358,7 +1358,7 @@ gst_audio_base_sink_resample_slaving (GstAudioBaseSink * sink,
   /* FIXME, we can sample and add observations here or use the timeouts on the
    * clock. No idea which one is better or more stable. The timeout seems more
    * arbitrary but this one seems more demanding and does not work when there is
-   * no data comming in to the sink. */
+   * no data coming in to the sink. */
 #if 0
   GstClockTime etime, itime;
   gdouble r_squared;
@@ -1640,7 +1640,7 @@ gst_audio_base_sink_sync_latency (GstBaseSink * bsink, GstMiniObject * obj)
       goto flushing;
 
     /* retry if we got unscheduled, which means we did not reach the timeout
-     * yet. if some other error occures, we continue. */
+     * yet. if some other error occurs, we continue. */
   } while (status == GST_CLOCK_UNSCHEDULED);
 
   GST_DEBUG_OBJECT (sink, "latency synced");
@@ -1989,6 +1989,9 @@ gst_audio_base_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   render_stop =
       gst_segment_to_running_time (&bsink->segment, GST_FORMAT_TIME, stop);
 
+  if (render_start == GST_CLOCK_TIME_NONE || render_stop == GST_CLOCK_TIME_NONE)
+    goto too_late;
+
   GST_DEBUG_OBJECT (sink,
       "running: start %" GST_TIME_FORMAT " - stop %" GST_TIME_FORMAT,
       GST_TIME_ARGS (render_start), GST_TIME_ARGS (render_stop));
@@ -2421,7 +2424,7 @@ gst_audio_base_sink_change_state (GstElement * element,
       gst_audio_ring_buffer_may_start (sink->ringbuffer, FALSE);
 
       /* Only post clock-provide messages if this is the clock that
-       * we've created. If the subclass has overriden it the subclass
+       * we've created. If the subclass has overridden it the subclass
        * should post this messages whenever necessary */
       if (gst_audio_base_sink_is_self_provided_clock (sink))
         gst_element_post_message (element,
@@ -2441,7 +2444,7 @@ gst_audio_base_sink_change_state (GstElement * element,
       gst_audio_ring_buffer_may_start (sink->ringbuffer, TRUE);
       if (GST_BASE_SINK_CAST (sink)->pad_mode == GST_PAD_MODE_PULL ||
           g_atomic_int_get (&sink->eos_rendering) || eos) {
-        /* we always start the ringbuffer in pull mode immediatly */
+        /* we always start the ringbuffer in pull mode immediately */
         /* sync rendering on eos needs running clock,
          * and others need running clock when finished rendering eos */
         gst_audio_ring_buffer_start (sink->ringbuffer);
@@ -2459,7 +2462,7 @@ gst_audio_base_sink_change_state (GstElement * element,
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       /* Only post clock-lost messages if this is the clock that
-       * we've created. If the subclass has overriden it the subclass
+       * we've created. If the subclass has overridden it the subclass
        * should post this messages whenever necessary */
       if (gst_audio_base_sink_is_self_provided_clock (sink))
         gst_element_post_message (element,
