@@ -45,9 +45,6 @@
 GST_DEBUG_CATEGORY_STATIC (gst_audio_base_src_debug);
 #define GST_CAT_DEFAULT gst_audio_base_src_debug
 
-#define GST_AUDIO_BASE_SRC_GET_PRIVATE(obj)  \
-   (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_AUDIO_BASE_SRC, GstAudioBaseSrcPrivate))
-
 struct _GstAudioBaseSrcPrivate
 {
   /* the clock slaving algorithm in use */
@@ -97,6 +94,7 @@ _do_init (GType type)
 
 #define gst_audio_base_src_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstAudioBaseSrc, gst_audio_base_src, GST_TYPE_PUSH_SRC,
+    G_ADD_PRIVATE (GstAudioBaseSrc)
     _do_init (g_define_type_id));
 
 static void gst_audio_base_src_set_property (GObject * object, guint prop_id,
@@ -136,8 +134,6 @@ gst_audio_base_src_class_init (GstAudioBaseSrcClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasesrc_class = (GstBaseSrcClass *) klass;
 
-  g_type_class_add_private (klass, sizeof (GstAudioBaseSrcPrivate));
-
   gobject_class->set_property = gst_audio_base_src_set_property;
   gobject_class->get_property = gst_audio_base_src_get_property;
   gobject_class->dispose = gst_audio_base_src_dispose;
@@ -217,7 +213,7 @@ gst_audio_base_src_class_init (GstAudioBaseSrcClass * klass)
 static void
 gst_audio_base_src_init (GstAudioBaseSrc * audiobasesrc)
 {
-  audiobasesrc->priv = GST_AUDIO_BASE_SRC_GET_PRIVATE (audiobasesrc);
+  audiobasesrc->priv = gst_audio_base_src_get_instance_private (audiobasesrc);
 
   audiobasesrc->buffer_time = DEFAULT_BUFFER_TIME;
   audiobasesrc->latency_time = DEFAULT_LATENCY_TIME;
@@ -1138,7 +1134,7 @@ gst_audio_base_src_change_state (GstElement * element,
       gst_audio_ring_buffer_set_flushing (src->ringbuffer, FALSE);
       gst_audio_ring_buffer_may_start (src->ringbuffer, FALSE);
       /* Only post clock-provide messages if this is the clock that
-       * we've created. If the subclass has overriden it the subclass
+       * we've created. If the subclass has overridden it the subclass
        * should post this messages whenever necessary */
       if (src->clock && GST_IS_AUDIO_CLOCK (src->clock) &&
           GST_AUDIO_CLOCK_CAST (src->clock)->func ==
@@ -1159,7 +1155,7 @@ gst_audio_base_src_change_state (GstElement * element,
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       GST_DEBUG_OBJECT (src, "PAUSED->READY");
       /* Only post clock-lost messages if this is the clock that
-       * we've created. If the subclass has overriden it the subclass
+       * we've created. If the subclass has overridden it the subclass
        * should post this messages whenever necessary */
       if (src->clock && GST_IS_AUDIO_CLOCK (src->clock) &&
           GST_AUDIO_CLOCK_CAST (src->clock)->func ==
@@ -1215,7 +1211,7 @@ gst_audio_base_src_post_message (GstElement * element, GstMessage * message)
   GstAudioBaseSrc *src = GST_AUDIO_BASE_SRC (element);
   gboolean ret;
 
-  if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_ERROR) {
+  if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_ERROR && src->ringbuffer) {
     GstAudioRingBuffer *ringbuffer;
 
     GST_INFO_OBJECT (element, "subclass posted error");
