@@ -16,60 +16,40 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
  * SECTION:gstaudiosrc
+ * @title: GstAudioSrc
  * @short_description: Simple base class for audio sources
  * @see_also: #GstAudioBaseSrc, #GstAudioRingBuffer, #GstAudioSrc.
  *
  * This is the most simple base class for audio sources that only requires
  * subclasses to implement a set of simple functions:
  *
- * <variablelist>
- *   <varlistentry>
- *     <term>open()</term>
- *     <listitem><para>Open the device.</para></listitem>
- *   </varlistentry>
- *   <varlistentry>
- *     <term>prepare()</term>
- *     <listitem><para>Configure the device with the specified format.</para></listitem>
- *   </varlistentry>
- *   <varlistentry>
- *     <term>read()</term>
- *     <listitem><para>Read samples from the device.</para></listitem>
- *   </varlistentry>
- *   <varlistentry>
- *     <term>reset()</term>
- *     <listitem><para>Unblock reads and flush the device.</para></listitem>
- *   </varlistentry>
- *   <varlistentry>
- *     <term>delay()</term>
- *     <listitem><para>Get the number of samples in the device but not yet read.
- *     </para></listitem>
- *   </varlistentry>
- *   <varlistentry>
- *     <term>unprepare()</term>
- *     <listitem><para>Undo operations done by prepare.</para></listitem>
- *   </varlistentry>
- *   <varlistentry>
- *     <term>close()</term>
- *     <listitem><para>Close the device.</para></listitem>
- *   </varlistentry>
- * </variablelist>
+ * * `open()` :Open the device.
+ * * `prepare()` :Configure the device with the specified format.
+ * * `read()` :Read samples from the device.
+ * * `reset()` :Unblock reads and flush the device.
+ * * `delay()` :Get the number of samples in the device but not yet read.
+ * * `unprepare()` :Undo operations done by prepare.
+ * * `close()` :Close the device.
  *
  * All scheduling of samples and timestamps is done in this base class
  * together with #GstAudioBaseSrc using a default implementation of a
  * #GstAudioRingBuffer that uses threads.
- *
- * Last reviewed on 2006-09-27 (0.10.12)
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <string.h>
 
+#include <gst/audio/audio.h>
 #include "gstaudiosrc.h"
+#include "gstaudioutilsprivate.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_audio_src_debug);
 #define GST_CAT_DEFAULT gst_audio_src_debug
@@ -195,7 +175,7 @@ typedef guint (*ReadFunc)
 
 /* this internal thread does nothing else but read samples from the audio device.
  * It will read each segment in the ringbuffer and will update the play
- * pointer. 
+ * pointer.
  * The start/stop methods control the thread.
  */
 static void
@@ -207,6 +187,7 @@ audioringbuffer_thread_func (GstAudioRingBuffer * buf)
   ReadFunc readfunc;
   GstMessage *message;
   GValue val = { 0 };
+  gpointer handle;
 
   src = GST_AUDIO_SRC (GST_OBJECT_PARENT (buf));
   csrc = GST_AUDIO_SRC_GET_CLASS (src);
@@ -216,12 +197,15 @@ audioringbuffer_thread_func (GstAudioRingBuffer * buf)
   if ((readfunc = csrc->read) == NULL)
     goto no_function;
 
-  /* FIXME: maybe we should at least use a custom pointer type here? */
-  g_value_init (&val, G_TYPE_POINTER);
-  g_value_set_pointer (&val, src->thread);
+  if (G_UNLIKELY (!__gst_audio_set_thread_priority (&handle)))
+    GST_WARNING_OBJECT (src, "failed to set thread priority");
+
   message = gst_message_new_stream_status (GST_OBJECT_CAST (buf),
       GST_STREAM_STATUS_TYPE_ENTER, GST_ELEMENT_CAST (src));
+  g_value_init (&val, GST_TYPE_G_THREAD);
+  g_value_set_boxed (&val, g_thread_self ());
   gst_message_set_stream_status_object (message, &val);
+  g_value_unset (&val);
   GST_DEBUG_OBJECT (src, "posting ENTER stream status");
   gst_element_post_message (GST_ELEMENT_CAST (src), message);
 
@@ -237,7 +221,7 @@ audioringbuffer_thread_func (GstAudioRingBuffer * buf)
       left = len;
       do {
         read = readfunc (src, readptr, left, &timestamp);
-        GST_LOG_OBJECT (src, "transfered %d bytes of %d to segment %d", read,
+        GST_LOG_OBJECT (src, "transferred %d bytes of %d to segment %d", read,
             left, readseg);
         if (read < 0 || read > left) {
           GST_WARNING_OBJECT (src,
@@ -247,7 +231,8 @@ audioringbuffer_thread_func (GstAudioRingBuffer * buf)
         }
         left -= read;
         readptr += read;
-      } while (left > 0);
+
+      } while (left > 0 && g_atomic_int_get (&abuf->running));
 
       /* Update timestamp on buffer if required */
       gst_audio_ring_buffer_set_timestamp (buf, readseg, timestamp);
@@ -291,9 +276,15 @@ stop_running:
     GST_DEBUG ("stop running, exit thread");
     message = gst_message_new_stream_status (GST_OBJECT_CAST (buf),
         GST_STREAM_STATUS_TYPE_LEAVE, GST_ELEMENT_CAST (src));
+    g_value_init (&val, GST_TYPE_G_THREAD);
+    g_value_set_boxed (&val, g_thread_self ());
     gst_message_set_stream_status_object (message, &val);
+    g_value_unset (&val);
     GST_DEBUG_OBJECT (src, "posting LEAVE stream status");
     gst_element_post_message (GST_ELEMENT_CAST (src), message);
+
+    if (G_UNLIKELY (!__gst_audio_restore_thread_priority (handle)))
+      GST_WARNING_OBJECT (src, "failed to restore thread priority");
     return;
   }
 }
@@ -391,7 +382,14 @@ gst_audio_src_ring_buffer_acquire (GstAudioRingBuffer * buf,
     goto could_not_open;
 
   buf->size = spec->segtotal * spec->segsize;
-  buf->memory = g_malloc0 (buf->size);
+  buf->memory = g_malloc (buf->size);
+  if (buf->spec.type == GST_AUDIO_RING_BUFFER_FORMAT_TYPE_RAW) {
+    gst_audio_format_fill_silence (buf->spec.info.finfo, buf->memory,
+        buf->size);
+  } else {
+    /* FIXME, non-raw formats get 0 as the empty sample */
+    memset (buf->memory, 0, buf->size);
+  }
 
   abuf = GST_AUDIO_SRC_RING_BUFFER (buf);
   abuf->running = TRUE;
@@ -469,7 +467,7 @@ gst_audio_src_ring_buffer_stop (GstAudioRingBuffer * buf)
 #if 0
   GST_DEBUG ("stop, waiting...");
   GST_AUDIO_SRC_RING_BUFFER_WAIT (buf);
-  GST_DEBUG ("stoped");
+  GST_DEBUG ("stopped");
 #endif
 
   return TRUE;
