@@ -153,7 +153,7 @@ gst_gl_window_dispmanx_egl_open (GstGLWindow * window, GError ** error)
         GST_GL_WINDOW_ERROR_RESOURCE_UNAVAILABLE, "Can't open display");
     return FALSE;
   }
-  GST_DEBUG ("Got display size: %dx%d\n", window_egl->dp_width,
+  GST_DEBUG ("Got display size: %dx%d", window_egl->dp_width,
       window_egl->dp_height);
 
   window_egl->native.element = 0;
@@ -192,7 +192,8 @@ gst_gl_window_dispmanx_egl_set_window_handle (GstGLWindow * window,
   GST_DEBUG_OBJECT (window, "set window handle with size %dx%d",
       foreign_window->width, foreign_window->height);
 
-  if (window_egl->native.element && window_egl->native.element != window_egl->foreign.element) {
+  if (window_egl->native.element
+      && window_egl->native.element != window_egl->foreign.element) {
     dispman_update = vc_dispmanx_update_start (0);
     vc_dispmanx_element_remove (dispman_update, window_egl->native.element);
     vc_dispmanx_update_submit_sync (dispman_update);
