@@ -35,8 +35,8 @@
  *
  * #GstGLBaseFilter handles the nitty gritty details of retrieving an OpenGL
  * context.  It also provided some wrappers around #GstBaseTransform's
- * start(), stop() and set_caps() virtual methods that ensure an OpenGL context
- * is available and current in the calling thread.
+ * `start()`, `stop()` and `set_caps()` virtual methods that ensure an OpenGL
+ * context is available and current in the calling thread.
  */
 
 #define GST_CAT_DEFAULT gst_gl_base_filter_debug
@@ -48,6 +48,9 @@ struct _GstGLBaseFilterPrivate
 
   gboolean gl_result;
   gboolean gl_started;
+
+  GRecMutex context_lock;
+  gboolean new_gl_context;
 };
 
 /* Properties */
@@ -91,6 +94,8 @@ static void gst_gl_base_filter_gl_stop (GstGLContext * context, gpointer data);
 static gboolean gst_gl_base_filter_default_gl_start (GstGLBaseFilter * filter);
 static void gst_gl_base_filter_default_gl_stop (GstGLBaseFilter * filter);
 
+static gboolean gst_gl_base_filter_find_gl_context_unlocked (GstGLBaseFilter *
+    filter);
 static void
 gst_gl_base_filter_class_init (GstGLBaseFilterClass * klass)
 {
@@ -131,6 +136,8 @@ gst_gl_base_filter_init (GstGLBaseFilter * filter)
   gst_base_transform_set_qos_enabled (GST_BASE_TRANSFORM (filter), TRUE);
 
   filter->priv = gst_gl_base_filter_get_instance_private (filter);
+
+  g_rec_mutex_init (&filter->priv->context_lock);
 }
 
 static void
@@ -141,6 +148,8 @@ gst_gl_base_filter_finalize (GObject * object)
   gst_caps_replace (&filter->in_caps, NULL);
   gst_caps_replace (&filter->out_caps, NULL);
 
+  g_rec_mutex_clear (&filter->priv->context_lock);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -171,33 +180,83 @@ gst_gl_base_filter_get_property (GObject * object, guint prop_id,
   }
 }
 
-static void
-gst_gl_base_filter_set_context (GstElement * element, GstContext * context)
+static gboolean
+_find_local_gl_context_unlocked (GstGLBaseFilter * filter)
 {
-  GstGLBaseFilter *filter = GST_GL_BASE_FILTER (element);
-  GstGLBaseFilterClass *filter_class = GST_GL_BASE_FILTER_GET_CLASS (filter);
+  GstGLContext *context, *prev_context;
+  gboolean ret;
 
-  GST_OBJECT_LOCK (filter);
-  gst_gl_handle_set_context (element, context, &filter->display,
-      &filter->priv->other_context);
-  if (filter->display)
-    gst_gl_display_filter_gl_api (filter->display,
-        filter_class->supported_gl_api);
-  GST_OBJECT_UNLOCK (filter);
+  if (filter->context && filter->context->display == filter->display)
+    return TRUE;
 
-  GST_ELEMENT_CLASS (parent_class)->set_context (element, context);
+  context = prev_context = filter->context;
+  g_rec_mutex_unlock (&filter->priv->context_lock);
+  /* we need to drop the lock to query as another element may also be
+   * performing a context query on us which would also attempt to take the
+   * context_lock. Our query could block on the same lock in the other element.
+   */
+  ret =
+      gst_gl_query_local_gl_context (GST_ELEMENT (filter), GST_PAD_SRC,
+      &context);
+  g_rec_mutex_lock (&filter->priv->context_lock);
+  if (ret) {
+    if (filter->context != prev_context) {
+      /* we need to recheck everything since we dropped the lock and the
+       * context has changed */
+      if (filter->context && filter->context->display == filter->display) {
+        if (context != filter->context)
+          gst_clear_object (&context);
+        return TRUE;
+      }
+    }
+
+    if (context->display == filter->display) {
+      filter->context = context;
+      return TRUE;
+    }
+    if (context != filter->context)
+      gst_clear_object (&context);
+  }
+
+  context = prev_context = filter->context;
+  g_rec_mutex_unlock (&filter->priv->context_lock);
+  /* we need to drop the lock to query as another element may also be
+   * performing a context query on us which would also attempt to take the
+   * context_lock. Our query could block on the same lock in the other element.
+   */
+  ret =
+      gst_gl_query_local_gl_context (GST_ELEMENT (filter), GST_PAD_SINK,
+      &context);
+  g_rec_mutex_lock (&filter->priv->context_lock);
+  if (ret) {
+    if (filter->context != prev_context) {
+      /* we need to recheck everything now that we dropped the lock */
+      if (filter->context && filter->context->display == filter->display) {
+        if (context != filter->context)
+          gst_clear_object (&context);
+        return TRUE;
+      }
+    }
+
+    if (context->display == filter->display) {
+      filter->context = context;
+      return TRUE;
+    }
+    if (context != filter->context)
+      gst_clear_object (&context);
+  }
+
+  return FALSE;
 }
 
 static gboolean
 _find_local_gl_context (GstGLBaseFilter * filter)
 {
-  if (gst_gl_query_local_gl_context (GST_ELEMENT (filter), GST_PAD_SRC,
-          &filter->context))
-    return TRUE;
-  if (gst_gl_query_local_gl_context (GST_ELEMENT (filter), GST_PAD_SINK,
-          &filter->context))
-    return TRUE;
-  return FALSE;
+  gboolean ret;
+  g_rec_mutex_lock (&filter->priv->context_lock);
+  ret = _find_local_gl_context_unlocked (filter);
+  g_rec_mutex_unlock (&filter->priv->context_lock);
+  return ret;
 }
 
 static gboolean
@@ -219,11 +278,26 @@ gst_gl_base_filter_query (GstBaseTransform * trans, GstPadDirection direction,
     }
     case GST_QUERY_CONTEXT:
     {
+      GstGLDisplay *display = NULL;
+      GstGLContext *other = NULL, *local = NULL;
       gboolean ret;
-      GST_OBJECT_LOCK (filter);
+
+      g_rec_mutex_lock (&filter->priv->context_lock);
+      if (filter->display)
+        display = gst_object_ref (filter->display);
+      if (filter->context)
+        local = gst_object_ref (filter->context);
+      if (filter->priv->other_context)
+        other = gst_object_ref (filter->priv->other_context);
+      g_rec_mutex_unlock (&filter->priv->context_lock);
+
       ret = gst_gl_handle_context_query ((GstElement *) filter, query,
-          filter->display, filter->context, filter->priv->other_context);
-      GST_OBJECT_UNLOCK (filter);
+          display, local, other);
+
+      gst_clear_object (&display);
+      gst_clear_object (&other);
+      gst_clear_object (&local);
+
       if (ret)
         return TRUE;
       break;
@@ -239,17 +313,14 @@ gst_gl_base_filter_query (GstBaseTransform * trans, GstPadDirection direction,
 static void
 gst_gl_base_filter_reset (GstGLBaseFilter * filter)
 {
-  GstGLBaseFilterClass *filter_class = GST_GL_BASE_FILTER_GET_CLASS (filter);
-
+  g_rec_mutex_lock (&filter->priv->context_lock);
   if (filter->context) {
-    if (filter_class->gl_stop != NULL) {
-      gst_gl_context_thread_add (filter->context, gst_gl_base_filter_gl_stop,
-          filter);
-    }
-
+    gst_gl_context_thread_add (filter->context, gst_gl_base_filter_gl_stop,
+        filter);
     gst_object_unref (filter->context);
     filter->context = NULL;
   }
+  g_rec_mutex_unlock (&filter->priv->context_lock);
 }
 
 static gboolean
@@ -280,6 +351,7 @@ gst_gl_base_filter_gl_start (GstGLContext * context, gpointer data)
   GstGLBaseFilter *filter = GST_GL_BASE_FILTER (data);
   GstGLBaseFilterClass *filter_class = GST_GL_BASE_FILTER_GET_CLASS (filter);
 
+  GST_INFO_OBJECT (filter, "starting");
   gst_gl_insert_debug_marker (filter->context,
       "starting element %s", GST_OBJECT_NAME (filter));
 
@@ -297,6 +369,7 @@ gst_gl_base_filter_gl_stop (GstGLContext * context, gpointer data)
   GstGLBaseFilter *filter = GST_GL_BASE_FILTER (data);
   GstGLBaseFilterClass *filter_class = GST_GL_BASE_FILTER_GET_CLASS (filter);
 
+  GST_INFO_OBJECT (filter, "stopping");
   gst_gl_insert_debug_marker (filter->context,
       "stopping element %s", GST_OBJECT_NAME (filter));
 
@@ -311,33 +384,53 @@ _gl_set_caps (GstGLContext * context, GstGLBaseFilter * filter)
 {
   GstGLBaseFilterClass *filter_class = GST_GL_BASE_FILTER_GET_CLASS (filter);
 
+  GST_INFO_OBJECT (filter, "set GL caps input %" GST_PTR_FORMAT,
+      filter->in_caps);
+  GST_INFO_OBJECT (filter, "set GL caps output %" GST_PTR_FORMAT,
+      filter->out_caps);
+
   if (filter_class->gl_set_caps)
     filter->priv->gl_result =
         filter_class->gl_set_caps (filter, filter->in_caps, filter->out_caps);
 }
 
+static gboolean
+gl_set_caps_unlocked (GstGLBaseFilter * filter)
+{
+  GstGLBaseFilterClass *filter_class = GST_GL_BASE_FILTER_GET_CLASS (filter);
+
+  if (filter_class->gl_set_caps) {
+    gst_gl_context_thread_add (filter->context,
+        (GstGLContextThreadFunc) _gl_set_caps, filter);
+    return filter->priv->gl_result;
+  }
+
+  return TRUE;
+}
+
 static gboolean
 gst_gl_base_filter_decide_allocation (GstBaseTransform * trans,
     GstQuery * query)
 {
   GstGLBaseFilter *filter = GST_GL_BASE_FILTER (trans);
-  GstGLBaseFilterClass *filter_class = GST_GL_BASE_FILTER_GET_CLASS (filter);
 
-  if (!gst_gl_base_filter_find_gl_context (filter))
+  g_rec_mutex_lock (&filter->priv->context_lock);
+  if (!gst_gl_base_filter_find_gl_context_unlocked (filter)) {
+    g_rec_mutex_unlock (&filter->priv->context_lock);
     return FALSE;
-
-  if (filter_class->gl_set_caps) {
-    gst_gl_context_thread_add (filter->context,
-        (GstGLContextThreadFunc) _gl_set_caps, filter);
-    if (!filter->priv->gl_result)
-      goto error;
   }
 
+  if (!gl_set_caps_unlocked (filter))
+    goto error;
+
+  g_rec_mutex_unlock (&filter->priv->context_lock);
+
   return GST_BASE_TRANSFORM_CLASS (parent_class)->decide_allocation (trans,
       query);
 
 error:
   {
+    g_rec_mutex_unlock (&filter->priv->context_lock);
     GST_ELEMENT_ERROR (trans, LIBRARY, INIT,
         ("Subclass failed to initialize."), (NULL));
     return FALSE;
@@ -371,10 +464,8 @@ gst_gl_base_filter_change_state (GstElement * element,
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
       if (!gst_gl_ensure_element_data (element, &filter->display,
-              &filter->priv->other_context)) {
-        GST_OBJECT_UNLOCK (filter);
+              &filter->priv->other_context))
         return GST_STATE_CHANGE_FAILURE;
-      }
 
       gst_gl_display_filter_gl_api (filter->display,
           filter_class->supported_gl_api);
@@ -389,20 +480,11 @@ gst_gl_base_filter_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_NULL:
-      if (filter->priv->other_context) {
-        gst_object_unref (filter->priv->other_context);
-        filter->priv->other_context = NULL;
-      }
-
-      if (filter->display) {
-        gst_object_unref (filter->display);
-        filter->display = NULL;
-      }
-
-      if (filter->context) {
-        gst_object_unref (filter->context);
-        filter->context = NULL;
-      }
+      g_rec_mutex_lock (&filter->priv->context_lock);
+      gst_clear_object (&filter->priv->other_context);
+      gst_clear_object (&filter->display);
+      gst_clear_object (&filter->context);
+      g_rec_mutex_unlock (&filter->priv->context_lock);
       break;
     default:
       break;
@@ -411,25 +493,58 @@ gst_gl_base_filter_change_state (GstElement * element,
   return ret;
 }
 
-/**
- * gst_gl_base_filter_find_gl_context:
- * @filter: a #GstGLBaseFilter
- *
- * Returns: Whether an OpenGL context could be retrieved or created successfully
- *
- * Since: 1.16
- */
-gboolean
-gst_gl_base_filter_find_gl_context (GstGLBaseFilter * filter)
+static void
+gst_gl_base_filter_set_context (GstElement * element, GstContext * context)
+{
+  GstGLBaseFilter *filter = GST_GL_BASE_FILTER (element);
+  GstGLBaseFilterClass *filter_class = GST_GL_BASE_FILTER_GET_CLASS (filter);
+  GstGLDisplay *old_display, *new_display;
+
+  g_rec_mutex_lock (&filter->priv->context_lock);
+  old_display = filter->display ? gst_object_ref (filter->display) : NULL;
+  gst_gl_handle_set_context (element, context, &filter->display,
+      &filter->priv->other_context);
+  if (filter->display)
+    gst_gl_display_filter_gl_api (filter->display,
+        filter_class->supported_gl_api);
+  new_display = filter->display ? gst_object_ref (filter->display) : NULL;
+
+  if (old_display && new_display) {
+    if (old_display != new_display) {
+      gst_clear_object (&filter->context);
+      if (gst_gl_base_filter_find_gl_context_unlocked (filter)) {
+        if (filter->in_caps && filter->out_caps) {
+          gl_set_caps_unlocked (filter);
+        }
+      }
+    }
+  }
+  g_rec_mutex_unlock (&filter->priv->context_lock);
+  gst_clear_object (&old_display);
+  gst_clear_object (&new_display);
+
+  GST_ELEMENT_CLASS (parent_class)->set_context (element, context);
+}
+
+static gboolean
+gst_gl_base_filter_find_gl_context_unlocked (GstGLBaseFilter * filter)
 {
   GstGLBaseFilterClass *filter_class = GST_GL_BASE_FILTER_GET_CLASS (filter);
   GError *error = NULL;
   gboolean new_context = FALSE;
 
+  GST_DEBUG_OBJECT (filter, "attempting to find an OpenGL context, existing %"
+      GST_PTR_FORMAT, filter->context);
+
   if (!filter->context)
     new_context = TRUE;
 
-  _find_local_gl_context (filter);
+  _find_local_gl_context_unlocked (filter);
+
+  if (!filter->display) {
+    GST_WARNING_OBJECT (filter, "filter has NULL display.");
+    return FALSE;
+  }
 
   if (!filter->context) {
     GST_OBJECT_LOCK (filter->display);
@@ -449,6 +564,8 @@ gst_gl_base_filter_find_gl_context (GstGLBaseFilter * filter)
     } while (!gst_gl_display_add_context (filter->display, filter->context));
     GST_OBJECT_UNLOCK (filter->display);
   }
+  GST_INFO_OBJECT (filter, "found OpenGL context %" GST_PTR_FORMAT,
+      filter->context);
 
   if (new_context || !filter->priv->gl_started) {
     if (filter->priv->gl_started)
@@ -499,3 +616,42 @@ error:
     return FALSE;
   }
 }
+
+/**
+ * gst_gl_base_filter_find_gl_context:
+ * @filter: a #GstGLBaseFilter
+ *
+ * Returns: Whether an OpenGL context could be retrieved or created successfully
+ *
+ * Since: 1.16
+ */
+gboolean
+gst_gl_base_filter_find_gl_context (GstGLBaseFilter * filter)
+{
+  gboolean ret;
+  g_rec_mutex_lock (&filter->priv->context_lock);
+  ret = gst_gl_base_filter_find_gl_context_unlocked (filter);
+  g_rec_mutex_unlock (&filter->priv->context_lock);
+  return ret;
+}
+
+/**
+ * gst_gl_base_filter_get_gl_context:
+ * @filter: a #GstGLBaseFilter
+ *
+ * Returns: (transfer full) (nullable): the #GstGLContext found by @filter
+ *
+ * Since: 1.18
+ */
+GstGLContext *
+gst_gl_base_filter_get_gl_context (GstGLBaseFilter * filter)
+{
+  GstGLContext *ret;
+
+  g_return_val_if_fail (GST_IS_GL_BASE_FILTER (filter), NULL);
+
+  g_rec_mutex_lock (&filter->priv->context_lock);
+  ret = filter->context ? gst_object_ref (filter->context) : NULL;
+  g_rec_mutex_unlock (&filter->priv->context_lock);
+  return ret;
+}
