@@ -27,6 +27,7 @@
 #include "gstglbasememory.h"
 
 #include "gstglcontext.h"
+#include "gstglcontext_private.h"
 #include "gstglquery.h"
 
 /**
@@ -52,10 +53,15 @@ GST_DEBUG_CATEGORY_STATIC (GST_CAT_GL_BASE_MEMORY);
 
 GST_DEFINE_MINI_OBJECT_TYPE (GstGLBaseMemory, gst_gl_base_memory);
 
+/**
+ * gst_gl_base_memory_error_quark:
+ *
+ * Returns: the quark used for #GstGLBaseMemory in #GError's
+ */
 GQuark
 gst_gl_base_memory_error_quark (void)
 {
-  return g_quark_from_static_string ("gst-gl-base-buffer-error-quark");
+  return g_quark_from_static_string ("gst-gl-base-memory-error-quark");
 }
 
 static gboolean
@@ -89,7 +95,11 @@ _mem_create_gl (GstGLContext * context, struct create_data *transfer)
 
   g_return_if_fail (alloc_class->create != NULL);
 
-  transfer->mem->query = gst_gl_query_new (context, GST_GL_QUERY_TIME_ELAPSED);
+  /* Don't do expensive queries when debugging is disabled */
+  transfer->mem->query = NULL;
+  if (_gst_gl_context_debug_is_enabled (context))
+    transfer->mem->query =
+        gst_gl_query_new (context, GST_GL_QUERY_TIME_ELAPSED);
 
   if ((transfer->result = alloc_class->create (transfer->mem, &error)))
     return;
@@ -199,7 +209,7 @@ _align_data (gpointer data, gsize align)
  * gst_gl_base_memory_alloc_data:
  * @gl_mem: a #GstGLBaseMemory
  *
- * Note: only intended for subclass usage to allocate the sytem memory buffer
+ * Note: only intended for subclass usage to allocate the system memory buffer
  * on demand.  If there is already a non-NULL data pointer in @gl_mem->data,
  * then this function imply returns TRUE.
  *
@@ -557,7 +567,7 @@ gst_is_gl_base_memory (GstMemory * mem)
  * @offset: the offset to start at
  * @size: the number of bytes to copy
  *
- * Returns: whether the copy suceeded.
+ * Returns: whether the copy succeeded.
  *
  * Since: 1.8
  */
@@ -613,7 +623,7 @@ gst_gl_base_memory_memcpy (GstGLBaseMemory * src, GstGLBaseMemory * dest,
  * @notify will be called once for each allocated memory using these @params
  * when freeing the memory.
  *
- * Returns: whether the paramaters could be initialized
+ * Returns: whether the parameters could be initialized
  *
  * Since: 1.8
  */
@@ -694,7 +704,7 @@ gst_gl_allocation_params_free (GstGLAllocationParams * params)
  * @params: the source #GstGLAllocationParams
  *
  * Frees the dynamically allocated data in @params.  Direct subclasses
- * should call this function in their own overriden free function.
+ * should call this function in their own overridden free function.
  *
  * Since: 1.8
  */
@@ -713,7 +723,7 @@ gst_gl_allocation_params_free_data (GstGLAllocationParams * params)
  * @dest: the destination #GstGLAllocationParams
  *
  * Copies the dynamically allocated data from @src to @dest.  Direct subclasses
- * should call this function in their own overriden copy function.
+ * should call this function in their own overridden copy function.
  *
  * Since: 1.8
  */
