@@ -43,6 +43,12 @@
  * with #GstGLVideoAllocationParams.
  *
  * Data is uploaded or downloaded from the GPU as is necessary.
+ *
+ * The #GstCaps that is used for #GstGLMemory based buffers should contain
+ * the %GST_CAPS_FEATURE_MEMORY_GL_MEMORY as a #GstCapsFeatures and should
+ * contain a 'texture-target' field with one of the #GstGLTextureTarget values
+ * as a string, i.e. some combination of 'texture-target=(string){2D,
+ * rectangle, external-oes}'.
  */
 
 #define USING_OPENGL(context) (gst_gl_context_check_gl_version (context, GST_GL_API_OPENGL, 1, 0))
@@ -60,7 +66,7 @@ static GstAllocator *_gl_memory_allocator;
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_GL_MEMORY);
 #define GST_CAT_DEFAULT GST_CAT_GL_MEMORY
 
-/* compatability definitions... */
+/* compatibility definitions... */
 #ifndef GL_UNPACK_ROW_LENGTH
 #define GL_UNPACK_ROW_LENGTH 0x0CF2
 #endif
@@ -360,11 +366,11 @@ gst_gl_memory_init (GstGLMemory * mem, GstAllocator * allocator,
 /**
  * gst_gl_memory_read_pixels:
  * @gl_mem: a #GstGLMemory
- * @read_pointer: the data pointer to pass to glReadPixels
+ * @write_pointer: the data pointer to pass to glReadPixels
  *
- * Reads the texture in #GstGLMemory into @read_pointer if no buffer is bound
- * to %GL_PIXEL_PACK_BUFFER.  Otherwise @read_pointer is the byte offset into
- * the currently bound %GL_PIXEL_PACK_BUFFER buffer to store the result of
+ * Reads the texture in #GstGLMemory into @write_pointer if no buffer is bound
+ * to `GL_PIXEL_PACK_BUFFER`.  Otherwise @write_pointer is the byte offset into
+ * the currently bound `GL_PIXEL_PACK_BUFFER` buffer to store the result of
  * glReadPixels.  See the OpenGL specification for glReadPixels for more
  * details.
  *
@@ -373,7 +379,7 @@ gst_gl_memory_init (GstGLMemory * mem, GstAllocator * allocator,
  * Since: 1.8
  */
 gboolean
-gst_gl_memory_read_pixels (GstGLMemory * gl_mem, gpointer read_pointer)
+gst_gl_memory_read_pixels (GstGLMemory * gl_mem, gpointer write_pointer)
 {
   GstGLContext *context = gl_mem->mem.context;
   const GstGLFuncs *gl = context->gl_vtable;
@@ -419,7 +425,7 @@ gst_gl_memory_read_pixels (GstGLMemory * gl_mem, gpointer read_pointer)
 
   _gst_gl_memory_start_log (gl_mem, "glReadPixels");
   gl->ReadPixels (0, 0, gl_mem->tex_width, GL_MEM_HEIGHT (gl_mem), format,
-      type, read_pointer);
+      type, write_pointer);
   _gst_gl_memory_end_log (gl_mem);
 
   gl->BindFramebuffer (GL_FRAMEBUFFER, 0);
@@ -518,6 +524,8 @@ _upload_cpu_write (GstGLMemory * gl_mem, GstMapInfo * info, gsize maxsize)
  * @gl_mem: a #GstGLMemory
  * @read_pointer: the data pointer to pass to glTexSubImage
  *
+ * Reads the texture in @read_pointer into @gl_mem.
+ *
  * See gst_gl_memory_read_pixels() for what @read_pointer signifies.
  *
  * Since: 1.8
@@ -982,10 +990,10 @@ gst_gl_memory_allocator_init (GstGLMemoryAllocator * allocator)
  * @width: width of @tex_id
  * @height: height of @tex_id
  *
- * Copies @gl_mem into the texture specfified by @tex_id.  The format of @tex_id
+ * Copies @gl_mem into the texture specified by @tex_id.  The format of @tex_id
  * is specified by @tex_format, @width and @height.
  *
- * Returns: Whether the copy suceeded
+ * Returns: Whether the copy succeeded
  *
  * Since: 1.8
  */
@@ -1470,6 +1478,8 @@ gst_gl_memory_setup_buffer (GstGLMemoryAllocator * allocator,
       || n_mem * views == n_wrapped_pointers, FALSE);
 
   for (v = 0; v < views; v++) {
+    GstVideoMeta *meta;
+
     for (i = 0; i < n_mem; i++) {
       GstGLMemory *gl_mem;
 
@@ -1498,11 +1508,14 @@ gst_gl_memory_setup_buffer (GstGLMemoryAllocator * allocator,
       gst_buffer_append_memory (buffer, (GstMemory *) gl_mem);
     }
 
-    gst_buffer_add_video_meta_full (buffer, v,
+    meta = gst_buffer_add_video_meta_full (buffer, v,
         GST_VIDEO_INFO_FORMAT (params->v_info),
         GST_VIDEO_INFO_WIDTH (params->v_info),
         GST_VIDEO_INFO_HEIGHT (params->v_info), n_mem, params->v_info->offset,
         params->v_info->stride);
+
+    if (params->valign)
+      gst_video_meta_set_alignment (meta, *params->valign);
   }
 
   return TRUE;
