@@ -25,7 +25,9 @@
 #include "config.h"
 #endif
 
-#include <linux/input.h>
+#include <locale.h>
+#include <sys/mman.h>
+#include <unistd.h>
 
 #include "wayland_event_source.h"
 
@@ -33,7 +35,6 @@
 #include <gst/gl/gstglcontext.h>
 
 #include "gstgldisplay_wayland.h"
-#include "gstgldisplay_wayland_private.h"
 #include "gstglwindow_wayland_egl.h"
 
 #include "../gstglwindow_private.h"
@@ -61,7 +62,6 @@ static gboolean gst_gl_window_wayland_egl_set_render_rectangle (GstGLWindow *
 static void gst_gl_window_wayland_egl_set_preferred_size (GstGLWindow * window,
     gint width, gint height);
 
-#if 0
 static void
 pointer_handle_enter (void *data, struct wl_pointer *pointer, uint32_t serial,
     struct wl_surface *surface, wl_fixed_t sx_w, wl_fixed_t sy_w)
@@ -72,6 +72,7 @@ pointer_handle_enter (void *data, struct wl_pointer *pointer, uint32_t serial,
 
   window_egl->display.serial = serial;
 
+  /* FIXME: Not sure how useful this is */
   if (window_egl->display.default_cursor) {
     image = window_egl->display.default_cursor->images[0];
     buffer = wl_cursor_image_get_buffer (image);
@@ -97,62 +98,44 @@ pointer_handle_motion (void *data, struct wl_pointer *pointer, uint32_t time,
     wl_fixed_t sx_w, wl_fixed_t sy_w)
 {
   GstGLWindowWaylandEGL *window_egl = data;
+  GstGLWindow *window = GST_GL_WINDOW (window_egl);
 
   window_egl->display.pointer_x = wl_fixed_to_double (sx_w);
   window_egl->display.pointer_y = wl_fixed_to_double (sy_w);
-}
-
-enum wl_edges
-{
-  WL_EDGE_NONE = 0,
-  WL_EDGE_TOP = 1,
-  WL_EDGE_BOTTOM = 2,
-  WL_EDGE_LEFT = 4,
-  WL_EDGE_RIGHT = 8,
-};
-
-static guint
-_get_closest_pointer_corner (GstGLWindowWaylandEGL * window_egl)
-{
-  guint edges = 0;
-  gdouble win_width, win_height;
-  gdouble p_x, p_y;
-
-  win_width = (gdouble) window_egl->window.window_width;
-  win_height = (gdouble) window_egl->window.window_height;
-  p_x = window_egl->display.pointer_x;
-  p_y = window_egl->display.pointer_y;
-
-  if (win_width == 0.0 || win_height == 0.0)
-    return WL_EDGE_NONE;
-
-  edges |= win_width / 2.0 - p_x < 0.0 ? WL_EDGE_RIGHT : WL_EDGE_LEFT;
-  edges |= win_height / 2.0 - p_y < 0.0 ? WL_EDGE_BOTTOM : WL_EDGE_TOP;
-
-  return edges;
+  gst_gl_window_send_mouse_event (window, "mouse-move", 0,
+      window_egl->display.pointer_x, window_egl->display.pointer_y);
 }
 
 static void
 pointer_handle_button (void *data, struct wl_pointer *pointer, uint32_t serial,
-    uint32_t time, uint32_t button, uint32_t state_w)
+    uint32_t time, uint32_t button, uint32_t state)
 {
   GstGLWindowWaylandEGL *window_egl = data;
-  guint edges = _get_closest_pointer_corner (window_egl);
-  window_egl->display.serial = serial;
+  GstGLWindow *window = GST_GL_WINDOW (window_egl);
+  const char *event_type;
 
-  if (button == BTN_LEFT && state_w == WL_POINTER_BUTTON_STATE_PRESSED)
-    wl_shell_surface_move (window_egl->window.wl_shell_surface,
-        window_egl->display.seat, serial);
-
-  if (button == BTN_RIGHT && state_w == WL_POINTER_BUTTON_STATE_PRESSED)
-    wl_shell_surface_resize (window_egl->window.wl_shell_surface,
-        window_egl->display.seat, serial, edges);
+  event_type = state == 1 ? "mouse-button-press" : "mouse-button-release";
+  gst_gl_window_send_mouse_event (window, event_type, button,
+      window_egl->display.pointer_x, window_egl->display.pointer_y);
 }
 
 static void
 pointer_handle_axis (void *data, struct wl_pointer *pointer, uint32_t time,
     uint32_t axis, wl_fixed_t value)
 {
+  GstGLWindowWaylandEGL *window_egl = data;
+  GstGLWindow *window = GST_GL_WINDOW (window_egl);
+  gdouble delta_x, delta_y;
+  gdouble delta = -wl_fixed_to_double (value);
+  if (axis == 1) {
+    delta_x = delta;
+    delta_y = 0;
+  } else {
+    delta_x = 0;
+    delta_y = delta;
+  }
+  gst_gl_window_send_scroll_event (window, window_egl->display.pointer_x,
+      window_egl->display.pointer_y, delta_x, delta_y);
 }
 
 static const struct wl_pointer_listener pointer_listener = {
@@ -190,10 +173,19 @@ seat_handle_capabilities (void *data, struct wl_seat *seat,
 #endif
 }
 
+static void
+seat_name (void *data, struct wl_seat *seat, const char *name)
+{
+  GstGLWindowWaylandEGL *window_egl = data;
+
+  GST_TRACE_OBJECT (window_egl, "seat %p has name %s", seat, name);
+}
+
 static const struct wl_seat_listener seat_listener = {
   seat_handle_capabilities,
+  seat_name
 };
-#endif
+
 static void
 handle_ping (void *data, struct wl_shell_surface *wl_shell_surface,
     uint32_t serial)
@@ -234,11 +226,14 @@ static const struct wl_shell_surface_listener wl_shell_surface_listener = {
 static void
 handle_xdg_toplevel_close (void *data, struct xdg_toplevel *xdg_toplevel)
 {
-  GstGLWindowWaylandEGL *window_egl = data;
+  GstGLWindow *window = data;
 
   GST_DEBUG ("XDG toplevel got a \"close\" event.");
 
-  gst_gl_window_wayland_egl_close (GST_GL_WINDOW (window_egl));
+  if (window->close)
+    window->close (window->close_data);
+
+  gst_gl_display_remove_window (window->display, window);
 }
 
 static void
@@ -298,35 +293,19 @@ destroy_surfaces (GstGLWindowWaylandEGL * window_egl)
 static void
 create_xdg_surface_and_toplevel (GstGLWindowWaylandEGL * window_egl)
 {
-  GstGLDisplayWayland *display =
-      GST_GL_DISPLAY_WAYLAND (GST_GL_WINDOW (window_egl)->display);
-  struct xdg_wm_base *xdg_wm_base;
   struct xdg_surface *xdg_surface;
   struct xdg_toplevel *xdg_toplevel;
 
   GST_DEBUG ("Creating surfaces XDG-shell");
 
   /* First create the XDG surface */
-  xdg_wm_base = gst_gl_display_wayland_get_xdg_wm_base (display);
-  if (window_egl->window.queue) {
-    wl_proxy_set_queue ((struct wl_proxy *) xdg_wm_base,
-        window_egl->window.queue);
-  }
-  xdg_surface = xdg_wm_base_get_xdg_surface (xdg_wm_base,
+  xdg_surface = xdg_wm_base_get_xdg_surface (window_egl->display.xdg_wm_base,
       window_egl->window.surface);
-  if (window_egl->window.queue) {
-    wl_proxy_set_queue ((struct wl_proxy *) xdg_surface,
-        window_egl->window.queue);
-  }
   xdg_surface_add_listener (xdg_surface, &xdg_surface_listener, window_egl);
 
   /* Then the XDG top-level */
   xdg_toplevel = xdg_surface_get_toplevel (xdg_surface);
   xdg_toplevel_set_title (xdg_toplevel, "OpenGL Renderer");
-  if (window_egl->window.queue) {
-    wl_proxy_set_queue ((struct wl_proxy *) xdg_toplevel,
-        window_egl->window.queue);
-  }
   xdg_toplevel_add_listener (xdg_toplevel, &xdg_toplevel_listener, window_egl);
 
   /* Commit the xdg_surface state */
@@ -340,20 +319,13 @@ create_xdg_surface_and_toplevel (GstGLWindowWaylandEGL * window_egl)
 static void
 create_wl_shell_surface (GstGLWindowWaylandEGL * window_egl)
 {
-  GstGLDisplayWayland *display =
-      GST_GL_DISPLAY_WAYLAND (GST_GL_WINDOW (window_egl)->display);
   struct wl_shell_surface *wl_shell_surface;
 
   GST_DEBUG ("Creating surfaces for wl-shell");
 
-  wl_shell_surface = wl_shell_get_shell_surface (display->shell,
+  wl_shell_surface = wl_shell_get_shell_surface (window_egl->display.shell,
       window_egl->window.surface);
 
-  if (window_egl->window.queue) {
-    wl_proxy_set_queue ((struct wl_proxy *) wl_shell_surface,
-        window_egl->window.queue);
-  }
-
   wl_shell_surface_add_listener (wl_shell_surface, &wl_shell_surface_listener,
       window_egl);
   wl_shell_surface_set_title (wl_shell_surface, "OpenGL Renderer");
@@ -365,21 +337,16 @@ create_wl_shell_surface (GstGLWindowWaylandEGL * window_egl)
 static void
 create_surfaces (GstGLWindowWaylandEGL * window_egl)
 {
-  GstGLDisplayWayland *display =
-      GST_GL_DISPLAY_WAYLAND (GST_GL_WINDOW (window_egl)->display);
   gint width, height;
 
   if (!window_egl->window.surface) {
     window_egl->window.surface =
-        wl_compositor_create_surface (display->compositor);
-    if (window_egl->window.queue)
-      wl_proxy_set_queue ((struct wl_proxy *) window_egl->window.surface,
-          window_egl->window.queue);
+        wl_compositor_create_surface (window_egl->display.compositor);
   }
 
   if (window_egl->window.foreign_surface) {
     /* (re)parent */
-    if (!display->subcompositor) {
+    if (!window_egl->display.subcompositor) {
       GST_ERROR_OBJECT (window_egl,
           "Wayland server does not support subsurfaces");
       window_egl->window.foreign_surface = NULL;
@@ -388,11 +355,8 @@ create_surfaces (GstGLWindowWaylandEGL * window_egl)
 
     if (!window_egl->window.subsurface) {
       window_egl->window.subsurface =
-          wl_subcompositor_get_subsurface (display->subcompositor,
+          wl_subcompositor_get_subsurface (window_egl->display.subcompositor,
           window_egl->window.surface, window_egl->window.foreign_surface);
-      if (window_egl->window.queue)
-        wl_proxy_set_queue ((struct wl_proxy *) window_egl->window.subsurface,
-            window_egl->window.queue);
 
       wl_subsurface_set_position (window_egl->window.subsurface,
           window_egl->window.window_x, window_egl->window.window_y);
@@ -400,7 +364,7 @@ create_surfaces (GstGLWindowWaylandEGL * window_egl)
     }
   } else {
   shell_window:
-    if (gst_gl_display_wayland_get_xdg_wm_base (display)) {
+    if (window_egl->display.xdg_wm_base) {
       if (!window_egl->window.xdg_surface)
         create_xdg_surface_and_toplevel (window_egl);
     } else if (!window_egl->window.wl_shell_surface) {
@@ -440,9 +404,6 @@ create_surfaces (GstGLWindowWaylandEGL * window_egl)
 
     window_egl->window.native =
         wl_egl_window_create (window_egl->window.surface, width, height);
-    if (window_egl->window.queue)
-      wl_proxy_set_queue ((struct wl_proxy *) window_egl->window.native,
-          window_egl->window.queue);
   }
 }
 
@@ -493,11 +454,18 @@ gst_gl_window_wayland_egl_new (GstGLDisplay * display)
 }
 
 static void
-gst_gl_window_wayland_egl_close (GstGLWindow * window)
+gst_gl_window_wayland_egl_close (GstGLWindow * gl_window)
 {
   GstGLWindowWaylandEGL *window_egl;
+  struct display *display;
 
-  window_egl = GST_GL_WINDOW_WAYLAND_EGL (window);
+  window_egl = GST_GL_WINDOW_WAYLAND_EGL (gl_window);
+  display = &window_egl->display;
+
+  if (display->pointer != NULL) {
+    wl_pointer_destroy (display->pointer);
+    display->pointer = NULL;
+  }
 
   destroy_surfaces (window_egl);
 
@@ -505,9 +473,55 @@ gst_gl_window_wayland_egl_close (GstGLWindow * window)
   g_source_unref (window_egl->wl_source);
   window_egl->wl_source = NULL;
 
-  GST_GL_WINDOW_CLASS (parent_class)->close (window);
+  GST_GL_WINDOW_CLASS (parent_class)->close (gl_window);
 }
 
+static void
+handle_xdg_wm_base_ping (void *user_data, struct xdg_wm_base *xdg_wm_base,
+    uint32_t serial)
+{
+  xdg_wm_base_pong (xdg_wm_base, serial);
+}
+
+static const struct xdg_wm_base_listener xdg_wm_base_listener = {
+  handle_xdg_wm_base_ping
+};
+
+static void
+registry_handle_global (void *data, struct wl_registry *registry,
+    uint32_t name, const char *interface, uint32_t version)
+{
+  GstGLWindowWaylandEGL *window_wayland = data;
+
+  GST_TRACE_OBJECT (window_wayland, "registry_handle_global with registry %p, "
+      "interface %s, version %u", registry, interface, version);
+
+  if (g_strcmp0 (interface, "wl_compositor") == 0) {
+    window_wayland->display.compositor =
+        wl_registry_bind (registry, name, &wl_compositor_interface, 1);
+  } else if (g_strcmp0 (interface, "wl_subcompositor") == 0) {
+    window_wayland->display.subcompositor =
+        wl_registry_bind (registry, name, &wl_subcompositor_interface, 1);
+  } else if (g_strcmp0 (interface, "xdg_wm_base") == 0) {
+    window_wayland->display.xdg_wm_base =
+        wl_registry_bind (registry, name, &xdg_wm_base_interface, 1);
+    xdg_wm_base_add_listener (window_wayland->display.xdg_wm_base,
+        &xdg_wm_base_listener, window_wayland);
+  } else if (g_strcmp0 (interface, "wl_shell") == 0) {
+    window_wayland->display.shell =
+        wl_registry_bind (registry, name, &wl_shell_interface, 1);
+  } else if (g_strcmp0 (interface, "wl_seat") == 0) {
+    window_wayland->display.seat =
+        wl_registry_bind (registry, name, &wl_seat_interface, 4);
+    wl_seat_add_listener (window_wayland->display.seat, &seat_listener,
+        window_wayland);
+  }
+}
+
+static const struct wl_registry_listener registry_listener = {
+  registry_handle_global
+};
+
 static gboolean
 gst_gl_window_wayland_egl_open (GstGLWindow * window, GError ** error)
 {
@@ -529,7 +543,29 @@ gst_gl_window_wayland_egl_open (GstGLWindow * window, GError ** error)
     return FALSE;
   }
 
-  window_egl->window.queue = wl_display_create_queue (display->display);
+  /* we create a proxy wrapper for the display so that we can set the queue on
+   * it. This allows us to avoid sprinkling `wl_proxy_set_queue()` calls for
+   * each wayland resource we create as well as removing a race between
+   * creation and the `wl_proxy_set_queue()` call. */
+  window_egl->display.display = wl_proxy_create_wrapper (display->display);
+  window_egl->window.queue =
+      wl_display_create_queue (window_egl->display.display);
+
+  wl_proxy_set_queue ((struct wl_proxy *) window_egl->display.display,
+      window_egl->window.queue);
+
+  window_egl->display.registry =
+      wl_display_get_registry (window_egl->display.display);
+  wl_registry_add_listener (window_egl->display.registry, &registry_listener,
+      window_egl);
+
+  if (wl_display_roundtrip_queue (display->display,
+          window_egl->window.queue) < 0) {
+    g_set_error (error, GST_GL_WINDOW_ERROR,
+        GST_GL_WINDOW_ERROR_RESOURCE_UNAVAILABLE,
+        "Failed to perform a wayland roundtrip");
+    return FALSE;
+  }
 
   window_egl->wl_source = wayland_event_source_new (display->display,
       window_egl->window.queue);
@@ -574,13 +610,12 @@ gst_gl_window_wayland_egl_set_window_handle (GstGLWindow * window,
 static void
 _roundtrip_async (GstGLWindow * window)
 {
-  GstGLDisplayWayland *display_wayland =
-      GST_GL_DISPLAY_WAYLAND (window->display);
+  GstGLDisplayWayland *display = GST_GL_DISPLAY_WAYLAND (window->display);
   GstGLWindowWaylandEGL *window_egl = GST_GL_WINDOW_WAYLAND_EGL (window);
 
   create_surfaces (window_egl);
 
-  if (gst_gl_wl_display_roundtrip_queue (display_wayland->display,
+  if (wl_display_roundtrip_queue (display->display,
           window_egl->window.queue) < 0)
     GST_WARNING_OBJECT (window, "failed a roundtrip");
 }
