@@ -19,6 +19,7 @@
 
 /**
  * SECTION:gstrtpbaseaudiopayload
+ * @title: GstRTPBaseAudioPayload
  * @short_description: Base class for audio RTP payloader
  *
  * Provides a base class for audio RTP payloaders for frame or sample based
@@ -36,13 +37,12 @@
  * sent in a last RTP packet. In the case of frame based codecs, the resulting
  * RTP packets always contain full frames.
  *
- * <refsect2>
- * <title>Usage</title>
- * <para>
+ * ## Usage
+ *
  * To use this base class, your child element needs to call either
  * gst_rtp_base_audio_payload_set_frame_based() or
  * gst_rtp_base_audio_payload_set_sample_based(). This is usually done in the
- * element's _init() function. Then, the child element must call either
+ * element's `_init()` function. Then, the child element must call either
  * gst_rtp_base_audio_payload_set_frame_options(),
  * gst_rtp_base_audio_payload_set_sample_options() or
  * gst_rtp_base_audio_payload_set_samplebits_options. Since
@@ -50,8 +50,7 @@
  * must set any variables or call/override any functions required by that base
  * class. The child element does not need to override any other functions
  * specific to GstRTPBaseAudioPayload.
- * </para>
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -62,6 +61,7 @@
 #include <string.h>
 #include <gst/rtp/gstrtpbuffer.h>
 #include <gst/base/gstadapter.h>
+#include <gst/audio/audio.h>
 
 #include "gstrtpbaseaudiopayload.h"
 
@@ -110,15 +110,11 @@ struct _GstRTPBaseAudioPayloadPrivate
   guint cached_max_length;
   guint cached_ptime_multiple;
   guint cached_align;
+  guint cached_csrc_count;
 
   gboolean buffer_list;
 };
 
-
-#define GST_RTP_BASE_AUDIO_PAYLOAD_GET_PRIVATE(o) \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((o), GST_TYPE_RTP_BASE_AUDIO_PAYLOAD, \
-                                GstRTPBaseAudioPayloadPrivate))
-
 static void gst_rtp_base_audio_payload_finalize (GObject * object);
 
 static void gst_rtp_base_audio_payload_set_property (GObject * object,
@@ -157,8 +153,12 @@ static GstStateChangeReturn gst_rtp_base_payload_audio_change_state (GstElement
 static gboolean gst_rtp_base_payload_audio_sink_event (GstRTPBasePayload
     * payload, GstEvent * event);
 
+/* cached quark to avoid contention on the global quark table lock */
+#define META_TAG_AUDIO meta_tag_audio_quark
+static GQuark meta_tag_audio_quark;
+
 #define gst_rtp_base_audio_payload_parent_class parent_class
-G_DEFINE_TYPE (GstRTPBaseAudioPayload, gst_rtp_base_audio_payload,
+G_DEFINE_TYPE_WITH_PRIVATE (GstRTPBaseAudioPayload, gst_rtp_base_audio_payload,
     GST_TYPE_RTP_BASE_PAYLOAD);
 
 static void
@@ -168,7 +168,7 @@ gst_rtp_base_audio_payload_class_init (GstRTPBaseAudioPayloadClass * klass)
   GstElementClass *gstelement_class;
   GstRTPBasePayloadClass *gstrtpbasepayload_class;
 
-  g_type_class_add_private (klass, sizeof (GstRTPBaseAudioPayloadPrivate));
+  meta_tag_audio_quark = g_quark_from_static_string (GST_META_TAG_AUDIO_STR);
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
@@ -198,7 +198,7 @@ gst_rtp_base_audio_payload_class_init (GstRTPBaseAudioPayloadClass * klass)
 static void
 gst_rtp_base_audio_payload_init (GstRTPBaseAudioPayload * payload)
 {
-  payload->priv = GST_RTP_BASE_AUDIO_PAYLOAD_GET_PRIVATE (payload);
+  payload->priv = gst_rtp_base_audio_payload_get_instance_private (payload);
 
   /* these need to be set by child object if frame based */
   payload->frame_size = 0;
@@ -415,7 +415,7 @@ gst_rtp_base_audio_payload_set_meta (GstRTPBaseAudioPayload * payload,
   }
   gst_rtp_buffer_unmap (&rtp);
 
-  GST_BUFFER_TIMESTAMP (buffer) = timestamp;
+  GST_BUFFER_PTS (buffer) = timestamp;
 
   /* get the offset in RTP time */
   GST_BUFFER_OFFSET (buffer) = priv->bytes_to_rtptime (payload, priv->offset);
@@ -434,7 +434,7 @@ gst_rtp_base_audio_payload_set_meta (GstRTPBaseAudioPayload * payload,
 /**
  * gst_rtp_base_audio_payload_push:
  * @baseaudiopayload: a #GstRTPBasePayload
- * @data: data to set as payload
+ * @data: (array length=payload_len): data to set as payload
  * @payload_len: length of payload
  * @timestamp: a #GstClockTime
  *
@@ -460,7 +460,8 @@ gst_rtp_base_audio_payload_push (GstRTPBaseAudioPayload * baseaudiopayload,
       payload_len, GST_TIME_ARGS (timestamp));
 
   /* create buffer to hold the payload */
-  outbuf = gst_rtp_buffer_new_allocate (payload_len, 0, 0);
+  outbuf = gst_rtp_base_payload_allocate_output_buffer (basepayload,
+      payload_len, 0, 0);
 
   /* copy payload */
   gst_rtp_buffer_map (outbuf, GST_MAP_WRITE, &rtp);
@@ -477,6 +478,35 @@ gst_rtp_base_audio_payload_push (GstRTPBaseAudioPayload * baseaudiopayload,
   return ret;
 }
 
+typedef struct
+{
+  GstRTPBaseAudioPayload *pay;
+  GstBuffer *outbuf;
+} CopyMetaData;
+
+static gboolean
+foreach_metadata (GstBuffer * inbuf, GstMeta ** meta, gpointer user_data)
+{
+  CopyMetaData *data = user_data;
+  GstRTPBaseAudioPayload *pay = data->pay;
+  GstBuffer *outbuf = data->outbuf;
+  const GstMetaInfo *info = (*meta)->info;
+  const gchar *const *tags = gst_meta_api_type_get_tags (info->api);
+
+  if (info->transform_func && (!tags || (g_strv_length ((gchar **) tags) == 1
+              && gst_meta_api_type_has_tag (info->api, META_TAG_AUDIO)))) {
+    GstMetaTransformCopy copy_data = { FALSE, 0, -1 };
+    GST_DEBUG_OBJECT (pay, "copy metadata %s", g_type_name (info->api));
+    /* simply copy then */
+    info->transform_func (outbuf, *meta, inbuf,
+        _gst_meta_transform_copy, &copy_data);
+  } else {
+    GST_DEBUG_OBJECT (pay, "not copying metadata %s", g_type_name (info->api));
+  }
+
+  return TRUE;
+}
+
 static GstFlowReturn
 gst_rtp_base_audio_payload_push_buffer (GstRTPBaseAudioPayload *
     baseaudiopayload, GstBuffer * buffer, GstClockTime timestamp)
@@ -484,7 +514,6 @@ gst_rtp_base_audio_payload_push_buffer (GstRTPBaseAudioPayload *
   GstRTPBasePayload *basepayload;
   GstRTPBaseAudioPayloadPrivate *priv;
   GstBuffer *outbuf;
-  guint8 *payload;
   guint payload_len;
   GstFlowReturn ret;
 
@@ -496,13 +525,8 @@ gst_rtp_base_audio_payload_push_buffer (GstRTPBaseAudioPayload *
   GST_DEBUG_OBJECT (baseaudiopayload, "Pushing %d bytes ts %" GST_TIME_FORMAT,
       payload_len, GST_TIME_ARGS (timestamp));
 
-  if (priv->buffer_list) {
-    /* create just the RTP header buffer */
-    outbuf = gst_rtp_buffer_new_allocate (0, 0, 0);
-  } else {
-    /* create buffer to hold the payload */
-    outbuf = gst_rtp_buffer_new_allocate (payload_len, 0, 0);
-  }
+  /* create just the RTP header buffer */
+  outbuf = gst_rtp_base_payload_allocate_output_buffer (basepayload, 0, 0, 0);
 
   /* set metadata */
   gst_rtp_base_audio_payload_set_meta (baseaudiopayload, outbuf, payload_len,
@@ -525,15 +549,13 @@ gst_rtp_base_audio_payload_push_buffer (GstRTPBaseAudioPayload *
     GST_DEBUG_OBJECT (baseaudiopayload, "Pushing list %p", list);
     ret = gst_rtp_base_payload_push_list (basepayload, list);
   } else {
-    GstRTPBuffer rtp = { NULL };
+    CopyMetaData data;
 
     /* copy payload */
-    gst_rtp_buffer_map (outbuf, GST_MAP_WRITE, &rtp);
-    payload = gst_rtp_buffer_get_payload (&rtp);
-    gst_buffer_extract (buffer, 0, payload, payload_len);
-    gst_rtp_buffer_unmap (&rtp);
-
-    gst_buffer_unref (buffer);
+    data.pay = baseaudiopayload;
+    data.outbuf = outbuf;
+    gst_buffer_foreach_meta (buffer, foreach_metadata, &data);
+    outbuf = gst_buffer_append (outbuf, buffer);
 
     GST_DEBUG_OBJECT (baseaudiopayload, "Pushing buffer %p", outbuf);
     ret = gst_rtp_base_payload_push (basepayload, outbuf);
@@ -609,11 +631,17 @@ gst_rtp_base_audio_payload_flush (GstRTPBaseAudioPayload * baseaudiopayload,
         timestamp);
   } else {
     GstBuffer *paybuf;
+    CopyMetaData data;
+
 
     /* create buffer to hold the payload */
-    outbuf = gst_rtp_buffer_new_allocate (0, 0, 0);
+    outbuf = gst_rtp_base_payload_allocate_output_buffer (basepayload, 0, 0, 0);
 
     paybuf = gst_adapter_take_buffer_fast (adapter, payload_len);
+
+    data.pay = baseaudiopayload;
+    data.outbuf = outbuf;
+    gst_buffer_foreach_meta (paybuf, foreach_metadata, &data);
     outbuf = gst_buffer_append (outbuf, paybuf);
 
     /* set metadata */
@@ -632,8 +660,8 @@ gst_rtp_base_audio_payload_flush (GstRTPBaseAudioPayload * baseaudiopayload,
  * mtu and min/max_ptime values. We cache those so that we don't have to redo
  * all the calculations */
 static gboolean
-gst_rtp_base_audio_payload_get_lengths (GstRTPBasePayload *
-    basepayload, guint * min_payload_len, guint * max_payload_len,
+gst_rtp_base_audio_payload_get_lengths (GstRTPBasePayload * basepayload,
+    guint csrc_count, guint * min_payload_len, guint * max_payload_len,
     guint * align)
 {
   GstRTPBaseAudioPayload *payload;
@@ -651,13 +679,16 @@ gst_rtp_base_audio_payload_get_lengths (GstRTPBasePayload *
 
   mtu = GST_RTP_BASE_PAYLOAD_MTU (payload);
 
-  /* check cached values */
+  /* check cached values. Since csrc_count may vary for each packet, we only
+   * check whether the new value exceeds the cached value and thus result in
+   * smaller payload. */
   if (G_LIKELY (priv->cached_mtu == mtu
           && priv->cached_ptime_multiple ==
           basepayload->ptime_multiple
           && priv->cached_ptime == basepayload->ptime
           && priv->cached_max_ptime == basepayload->max_ptime
-          && priv->cached_min_ptime == basepayload->min_ptime)) {
+          && priv->cached_min_ptime == basepayload->min_ptime
+          && priv->cached_csrc_count >= csrc_count)) {
     /* if nothing changed, return cached values */
     *min_payload_len = priv->cached_min_length;
     *max_payload_len = priv->cached_max_length;
@@ -676,7 +707,7 @@ gst_rtp_base_audio_payload_get_lengths (GstRTPBasePayload *
     maxptime_octets = G_MAXUINT;
   }
   /* MTU max */
-  max_mtu = gst_rtp_buffer_calc_payload_len (mtu, 0, 0);
+  max_mtu = gst_rtp_buffer_calc_payload_len (mtu, 0, csrc_count);
   /* round down to alignment */
   max_mtu = ALIGN_DOWN (max_mtu, *align);
 
@@ -712,6 +743,7 @@ gst_rtp_base_audio_payload_get_lengths (GstRTPBasePayload *
   priv->cached_min_length = *min_payload_len;
   priv->cached_max_length = *max_payload_len;
   priv->cached_align = *align;
+  priv->cached_csrc_count = csrc_count;
 
   return TRUE;
 }
@@ -820,7 +852,7 @@ gst_rtp_base_audio_payload_handle_buffer (GstRTPBasePayload *
   payload = GST_RTP_BASE_AUDIO_PAYLOAD_CAST (basepayload);
   priv = payload->priv;
 
-  timestamp = GST_BUFFER_TIMESTAMP (buffer);
+  timestamp = GST_BUFFER_PTS (buffer);
   discont = GST_BUFFER_IS_DISCONT (buffer);
   if (discont) {
 
@@ -854,8 +886,9 @@ gst_rtp_base_audio_payload_handle_buffer (GstRTPBasePayload *
     }
   }
 
-  if (!gst_rtp_base_audio_payload_get_lengths (basepayload, &min_payload_len,
-          &max_payload_len, &align))
+  if (!gst_rtp_base_audio_payload_get_lengths (basepayload,
+          gst_rtp_base_payload_get_source_count (basepayload, buffer),
+          &min_payload_len, &max_payload_len, &align))
     goto config_error;
 
   GST_DEBUG_OBJECT (payload,
@@ -885,6 +918,7 @@ gst_rtp_base_audio_payload_handle_buffer (GstRTPBasePayload *
     GST_DEBUG_OBJECT (payload, "available now %u", available);
 
     /* as long as we have full frames */
+    /* TODO: Use buffer lists here */
     while (available >= min_payload_len) {
       /* get multiple of alignment */
       payload_len = MIN (max_payload_len, available);
