@@ -20,19 +20,21 @@
 
 /**
  * SECTION:gstrtpbasedepayload
+ * @title: GstRTPBaseDepayload
  * @short_description: Base class for RTP depayloader
  *
  * Provides a base class for RTP depayloaders
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "gstrtpbasedepayload.h"
+#include "gstrtpmeta.h"
 
 GST_DEBUG_CATEGORY_STATIC (rtpbasedepayload_debug);
 #define GST_CAT_DEFAULT (rtpbasedepayload_debug)
 
-#define GST_RTP_BASE_DEPAYLOAD_GET_PRIVATE(obj)  \
-   (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_RTP_BASE_DEPAYLOAD, GstRTPBaseDepayloadPrivate))
-
 struct _GstRTPBaseDepayloadPrivate
 {
   GstClockTime npt_start;
@@ -40,6 +42,7 @@ struct _GstRTPBaseDepayloadPrivate
   gdouble play_speed;
   gdouble play_scale;
   guint clock_base;
+  gboolean onvif_mode;
 
   gboolean discont;
   GstClockTime pts;
@@ -50,11 +53,18 @@ struct _GstRTPBaseDepayloadPrivate
   guint32 last_seqnum;
   guint32 last_rtptime;
   guint32 next_seqnum;
+  gint max_reorder;
 
   gboolean negotiated;
 
   GstCaps *last_caps;
   GstEvent *segment_event;
+  guint32 segment_seqnum;       /* Note: this is a GstEvent seqnum */
+
+  gboolean source_info;
+  GstBuffer *input_buffer;
+
+  GstFlowReturn process_flow_ret;
 };
 
 /* Filter signals and args */
@@ -64,10 +74,15 @@ enum
   LAST_SIGNAL
 };
 
+#define DEFAULT_SOURCE_INFO FALSE
+#define DEFAULT_MAX_REORDER 100
+
 enum
 {
   PROP_0,
   PROP_STATS,
+  PROP_SOURCE_INFO,
+  PROP_MAX_REORDER,
   PROP_LAST
 };
 
@@ -93,6 +108,8 @@ static gboolean gst_rtp_base_depayload_handle_event (GstRTPBaseDepayload *
     filter, GstEvent * event);
 
 static GstElementClass *parent_class = NULL;
+static gint private_offset = 0;
+
 static void gst_rtp_base_depayload_class_init (GstRTPBaseDepayloadClass *
     klass);
 static void gst_rtp_base_depayload_init (GstRTPBaseDepayload * rtpbasepayload,
@@ -117,14 +134,26 @@ gst_rtp_base_depayload_get_type (void)
       0,
       (GInstanceInitFunc) gst_rtp_base_depayload_init,
     };
+    GType _type;
 
-    g_once_init_leave ((gsize *) & rtp_base_depayload_type,
-        g_type_register_static (GST_TYPE_ELEMENT, "GstRTPBaseDepayload",
-            &rtp_base_depayload_info, G_TYPE_FLAG_ABSTRACT));
+    _type = g_type_register_static (GST_TYPE_ELEMENT, "GstRTPBaseDepayload",
+        &rtp_base_depayload_info, G_TYPE_FLAG_ABSTRACT);
+
+    private_offset =
+        g_type_add_instance_private (_type,
+        sizeof (GstRTPBaseDepayloadPrivate));
+
+    g_once_init_leave ((gsize *) & rtp_base_depayload_type, _type);
   }
   return rtp_base_depayload_type;
 }
 
+static inline GstRTPBaseDepayloadPrivate *
+gst_rtp_base_depayload_get_instance_private (GstRTPBaseDepayload * self)
+{
+  return (G_STRUCT_MEMBER_P (self, private_offset));
+}
+
 static void
 gst_rtp_base_depayload_class_init (GstRTPBaseDepayloadClass * klass)
 {
@@ -135,7 +164,8 @@ gst_rtp_base_depayload_class_init (GstRTPBaseDepayloadClass * klass)
   gstelement_class = (GstElementClass *) klass;
   parent_class = g_type_class_peek_parent (klass);
 
-  g_type_class_add_private (klass, sizeof (GstRTPBaseDepayloadPrivate));
+  if (private_offset != 0)
+    g_type_class_adjust_private_offset (klass, &private_offset);
 
   gobject_class->finalize = gst_rtp_base_depayload_finalize;
   gobject_class->set_property = gst_rtp_base_depayload_set_property;
@@ -150,60 +180,49 @@ gst_rtp_base_depayload_class_init (GstRTPBaseDepayloadClass * klass)
    * application/x-rtp-depayload-stats containing the following fields relating to
    * the last processed buffer and current state of the stream being depayloaded:
    *
-   * <variablelist>
-   *   <varlistentry>
-   *     <term>clock-rate</term>
-   *     <listitem><para>#G_TYPE_UINT, clock-rate of the
-   *     stream</para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>npt-start</term>
-   *     <listitem><para>#G_TYPE_UINT64, time of playback start
-   *     </para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>npt-stop</term>
-   *     <listitem><para>#G_TYPE_UINT64, time of playback stop
-   *     </para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>play-speed</term>
-   *     <listitem><para>#G_TYPE_DOUBLE, the playback speed
-   *     </para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>play-scale</term>
-   *     <listitem><para>#G_TYPE_DOUBLE, the playback scale
-   *     </para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>running-time-dts</term>
-   *     <listitem><para>#G_TYPE_UINT64, the last running-time of the
+   *   * `clock-rate`: #G_TYPE_UINT, clock-rate of the stream
+   *   * `npt-start`: #G_TYPE_UINT64, time of playback start
+   *   * `npt-stop`: #G_TYPE_UINT64, time of playback stop
+   *   * `play-speed`: #G_TYPE_DOUBLE, the playback speed
+   *   * `play-scale`: #G_TYPE_DOUBLE, the playback scale
+   *   * `running-time-dts`: #G_TYPE_UINT64, the last running-time of the
    *      last DTS
-   *     </para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>running-time-pts</term>
-   *     <listitem><para>#G_TYPE_UINT64, the last running-time of the
+   *   * `running-time-pts`: #G_TYPE_UINT64, the last running-time of the
    *      last PTS
-   *     </para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>seqnum</term>
-   *     <listitem><para>#G_TYPE_UINT, the last seen seqnum
-   *     </para></listitem>
-   *   </varlistentry>
-   *   <varlistentry>
-   *     <term>timestamp</term>
-   *     <listitem><para>#G_TYPE_UINT, the last seen RTP timestamp
-   *     </para></listitem>
-   *   </varlistentry>
-   * </variablelist>
+   *   * `seqnum`: #G_TYPE_UINT, the last seen seqnum
+   *   * `timestamp`: #G_TYPE_UINT, the last seen RTP timestamp
    **/
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_STATS,
       g_param_spec_boxed ("stats", "Statistics", "Various statistics",
           GST_TYPE_STRUCTURE, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstRTPBaseDepayload:source-info:
+   *
+   * Add RTP source information found in RTP header as meta to output buffer.
+   *
+   * Since: 1.16
+   **/
+  g_object_class_install_property (gobject_class, PROP_SOURCE_INFO,
+      g_param_spec_boolean ("source-info", "RTP source information",
+          "Add RTP source information as buffer meta",
+          DEFAULT_SOURCE_INFO, G_PARAM_READWRITE));
+
+  /**
+   * GstRTPBaseDepayload:max-reorder:
+   *
+   * Max seqnum reorder before the sender is assumed to have restarted.
+   *
+   * When max-reorder is set to 0 all reordered/duplicate packets are
+   * considered coming from a restarted sender.
+   *
+   * Since: 1.18
+   **/
+  g_object_class_install_property (gobject_class, PROP_MAX_REORDER,
+      g_param_spec_int ("max-reorder", "Max Reorder",
+          "Max seqnum reorder before assuming sender has restarted",
+          0, G_MAXINT, DEFAULT_MAX_REORDER, G_PARAM_READWRITE));
+
   gstelement_class->change_state = gst_rtp_base_depayload_change_state;
 
   klass->packet_lost = gst_rtp_base_depayload_packet_lost;
@@ -220,7 +239,8 @@ gst_rtp_base_depayload_init (GstRTPBaseDepayload * filter,
   GstPadTemplate *pad_template;
   GstRTPBaseDepayloadPrivate *priv;
 
-  priv = GST_RTP_BASE_DEPAYLOAD_GET_PRIVATE (filter);
+  priv = gst_rtp_base_depayload_get_instance_private (filter);
+
   filter->priv = priv;
 
   GST_DEBUG_OBJECT (filter, "init");
@@ -248,9 +268,12 @@ gst_rtp_base_depayload_init (GstRTPBaseDepayload * filter,
   priv->play_speed = 1.0;
   priv->play_scale = 1.0;
   priv->clock_base = -1;
+  priv->onvif_mode = FALSE;
   priv->dts = -1;
   priv->pts = -1;
   priv->duration = -1;
+  priv->source_info = DEFAULT_SOURCE_INFO;
+  priv->max_reorder = DEFAULT_MAX_REORDER;
 
   gst_segment_init (&filter->segment, GST_FORMAT_UNDEFINED);
 }
@@ -288,6 +311,16 @@ gst_rtp_base_depayload_setcaps (GstRTPBaseDepayload * filter, GstCaps * caps)
 
   caps_struct = gst_caps_get_structure (caps, 0);
 
+  value = gst_structure_get_value (caps_struct, "onvif-mode");
+  if (value && G_VALUE_HOLDS_BOOLEAN (value))
+    priv->onvif_mode = g_value_get_boolean (value);
+  else
+    priv->onvif_mode = FALSE;
+  GST_DEBUG_OBJECT (filter, "Onvif mode: %d", priv->onvif_mode);
+
+  if (priv->onvif_mode)
+    filter->need_newsegment = FALSE;
+
   /* get other values for newsegment */
   value = gst_structure_get_value (caps_struct, "npt-start");
   if (value && G_VALUE_HOLDS_UINT64 (value))
@@ -355,7 +388,6 @@ gst_rtp_base_depayload_handle_buffer (GstRTPBaseDepayload * filter,
       GstRTPBuffer * rtp_buffer);
   GstBuffer *(*process_func) (GstRTPBaseDepayload * base, GstBuffer * in);
   GstRTPBaseDepayloadPrivate *priv;
-  GstFlowReturn ret = GST_FLOW_OK;
   GstBuffer *out_buf;
   guint32 ssrc;
   guint16 seqnum;
@@ -365,6 +397,7 @@ gst_rtp_base_depayload_handle_buffer (GstRTPBaseDepayload * filter,
   GstRTPBuffer rtp = { NULL };
 
   priv = filter->priv;
+  priv->process_flow_ret = GST_FLOW_OK;
 
   process_func = bclass->process;
   process_rtp_packet_func = bclass->process_rtp_packet;
@@ -417,15 +450,19 @@ gst_rtp_base_depayload_handle_buffer (GstRTPBaseDepayload * filter,
           GST_LOG_OBJECT (filter, "%d missing packets", gap);
           discont = TRUE;
         } else {
-          /* seqnum < next_seqnum, we have seen this packet before or the sender
-           * could be restarted. If the packet is not too old, we throw it away as
-           * a duplicate, otherwise we mark discont and continue. 100 misordered
-           * packets is a good threshold. See also RFC 4737. */
-          if (gap < 100)
+          /* seqnum < next_seqnum, we have seen this packet before, have a
+           * reordered packet or the sender could be restarted. If the packet
+           * is not too old, we throw it away as a duplicate. Otherwise we
+           * mark discont and continue assuming the sender has restarted. See
+           * also RFC 4737. */
+          if (gap <= priv->max_reorder) {
+            GST_WARNING_OBJECT (filter, "got old packet %u, expected %u, "
+                "gap %d <= max_reorder (%d), dropping!",
+                seqnum, priv->next_seqnum, gap, priv->max_reorder);
             goto dropping;
-
-          GST_LOG_OBJECT (filter,
-              "%d > 100, packet too old, sender likely restarted", gap);
+          }
+          GST_WARNING_OBJECT (filter, "got old packet %u, expected %u, "
+              "marking discont", seqnum, priv->next_seqnum);
           discont = TRUE;
         }
       }
@@ -462,6 +499,8 @@ gst_rtp_base_depayload_handle_buffer (GstRTPBaseDepayload * filter,
     filter->need_newsegment = FALSE;
   }
 
+  priv->input_buffer = in;
+
   if (process_rtp_packet_func != NULL) {
     out_buf = process_rtp_packet_func (filter, &rtp);
     gst_rtp_buffer_unmap (&rtp);
@@ -474,12 +513,16 @@ gst_rtp_base_depayload_handle_buffer (GstRTPBaseDepayload * filter,
 
   /* let's send it out to processing */
   if (out_buf) {
-    ret = gst_rtp_base_depayload_push (filter, out_buf);
+    if (priv->process_flow_ret == GST_FLOW_OK)
+      priv->process_flow_ret = gst_rtp_base_depayload_push (filter, out_buf);
+    else
+      gst_buffer_unref (out_buf);
   }
 
   gst_buffer_unref (in);
+  priv->input_buffer = NULL;
 
-  return ret;
+  return priv->process_flow_ret;
 
   /* ERRORS */
 not_negotiated:
@@ -507,7 +550,6 @@ invalid_buffer:
 dropping:
   {
     gst_rtp_buffer_unmap (&rtp);
-    GST_WARNING_OBJECT (filter, "%d <= 100, dropping old packet", gap);
     gst_buffer_unref (in);
     return GST_FLOW_OK;
   }
@@ -595,7 +637,7 @@ gst_rtp_base_depayload_handle_event (GstRTPBaseDepayload * filter,
       gst_segment_init (&filter->segment, GST_FORMAT_UNDEFINED);
       GST_OBJECT_UNLOCK (filter);
 
-      filter->need_newsegment = TRUE;
+      filter->need_newsegment = !filter->priv->onvif_mode;
       filter->priv->next_seqnum = -1;
       gst_event_replace (&filter->priv->segment_event, NULL);
       break;
@@ -611,16 +653,25 @@ gst_rtp_base_depayload_handle_event (GstRTPBaseDepayload * filter,
     }
     case GST_EVENT_SEGMENT:
     {
+      GstSegment segment;
+
       GST_OBJECT_LOCK (filter);
-      gst_event_copy_segment (event, &filter->segment);
-      if (filter->segment.format != GST_FORMAT_TIME)
-        GST_ERROR_OBJECT (filter,
-            "Non-TIME segments are not supported and will likely fail");
+      gst_event_copy_segment (event, &segment);
+
+      if (segment.format != GST_FORMAT_TIME) {
+        GST_ERROR_OBJECT (filter, "Segment with non-TIME format not supported");
+        res = FALSE;
+      }
+      filter->priv->segment_seqnum = gst_event_get_seqnum (event);
+      filter->segment = segment;
       GST_OBJECT_UNLOCK (filter);
 
-      /* don't pass the event downstream, we generate our own segment including
-       * the NTP time and other things we receive in caps */
-      forward = FALSE;
+      /* In ONVIF mode, upstream is expected to send us the correct segment */
+      if (!filter->priv->onvif_mode) {
+        /* don't pass the event downstream, we generate our own segment including
+         * the NTP time and other things we receive in caps */
+        forward = FALSE;
+      }
       break;
     }
     case GST_EVENT_CUSTOM_DOWNSTREAM:
@@ -720,11 +771,8 @@ create_segment_event (GstRTPBaseDepayload * filter, guint rtptime,
   if (position == -1)
     position = start;
 
-  if (G_LIKELY (filter->segment.format == GST_FORMAT_TIME))
-    running_time = gst_segment_to_running_time (&filter->segment,
-        GST_FORMAT_TIME, start);
-  else
-    running_time = 0;
+  running_time = gst_segment_to_running_time (&filter->segment,
+      GST_FORMAT_TIME, start);
 
   gst_segment_init (&segment, GST_FORMAT_TIME);
   segment.rate = priv->play_speed;
@@ -738,10 +786,54 @@ create_segment_event (GstRTPBaseDepayload * filter, guint rtptime,
   GST_DEBUG_OBJECT (filter, "Creating segment event %" GST_SEGMENT_FORMAT,
       &segment);
   event = gst_event_new_segment (&segment);
+  if (filter->priv->segment_seqnum != GST_SEQNUM_INVALID)
+    gst_event_set_seqnum (event, filter->priv->segment_seqnum);
 
   return event;
 }
 
+static gboolean
+foreach_metadata_drop (GstBuffer * buffer, GstMeta ** meta, gpointer user_data)
+{
+  GType drop_api_type = (GType) user_data;
+  const GstMetaInfo *info = (*meta)->info;
+
+  if (info->api == drop_api_type)
+    *meta = NULL;
+
+  return TRUE;
+}
+
+static void
+add_rtp_source_meta (GstBuffer * outbuf, GstBuffer * rtpbuf)
+{
+  GstRTPBuffer rtp = GST_RTP_BUFFER_INIT;
+  GstRTPSourceMeta *meta;
+  guint32 ssrc;
+  GType source_meta_api = gst_rtp_source_meta_api_get_type ();
+
+  if (!gst_rtp_buffer_map (rtpbuf, GST_MAP_READ, &rtp))
+    return;
+
+  ssrc = gst_rtp_buffer_get_ssrc (&rtp);
+
+  /* remove any pre-existing source-meta */
+  gst_buffer_foreach_meta (outbuf, foreach_metadata_drop,
+      (gpointer) source_meta_api);
+
+  meta = gst_buffer_add_rtp_source_meta (outbuf, &ssrc, NULL, 0);
+  if (meta != NULL) {
+    gint i;
+    gint csrc_count = gst_rtp_buffer_get_csrc_count (&rtp);
+    for (i = 0; i < csrc_count; i++) {
+      guint32 csrc = gst_rtp_buffer_get_csrc (&rtp, i);
+      gst_rtp_source_meta_append_csrc (meta, &csrc, 1);
+    }
+  }
+
+  gst_rtp_buffer_unmap (&rtp);
+}
+
 static gboolean
 set_headers (GstBuffer ** buffer, guint idx, GstRTPBaseDepayload * depayload)
 {
@@ -754,7 +846,7 @@ set_headers (GstBuffer ** buffer, guint idx, GstRTPBaseDepayload * depayload)
   dts = GST_BUFFER_DTS (*buffer);
   duration = GST_BUFFER_DURATION (*buffer);
 
-  /* apply last incomming timestamp and duration to outgoing buffer if
+  /* apply last incoming timestamp and duration to outgoing buffer if
    * not otherwise set. */
   if (!GST_CLOCK_TIME_IS_VALID (pts))
     GST_BUFFER_PTS (*buffer) = priv->pts;
@@ -774,6 +866,9 @@ set_headers (GstBuffer ** buffer, guint idx, GstRTPBaseDepayload * depayload)
   priv->dts = GST_CLOCK_TIME_NONE;
   priv->duration = GST_CLOCK_TIME_NONE;
 
+  if (priv->source_info && priv->input_buffer)
+    add_rtp_source_meta (*buffer, priv->input_buffer);
+
   return TRUE;
 }
 
@@ -807,7 +902,7 @@ gst_rtp_base_depayload_prepare_push (GstRTPBaseDepayload * filter,
  * Push @out_buf to the peer of @filter. This function takes ownership of
  * @out_buf.
  *
- * This function will by default apply the last incomming timestamp on
+ * This function will by default apply the last incoming timestamp on
  * the outgoing buffer when it didn't have a timestamp already.
  *
  * Returns: a #GstFlowReturn.
@@ -824,6 +919,9 @@ gst_rtp_base_depayload_push (GstRTPBaseDepayload * filter, GstBuffer * out_buf)
   else
     gst_buffer_unref (out_buf);
 
+  if (res != GST_FLOW_OK)
+    filter->priv->process_flow_ret = res;
+
   return res;
 }
 
@@ -850,6 +948,9 @@ gst_rtp_base_depayload_push_list (GstRTPBaseDepayload * filter,
   else
     gst_buffer_list_unref (out_list);
 
+  if (res != GST_FLOW_OK)
+    filter->priv->process_flow_ret = res;
+
   return res;
 }
 
@@ -862,6 +963,8 @@ gst_rtp_base_depayload_packet_lost (GstRTPBaseDepayload * filter,
   GstClockTime timestamp, duration;
   GstEvent *sevent;
   const GstStructure *s;
+  gboolean might_have_been_fec;
+  gboolean res = TRUE;
 
   s = gst_event_get_structure (event);
 
@@ -876,10 +979,23 @@ gst_rtp_base_depayload_packet_lost (GstRTPBaseDepayload * filter,
     return FALSE;
   }
 
-  /* send GAP event */
-  sevent = gst_event_new_gap (timestamp, duration);
+  sevent = gst_pad_get_sticky_event (filter->srcpad, GST_EVENT_SEGMENT, 0);
+  if (G_UNLIKELY (!sevent)) {
+    /* Typically happens if lost event arrives before first buffer */
+    GST_DEBUG_OBJECT (filter,
+        "Ignore packet loss because segment event missing");
+    return FALSE;
+  }
+  gst_event_unref (sevent);
 
-  return gst_pad_push_event (filter->srcpad, sevent);
+  if (!gst_structure_get_boolean (s, "might-have-been-fec",
+          &might_have_been_fec) || !might_have_been_fec) {
+    /* send GAP event */
+    sevent = gst_event_new_gap (timestamp, duration);
+    res = gst_pad_push_event (filter->srcpad, sevent);
+  }
+
+  return res;
 }
 
 static GstStateChangeReturn
@@ -903,9 +1019,11 @@ gst_rtp_base_depayload_change_state (GstElement * element,
       priv->play_speed = 1.0;
       priv->play_scale = 1.0;
       priv->clock_base = -1;
+      priv->onvif_mode = FALSE;
       priv->next_seqnum = -1;
       priv->negotiated = FALSE;
       priv->discont = FALSE;
+      priv->segment_seqnum = GST_SEQNUM_INVALID;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -967,7 +1085,20 @@ static void
 gst_rtp_base_depayload_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
+  GstRTPBaseDepayload *depayload;
+  GstRTPBaseDepayloadPrivate *priv;
+
+  depayload = GST_RTP_BASE_DEPAYLOAD (object);
+  priv = depayload->priv;
+
   switch (prop_id) {
+    case PROP_SOURCE_INFO:
+      gst_rtp_base_depayload_set_source_info_enabled (depayload,
+          g_value_get_boolean (value));
+      break;
+    case PROP_MAX_REORDER:
+      priv->max_reorder = g_value_get_int (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -979,16 +1110,57 @@ gst_rtp_base_depayload_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
   GstRTPBaseDepayload *depayload;
+  GstRTPBaseDepayloadPrivate *priv;
 
   depayload = GST_RTP_BASE_DEPAYLOAD (object);
+  priv = depayload->priv;
 
   switch (prop_id) {
     case PROP_STATS:
       g_value_take_boxed (value,
           gst_rtp_base_depayload_create_stats (depayload));
       break;
+    case PROP_SOURCE_INFO:
+      g_value_set_boolean (value,
+          gst_rtp_base_depayload_is_source_info_enabled (depayload));
+      break;
+    case PROP_MAX_REORDER:
+      g_value_set_int (value, priv->max_reorder);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
+
+/**
+ * gst_rtp_base_depayload_set_source_info_enabled:
+ * @depayload: a #GstRTPBaseDepayload
+ * @enable: whether to add meta about RTP sources to buffer
+ *
+ * Enable or disable adding #GstRTPSourceMeta to depayloaded buffers.
+ *
+ * Since: 1.16
+ **/
+void
+gst_rtp_base_depayload_set_source_info_enabled (GstRTPBaseDepayload * depayload,
+    gboolean enable)
+{
+  depayload->priv->source_info = enable;
+}
+
+/**
+ * gst_rtp_base_depayload_is_source_info_enabled:
+ * @depayload: a #GstRTPBaseDepayload
+ *
+ * Queries whether #GstRTPSourceMeta will be added to depayloaded buffers.
+ *
+ * Returns: %TRUE if source-info is enabled.
+ *
+ * Since: 1.16
+ **/
+gboolean
+gst_rtp_base_depayload_is_source_info_enabled (GstRTPBaseDepayload * depayload)
+{
+  return depayload->priv->source_info;
+}
