@@ -1314,6 +1314,94 @@ gst_rtp_buffer_ext_timestamp (guint64 * exttimestamp, guint32 timestamp)
   return result;
 }
 
+
+static gboolean
+_get_extension_onebyte_header (const guint8 * pdata, guint len,
+    guint16 bit_pattern, guint8 id, guint nth, gpointer * data, guint * size)
+{
+  gulong offset = 0;
+  guint count = 0;
+
+  g_return_val_if_fail (id > 0 && id < 15, FALSE);
+
+  if (bit_pattern != 0xBEDE)
+    return FALSE;
+
+  for (;;) {
+    guint8 read_id, read_len;
+
+    if (offset + 1 >= len)
+      break;
+
+    read_id = GST_READ_UINT8 (pdata + offset) >> 4;
+    read_len = (GST_READ_UINT8 (pdata + offset) & 0x0F) + 1;
+    offset += 1;
+
+    /* ID 0 means its padding, skip */
+    if (read_id == 0)
+      continue;
+
+    /* ID 15 is special and means we should stop parsing */
+    if (read_id == 15)
+      break;
+
+    /* Ignore extension headers where the size does not fit */
+    if (offset + read_len > len)
+      break;
+
+    /* If we have the right one */
+    if (id == read_id) {
+      if (nth == count) {
+        if (data)
+          *data = (gpointer) & pdata[offset];
+        if (size)
+          *size = read_len;
+
+        return TRUE;
+      }
+
+      count++;
+    }
+    offset += read_len;
+
+    if (offset >= len)
+      break;
+  }
+
+  return FALSE;
+}
+
+
+/**
+ * gst_rtp_buffer_get_extension_onebyte_header_from_bytes:
+ * @bytes: #GBytes
+ * @bit_pattern: The bit-pattern. Anything but 0xBEDE is rejected.
+ * @id: The ID of the header extension to be read (between 1 and 14).
+ * @nth: Read the nth extension packet with the requested ID
+ * @data: (out) (array length=size) (element-type guint8) (transfer none):
+ *   location for data
+ * @size: (out): the size of the data in bytes
+ *
+ * Similar to gst_rtp_buffer_get_extension_onebyte_header, but working
+ * on the #GBytes you get from gst_rtp_buffer_get_extension_bytes.
+ * Parses RFC 5285 style header extensions with a one byte header. It will
+ * return the nth extension with the requested id.
+ *
+ * Returns: TRUE if @bytes had the requested header extension
+ *
+ * Since: 1.18
+ */
+gboolean
+gst_rtp_buffer_get_extension_onebyte_header_from_bytes (GBytes * bytes,
+    guint16 bit_pattern, guint8 id, guint nth, gpointer * data, guint * size)
+{
+  const guint8 *pdata = g_bytes_get_data (bytes, NULL);
+  gsize len = g_bytes_get_size (bytes);
+  return _get_extension_onebyte_header (pdata, len, bit_pattern, id, nth, data,
+      size);
+}
+
+
 /**
  * gst_rtp_buffer_get_extension_onebyte_header:
  * @rtp: the RTP packet
@@ -1333,65 +1421,19 @@ gboolean
 gst_rtp_buffer_get_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
     guint nth, gpointer * data, guint * size)
 {
-  guint16 bits;
+  guint16 bit_pattern;
   guint8 *pdata;
   guint wordlen;
-  gulong offset = 0;
-  guint count = 0;
 
-  g_return_val_if_fail (id > 0 && id < 15, FALSE);
-
-  if (!gst_rtp_buffer_get_extension_data (rtp, &bits, (gpointer) & pdata,
+  if (!gst_rtp_buffer_get_extension_data (rtp, &bit_pattern, (gpointer) & pdata,
           &wordlen))
     return FALSE;
 
-  if (bits != 0xBEDE)
-    return FALSE;
-
-  for (;;) {
-    guint8 read_id, read_len;
-
-    if (offset + 1 >= wordlen * 4)
-      break;
-
-    read_id = GST_READ_UINT8 (pdata + offset) >> 4;
-    read_len = (GST_READ_UINT8 (pdata + offset) & 0x0F) + 1;
-    offset += 1;
-
-    /* ID 0 means its padding, skip */
-    if (read_id == 0)
-      continue;
-
-    /* ID 15 is special and means we should stop parsing */
-    if (read_id == 15)
-      break;
-
-    /* Ignore extension headers where the size does not fit */
-    if (offset + read_len > wordlen * 4)
-      break;
-
-    /* If we have the right one */
-    if (id == read_id) {
-      if (nth == count) {
-        if (data)
-          *data = pdata + offset;
-        if (size)
-          *size = read_len;
-
-        return TRUE;
-      }
-
-      count++;
-    }
-    offset += read_len;
-
-    if (offset >= wordlen * 4)
-      break;
-  }
-
-  return FALSE;
+  return _get_extension_onebyte_header (pdata, wordlen * 4, bit_pattern, id,
+      nth, data, size);
 }
 
+
 /**
  * gst_rtp_buffer_get_extension_twobytes_header:
  * @rtp: the RTP packet
@@ -1468,19 +1510,20 @@ gst_rtp_buffer_get_extension_twobytes_header (GstRTPBuffer * rtp,
   return FALSE;
 }
 
-static guint
-get_onebyte_header_end_offset (guint8 * pdata, guint wordlen)
+static gboolean
+get_onebyte_header_end_offset (guint8 * pdata, guint wordlen, guint * offset)
 {
-  guint offset = 0;
   guint bytelen = wordlen * 4;
   guint paddingcount = 0;
 
-  while (offset + 1 < bytelen) {
+  *offset = 0;
+
+  while (*offset + 1 < bytelen) {
     guint8 read_id, read_len;
 
-    read_id = GST_READ_UINT8 (pdata + offset) >> 4;
-    read_len = (GST_READ_UINT8 (pdata + offset) & 0x0F) + 1;
-    offset += 1;
+    read_id = GST_READ_UINT8 (pdata + *offset) >> 4;
+    read_len = (GST_READ_UINT8 (pdata + *offset) & 0x0F) + 1;
+    *offset += 1;
 
     /* ID 0 means its padding, skip */
     if (read_id == 0) {
@@ -1492,17 +1535,21 @@ get_onebyte_header_end_offset (guint8 * pdata, guint wordlen)
 
     /* ID 15 is special and means we should stop parsing */
     /* It also means we can't add an extra packet */
-    if (read_id == 15)
-      return 0;
+    if (read_id == 15) {
+      return FALSE;
+    }
 
     /* Ignore extension headers where the size does not fit */
-    if (offset + read_len > bytelen)
-      return 0;
+    if (*offset + read_len > bytelen) {
+      return FALSE;
+    }
 
-    offset += read_len;
+    *offset += read_len;
   }
 
-  return offset - paddingcount;
+  *offset -= paddingcount;
+
+  return TRUE;
 }
 
 /**
@@ -1516,7 +1563,7 @@ get_onebyte_header_end_offset (guint8 * pdata, guint wordlen)
  * RTP header. If there is already a RFC 5285 header extension with a one byte
  * header, the new extension will be appended.
  * It will not work if there is already a header extension that does not follow
- * the mecanism described in RFC 5285 or if there is a header extension with
+ * the mechanism described in RFC 5285 or if there is a header extension with
  * a two bytes header as described in RFC 5285. In that case, use
  * gst_rtp_buffer_add_extension_twobytes_header()
  *
@@ -1529,7 +1576,8 @@ gst_rtp_buffer_add_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
 {
   guint16 bits;
   guint8 *pdata = 0;
-  guint wordlen;
+  guint wordlen = 0;
+  guint wordlen_new;
   gboolean has_bit;
   guint extlen, offset = 0;
 
@@ -1544,17 +1592,17 @@ gst_rtp_buffer_add_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
     if (bits != 0xBEDE)
       return FALSE;
 
-    offset = get_onebyte_header_end_offset (pdata, wordlen);
-    if (offset == 0)
+    if (!get_onebyte_header_end_offset (pdata, wordlen, &offset))
       return FALSE;
   }
 
   /* the required size of the new extension data */
   extlen = offset + size + 1;
   /* calculate amount of words */
-  wordlen = extlen / 4 + ((extlen % 4) ? 1 : 0);
+  wordlen_new = extlen / 4 + ((extlen % 4) ? 1 : 0);
+  wordlen_new = MAX (wordlen_new, wordlen);
 
-  gst_rtp_buffer_set_extension_data (rtp, 0xBEDE, wordlen);
+  gst_rtp_buffer_set_extension_data (rtp, 0xBEDE, wordlen_new);
   gst_rtp_buffer_get_extension_data (rtp, &bits, (gpointer) & pdata, &wordlen);
 
   pdata += offset;
@@ -1615,7 +1663,7 @@ get_twobytes_header_end_offset (const guint8 * pdata, guint wordlen)
  * RTP header. If there is already a RFC 5285 header extension with a two bytes
  * header, the new extension will be appended.
  * It will not work if there is already a header extension that does not follow
- * the mecanism described in RFC 5285 or if there is a header extension with
+ * the mechanism described in RFC 5285 or if there is a header extension with
  * a one byte header as described in RFC 5285. In that case, use
  * gst_rtp_buffer_add_extension_onebyte_header()
  *
