@@ -76,10 +76,10 @@ GST_RTSP_API
 GstRTSPResult      gst_rtsp_connection_accept                 (GSocket * socket, GstRTSPConnection ** conn, GCancellable * cancellable);
 
 GST_RTSP_API
-GstRTSPResult      gst_rtsp_connection_connect                (GstRTSPConnection * conn, GTimeVal * timeout);
+GstRTSPResult      gst_rtsp_connection_connect_usec           (GstRTSPConnection * conn, gint64 timeout);
 
 GST_RTSP_API
-GstRTSPResult      gst_rtsp_connection_connect_with_response  (GstRTSPConnection * conn, GTimeVal * timeout, GstRTSPMessage * response);
+GstRTSPResult      gst_rtsp_connection_connect_with_response_usec (GstRTSPConnection * conn, gint64 timeout, GstRTSPMessage * response);
 
 GST_RTSP_API
 GstRTSPResult      gst_rtsp_connection_close                  (GstRTSPConnection *conn);
@@ -123,33 +123,37 @@ void                 gst_rtsp_connection_set_accept_certificate_func (GstRTSPCon
 /* sending/receiving raw bytes */
 
 GST_RTSP_API
-GstRTSPResult      gst_rtsp_connection_read           (GstRTSPConnection * conn, guint8 * data,
-                                                       guint size, GTimeVal * timeout);
+GstRTSPResult      gst_rtsp_connection_read_usec      (GstRTSPConnection * conn, guint8 * data,
+                                                       guint size, gint64 timeout);
 
 GST_RTSP_API
-GstRTSPResult      gst_rtsp_connection_write          (GstRTSPConnection * conn, const guint8 * data,
-                                                       guint size, GTimeVal * timeout);
+GstRTSPResult      gst_rtsp_connection_write_usec     (GstRTSPConnection * conn, const guint8 * data,
+                                                       guint size, gint64 timeout);
 
 /* sending/receiving messages */
 
 GST_RTSP_API
-GstRTSPResult      gst_rtsp_connection_send           (GstRTSPConnection *conn, GstRTSPMessage *message,
-                                                       GTimeVal *timeout);
+GstRTSPResult      gst_rtsp_connection_send_usec      (GstRTSPConnection *conn, GstRTSPMessage *message,
+                                                       gint64 timeout);
 
 GST_RTSP_API
-GstRTSPResult      gst_rtsp_connection_receive        (GstRTSPConnection *conn, GstRTSPMessage *message,
-                                                       GTimeVal *timeout);
+GstRTSPResult      gst_rtsp_connection_send_messages_usec (GstRTSPConnection *conn, GstRTSPMessage *messages, guint n_messages,
+                                                       gint64 timeout);
+
+GST_RTSP_API
+GstRTSPResult      gst_rtsp_connection_receive_usec    (GstRTSPConnection *conn, GstRTSPMessage *message,
+                                                       gint64 timeout);
 
 /* status management */
 
 GST_RTSP_API
-GstRTSPResult      gst_rtsp_connection_poll           (GstRTSPConnection *conn, GstRTSPEvent events,
-                                                       GstRTSPEvent *revents, GTimeVal *timeout);
+GstRTSPResult      gst_rtsp_connection_poll_usec      (GstRTSPConnection *conn, GstRTSPEvent events,
+                                                       GstRTSPEvent *revents, gint64 timeout);
 
 /* reset the timeout */
 
 GST_RTSP_API
-GstRTSPResult      gst_rtsp_connection_next_timeout   (GstRTSPConnection *conn, GTimeVal *timeout);
+gint64             gst_rtsp_connection_next_timeout_usec (GstRTSPConnection *conn);
 
 GST_RTSP_API
 GstRTSPResult      gst_rtsp_connection_reset_timeout  (GstRTSPConnection *conn);
@@ -185,6 +189,11 @@ GST_RTSP_API
 GstRTSPResult      gst_rtsp_connection_set_qos_dscp   (GstRTSPConnection *conn,
                                                        guint qos_dscp);
 
+/* Content-Length limit */
+GST_RTSP_API
+void               gst_rtsp_connection_set_content_length_limit (GstRTSPConnection *conn,
+                                                                 guint limit);
+
 /* accessors */
 
 GST_RTSP_API
@@ -241,18 +250,18 @@ typedef struct _GstRTSPWatch GstRTSPWatch;
  * @message_received: callback when a message was received
  * @message_sent: callback when a message was sent
  * @closed: callback when the connection is closed
- * @error: callback when an error occured
+ * @error: callback when an error occurred
  * @tunnel_start: a client started a tunneled connection. The tunnelid of the
  *   connection must be saved.
  * @tunnel_complete: a client finished a tunneled connection. In this callback
  *   you usually pair the tunnelid of this connection with the saved one using
  *   gst_rtsp_connection_do_tunnel().
- * @error_full: callback when an error occured with more information than
+ * @error_full: callback when an error occurred with more information than
  *   the @error callback.
  * @tunnel_lost: callback when the post connection of a tunnel is closed.
  * @tunnel_http_response: callback when an HTTP response to the GET request
  *   is about to be sent for a tunneled connection. The response can be
- *   modified in the callback. Since 1.4.
+ *   modified in the callback. Since: 1.4.
  *
  * Callback functions from a #GstRTSPWatch.
  */
@@ -314,12 +323,67 @@ GstRTSPResult      gst_rtsp_watch_send_message       (GstRTSPWatch *watch,
                                                       guint *id);
 
 GST_RTSP_API
-GstRTSPResult      gst_rtsp_watch_wait_backlog       (GstRTSPWatch * watch,
-                                                      GTimeVal *timeout);
+GstRTSPResult      gst_rtsp_watch_send_messages      (GstRTSPWatch *watch,
+                                                      GstRTSPMessage *messages,
+                                                      guint n_messages,
+                                                      guint *id);
+
+GST_RTSP_API
+GstRTSPResult      gst_rtsp_watch_wait_backlog_usec  (GstRTSPWatch * watch,
+                                                      gint64 timeout);
 
 GST_RTSP_API
 void               gst_rtsp_watch_set_flushing       (GstRTSPWatch * watch,
                                                       gboolean flushing);
+
+#ifndef GST_DISABLE_DEPRECATED
+
+/* Deprecated */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
+GST_RTSP_DEPRECATED_FOR (gst_rtsp_connection_connect_usec)
+GstRTSPResult      gst_rtsp_connection_connect                (GstRTSPConnection * conn, GTimeVal * timeout);
+
+GST_RTSP_DEPRECATED_FOR (gst_rtsp_connection_connect_with_response_usec)
+GstRTSPResult      gst_rtsp_connection_connect_with_response  (GstRTSPConnection * conn, GTimeVal * timeout, GstRTSPMessage * response);
+
+
+GST_RTSP_DEPRECATED_FOR (gst_rtsp_connection_read_usec)
+GstRTSPResult      gst_rtsp_connection_read           (GstRTSPConnection * conn, guint8 * data,
+                                                       guint size, GTimeVal * timeout);
+
+
+GST_RTSP_DEPRECATED_FOR (gst_rtsp_connection_write_usec)
+GstRTSPResult      gst_rtsp_connection_write          (GstRTSPConnection * conn, const guint8 * data,
+                                                       guint size, GTimeVal * timeout);
+
+GST_RTSP_DEPRECATED_FOR (gst_rtsp_connection_send_usec)
+GstRTSPResult      gst_rtsp_connection_send           (GstRTSPConnection *conn, GstRTSPMessage *message,
+                                                       GTimeVal *timeout);
+
+GST_RTSP_DEPRECATED_FOR (gst_rtsp_connection_send_messages_usec)
+GstRTSPResult      gst_rtsp_connection_send_messages  (GstRTSPConnection *conn, GstRTSPMessage *messages, guint n_messages,
+                                                       GTimeVal *timeout);
+
+GST_RTSP_DEPRECATED_FOR (gst_rtsp_connection_receive_usec)
+GstRTSPResult      gst_rtsp_connection_receive        (GstRTSPConnection *conn, GstRTSPMessage *message,
+                                                       GTimeVal *timeout);
+
+GST_RTSP_DEPRECATED_FOR (gst_rtsp_connection_poll_usec)
+GstRTSPResult      gst_rtsp_connection_poll           (GstRTSPConnection *conn, GstRTSPEvent events,
+                                                       GstRTSPEvent *revents, GTimeVal *timeout);
+
+GST_RTSP_DEPRECATED_FOR (gst_rtsp_connection_next_timeout_usec)
+GstRTSPResult      gst_rtsp_connection_next_timeout   (GstRTSPConnection *conn, GTimeVal *timeout);
+
+GST_RTSP_DEPRECATED_FOR (gst_rtsp_watch_wait_backlog_usec)
+GstRTSPResult      gst_rtsp_watch_wait_backlog       (GstRTSPWatch * watch,
+                                                      GTimeVal *timeout);
+
+G_GNUC_END_IGNORE_DEPRECATIONS
+
+#endif /* GST_DISABLE_DEPRECATED */
+
 G_END_DECLS
 
 #endif /* __GST_RTSP_CONNECTION_H__ */
