@@ -68,6 +68,7 @@ enum
 struct _GstVideoAggregatorPadPrivate
 {
   GstBuffer *buffer;
+  GstCaps *caps;
   GstVideoFrame prepared_frame;
 
   /* properties */
@@ -83,6 +84,7 @@ struct _GstVideoAggregatorPadPrivate
   GstClockTime end_time;
 
   GstVideoInfo pending_vinfo;
+  GstCaps *pending_caps;
 };
 
 
@@ -161,6 +163,7 @@ _flush_pad (GstAggregatorPad * aggpad, GstAggregator * aggregator)
 
   gst_video_aggregator_reset_qos (vagg);
   gst_buffer_replace (&pad->priv->buffer, NULL);
+  gst_caps_replace (&pad->priv->caps, NULL);
   pad->priv->start_time = -1;
   pad->priv->end_time = -1;
 
@@ -221,9 +224,8 @@ gst_video_aggregator_peek_next_sample (GstAggregator * agg,
   GstSample *res = NULL;
 
   if (vaggpad->priv->buffer) {
-    GstCaps *caps = gst_pad_get_current_caps (GST_PAD (aggpad));
-    res = gst_sample_new (vaggpad->priv->buffer, caps, &aggpad->segment, NULL);
-    gst_caps_unref (caps);
+    res = gst_sample_new (vaggpad->priv->buffer, vaggpad->priv->caps,
+        &aggpad->segment, NULL);
   }
 
   return res;
@@ -399,6 +401,9 @@ enum
 
 struct _GstVideoAggregatorConvertPadPrivate
 {
+  /* The following fields are only used from the aggregate thread and when
+   * initializing / finalizing */
+
   /* Converter, if NULL no conversion is done */
   GstVideoConverter *convert;
 
@@ -406,6 +411,8 @@ struct _GstVideoAggregatorConvertPadPrivate
   GstVideoInfo conversion_info;
   GstBuffer *converted_buffer;
 
+  /* The following fields are accessed from the property setters / getters,
+   * and as such are protected with the object lock */
   GstStructure *converter_config;
   gboolean converter_config_changed;
 };
@@ -435,7 +442,9 @@ static void
 {
   GstVideoAggregatorConvertPad *pad = GST_VIDEO_AGGREGATOR_CONVERT_PAD (vpad);
 
+  GST_OBJECT_LOCK (pad);
   pad->priv->converter_config_changed = TRUE;
+  GST_OBJECT_UNLOCK (pad);
 }
 
 static gboolean
@@ -447,6 +456,7 @@ gst_video_aggregator_convert_pad_prepare_frame (GstVideoAggregatorPad * vpad,
   GstVideoFrame frame;
 
   /* Update/create converter as needed */
+  GST_OBJECT_LOCK (pad);
   if (pad->priv->converter_config_changed) {
     GstVideoAggregatorConvertPadClass *klass =
         GST_VIDEO_AGGREGATOR_CONVERT_PAD_GET_CLASS (pad);
@@ -474,13 +484,15 @@ gst_video_aggregator_convert_pad_prepare_frame (GstVideoAggregatorPad * vpad,
         return FALSE;
       }
 
-      GST_DEBUG_OBJECT (pad, "This pad will be converted from %d to %d",
-          GST_VIDEO_INFO_FORMAT (&vpad->info),
-          GST_VIDEO_INFO_FORMAT (&pad->priv->conversion_info));
+      GST_DEBUG_OBJECT (pad, "This pad will be converted from %s to %s",
+          gst_video_format_to_string (GST_VIDEO_INFO_FORMAT (&vpad->info)),
+          gst_video_format_to_string (GST_VIDEO_INFO_FORMAT (&pad->
+                  priv->conversion_info)));
     } else {
       GST_DEBUG_OBJECT (pad, "This pad will not need conversion");
     }
   }
+  GST_OBJECT_UNLOCK (pad);
 
   if (!gst_video_frame_map (&frame, &vpad->info, buffer, GST_MAP_READ)) {
     GST_WARNING_OBJECT (vagg, "Could not map input buffer");
@@ -693,7 +705,9 @@ void gst_video_aggregator_convert_pad_update_conversion_info
 {
   g_return_if_fail (GST_IS_VIDEO_AGGREGATOR_CONVERT_PAD (pad));
 
+  GST_OBJECT_LOCK (pad);
   pad->priv->converter_config_changed = TRUE;
+  GST_OBJECT_UNLOCK (pad);
 }
 
 /**************************************
@@ -737,6 +751,10 @@ struct _GstVideoAggregatorPrivate
   GstCaps *current_caps;
 
   gboolean live;
+
+  /* The (ordered) list of #GstVideoFormatInfo supported by the aggregation
+     method (from the srcpad template caps). */
+  GPtrArray *supported_formats;
 };
 
 /* Can't use the G_DEFINE_TYPE macros because we need the
@@ -778,6 +796,35 @@ gst_video_aggregator_get_instance_private (GstVideoAggregator * self)
   return (G_STRUCT_MEMBER_P (self, video_aggregator_private_offset));
 }
 
+static gboolean
+gst_video_aggregator_supports_format (GstVideoAggregator * vagg,
+    GstVideoFormat format)
+{
+  gint i;
+
+  for (i = 0; i < vagg->priv->supported_formats->len; i++) {
+    GstVideoFormatInfo *format_info = vagg->priv->supported_formats->pdata[i];
+
+    if (GST_VIDEO_FORMAT_INFO_FORMAT (format_info) == format)
+      return TRUE;
+  }
+
+  return FALSE;
+}
+
+static GstCaps *
+gst_video_aggregator_get_possible_caps_for_info (GstVideoInfo * info)
+{
+  GstStructure *s;
+  GstCaps *possible_caps = gst_video_info_to_caps (info);
+
+  s = gst_caps_get_structure (possible_caps, 0);
+  gst_structure_remove_fields (s, "width", "height", "framerate",
+      "pixel-aspect-ratio", "interlace-mode", NULL);
+
+  return possible_caps;
+}
+
 static void
 gst_video_aggregator_find_best_format (GstVideoAggregator * vagg,
     GstCaps * downstream_caps, GstVideoInfo * best_info,
@@ -787,13 +834,12 @@ gst_video_aggregator_find_best_format (GstVideoAggregator * vagg,
   GstCaps *possible_caps;
   GstVideoAggregatorPad *pad;
   gboolean need_alpha = FALSE;
-  gint best_format_number = 0;
+  gint best_format_number = 0, i;
   GHashTable *formats_table = g_hash_table_new (g_direct_hash, g_direct_equal);
 
   GST_OBJECT_LOCK (vagg);
   for (tmp = GST_ELEMENT (vagg)->sinkpads; tmp; tmp = tmp->next) {
-    GstStructure *s;
-    gint format_number;
+    gint format_number = 0;
 
     pad = tmp->data;
 
@@ -811,28 +857,29 @@ gst_video_aggregator_find_best_format (GstVideoAggregator * vagg,
     if (GST_VIDEO_INFO_FORMAT (&pad->info) == GST_VIDEO_FORMAT_UNKNOWN)
       continue;
 
-    possible_caps = gst_video_info_to_caps (&pad->info);
-
-    s = gst_caps_get_structure (possible_caps, 0);
-    gst_structure_remove_fields (s, "width", "height", "framerate",
-        "pixel-aspect-ratio", "interlace-mode", NULL);
-
     /* Can downstream accept this format ? */
-    if (!gst_caps_can_intersect (downstream_caps, possible_caps)) {
-      gst_caps_unref (possible_caps);
-      continue;
+    if (!GST_IS_VIDEO_AGGREGATOR_CONVERT_PAD (pad)) {
+      possible_caps =
+          gst_video_aggregator_get_possible_caps_for_info (&pad->info);
+      if (!gst_caps_can_intersect (downstream_caps, possible_caps)) {
+        gst_caps_unref (possible_caps);
+        continue;
+      }
     }
 
-    gst_caps_unref (possible_caps);
+    /* If the format is supported, consider it very high weight */
+    if (gst_video_aggregator_supports_format (vagg,
+            GST_VIDEO_INFO_FORMAT (&pad->info))) {
+      format_number =
+          GPOINTER_TO_INT (g_hash_table_lookup (formats_table,
+              GINT_TO_POINTER (GST_VIDEO_INFO_FORMAT (&pad->info))));
 
-    format_number =
-        GPOINTER_TO_INT (g_hash_table_lookup (formats_table,
-            GINT_TO_POINTER (GST_VIDEO_INFO_FORMAT (&pad->info))));
-    format_number += pad->info.width * pad->info.height;
+      format_number += pad->info.width * pad->info.height;
 
-    g_hash_table_replace (formats_table,
-        GINT_TO_POINTER (GST_VIDEO_INFO_FORMAT (&pad->info)),
-        GINT_TO_POINTER (format_number));
+      g_hash_table_replace (formats_table,
+          GINT_TO_POINTER (GST_VIDEO_INFO_FORMAT (&pad->info)),
+          GINT_TO_POINTER (format_number));
+    }
 
     /* If that pad is the first with alpha, set it as the new best format */
     if (!need_alpha && (pad->priv->needs_alpha
@@ -858,6 +905,41 @@ gst_video_aggregator_find_best_format (GstVideoAggregator * vagg,
   GST_OBJECT_UNLOCK (vagg);
 
   g_hash_table_unref (formats_table);
+
+  if (gst_video_aggregator_supports_format (vagg,
+          GST_VIDEO_INFO_FORMAT (best_info))) {
+    possible_caps = gst_video_aggregator_get_possible_caps_for_info (best_info);
+    if (gst_caps_can_intersect (downstream_caps, possible_caps)) {
+      gst_caps_unref (possible_caps);
+      return;
+    }
+    gst_caps_unref (possible_caps);
+  }
+
+  for (i = 0; i < vagg->priv->supported_formats->len; i++) {
+    GstVideoFormatInfo *format_info = vagg->priv->supported_formats->pdata[i];
+
+    if ((! !GST_VIDEO_FORMAT_INFO_HAS_ALPHA (format_info)) == (! !need_alpha)) {
+      gst_video_info_set_format (best_info, format_info->format,
+          best_info->width, best_info->height);
+      possible_caps =
+          gst_video_aggregator_get_possible_caps_for_info (best_info);
+
+      if (gst_caps_can_intersect (downstream_caps, possible_caps)) {
+        GST_INFO_OBJECT (vagg, "Using supported caps: %" GST_PTR_FORMAT,
+            possible_caps);
+        gst_caps_unref (possible_caps);
+
+        return;
+      }
+
+      gst_caps_unref (possible_caps);
+    }
+  }
+
+  GST_WARNING_OBJECT (vagg, "Nothing compatible with %" GST_PTR_FORMAT,
+      downstream_caps);
+  gst_video_info_init (best_info);
 }
 
 static GstCaps *
@@ -953,8 +1035,8 @@ gst_video_aggregator_default_update_caps (GstVideoAggregator * vagg,
   color_name = gst_video_colorimetry_to_string (&best_info.colorimetry);
 
   GST_DEBUG_OBJECT (vagg,
-      "The output format will now be : %d with chroma : %s and colorimetry %s",
-      best_format,
+      "The output format will now be : %s with chroma : %s and colorimetry %s",
+      gst_video_format_to_string (best_format),
       GST_STR_NULL (gst_video_chroma_to_string (best_info.chroma_site)),
       GST_STR_NULL (color_name));
 
@@ -981,29 +1063,6 @@ gst_video_aggregator_default_update_src_caps (GstAggregator * agg,
 {
   GstVideoAggregatorClass *vagg_klass = GST_VIDEO_AGGREGATOR_GET_CLASS (agg);
   GstVideoAggregator *vagg = GST_VIDEO_AGGREGATOR (agg);
-  gboolean at_least_one_pad_configured = FALSE;
-  GList *l;
-
-  GST_OBJECT_LOCK (vagg);
-  for (l = GST_ELEMENT (vagg)->sinkpads; l; l = l->next) {
-    GstVideoAggregatorPad *mpad = l->data;
-
-    if (GST_VIDEO_INFO_WIDTH (&mpad->info) == 0
-        || GST_VIDEO_INFO_HEIGHT (&mpad->info) == 0)
-      continue;
-
-    at_least_one_pad_configured = TRUE;
-  }
-  GST_OBJECT_UNLOCK (vagg);
-
-  if (!at_least_one_pad_configured) {
-    /* We couldn't decide the output video info because the sinkpads don't have
-     * all the caps yet, so we mark the pad as needing a reconfigure. This
-     * allows aggregate() to skip ahead a bit and try again later. */
-    GST_DEBUG_OBJECT (vagg, "Couldn't decide output video info");
-    gst_pad_mark_reconfigure (agg->srcpad);
-    return GST_AGGREGATOR_FLOW_NEED_DATA;
-  }
 
   g_assert (vagg_klass->update_caps);
 
@@ -1066,9 +1125,11 @@ gst_video_aggregator_default_negotiated_src_caps (GstAggregator * agg,
     gst_video_aggregator_reset_qos (vagg);
   }
 
+  GST_OBJECT_LOCK (vagg);
   vagg->info = info;
+  GST_OBJECT_UNLOCK (vagg);
 
-  finfo = vagg->info.finfo;
+  finfo = info.finfo;
 
   if (at_least_one_alpha && !(finfo->flags & GST_VIDEO_FORMAT_FLAG_ALPHA)) {
     GST_ELEMENT_ERROR (vagg, CORE, NEGOTIATION,
@@ -1089,7 +1150,7 @@ gst_video_aggregator_default_negotiated_src_caps (GstAggregator * agg,
 
     gst_aggregator_set_src_caps (agg, caps);
     latency = gst_util_uint64_scale (GST_SECOND,
-        GST_VIDEO_INFO_FPS_D (&vagg->info), GST_VIDEO_INFO_FPS_N (&vagg->info));
+        GST_VIDEO_INFO_FPS_D (&info), GST_VIDEO_INFO_FPS_N (&info));
     gst_aggregator_set_latency (agg, latency, latency);
   }
 
@@ -1170,12 +1231,14 @@ gst_video_aggregator_pad_sink_setcaps (GstPad * pad, GstObject * parent,
      * that this pad is getting configured; configure immediately to avoid
      * problems with the initial negotiation */
     vaggpad->info = info;
+    gst_caps_replace (&vaggpad->priv->caps, caps);
     gst_pad_mark_reconfigure (GST_AGGREGATOR_SRC_PAD (vagg));
   } else {
     /* this pad already had caps but received new ones; keep the new caps
      * pending until we pick the next buffer from the queue, otherwise we
      * might use an old buffer with the new caps and crash */
     vaggpad->priv->pending_vinfo = info;
+    gst_caps_replace (&vaggpad->priv->pending_caps, caps);
     GST_DEBUG_OBJECT (pad, "delaying caps change");
   }
   ret = TRUE;
@@ -1423,7 +1486,10 @@ gst_video_aggregator_reset (GstVideoAggregator * vagg)
   GstAggregator *agg = GST_AGGREGATOR (vagg);
   GList *l;
 
+  GST_OBJECT_LOCK (vagg);
   gst_video_info_init (&vagg->info);
+  GST_OBJECT_UNLOCK (vagg);
+
   vagg->priv->ts_offset = 0;
   vagg->priv->nframes = 0;
   vagg->priv->live = FALSE;
@@ -1437,6 +1503,7 @@ gst_video_aggregator_reset (GstVideoAggregator * vagg)
     GstVideoAggregatorPad *p = l->data;
 
     gst_buffer_replace (&p->priv->buffer, NULL);
+    gst_caps_replace (&p->priv->caps, NULL);
     p->priv->start_time = -1;
     p->priv->end_time = -1;
 
@@ -1448,7 +1515,7 @@ gst_video_aggregator_reset (GstVideoAggregator * vagg)
 static GstFlowReturn
 gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
     GstClockTime output_start_running_time,
-    GstClockTime output_end_running_time)
+    GstClockTime output_end_running_time, gboolean timeout)
 {
   GList *l;
   gboolean eos = TRUE;
@@ -1482,6 +1549,9 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
     if (buf) {
       GstClockTime start_time, end_time;
 
+    check_again:
+      GST_TRACE_OBJECT (pad, "Next buffer %" GST_PTR_FORMAT, buf);
+
       start_time = GST_BUFFER_TIMESTAMP (buf);
       if (start_time == -1) {
         gst_buffer_unref (buf);
@@ -1513,6 +1583,8 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
               "output_start_running_time.  Discarding old buffer");
           gst_buffer_replace (&pad->priv->buffer, buf);
           if (pad->priv->pending_vinfo.finfo) {
+            gst_caps_replace (&pad->priv->caps, pad->priv->pending_caps);
+            gst_caps_replace (&pad->priv->pending_caps, NULL);
             pad->info = pad->priv->pending_vinfo;
             need_reconfigure = TRUE;
             pad->priv->pending_vinfo.finfo = NULL;
@@ -1520,6 +1592,13 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
           gst_buffer_unref (buf);
           gst_aggregator_pad_drop_buffer (bpad);
           pad->priv->start_time = start_time;
+          if (timeout) {
+            /* If we're called for a timeout, we want to make sure we drain as
+             * much as possible any late data */
+            buf = gst_aggregator_pad_peek_buffer (bpad);
+            if (buf)
+              goto check_again;
+          }
           need_more_data = TRUE;
           continue;
         }
@@ -1527,6 +1606,8 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
         buf = gst_aggregator_pad_pop_buffer (bpad);
         gst_buffer_replace (&pad->priv->buffer, buf);
         if (pad->priv->pending_vinfo.finfo) {
+          gst_caps_replace (&pad->priv->caps, pad->priv->pending_caps);
+          gst_caps_replace (&pad->priv->pending_caps, NULL);
           pad->info = pad->priv->pending_vinfo;
           need_reconfigure = TRUE;
           pad->priv->pending_vinfo.finfo = NULL;
@@ -1587,6 +1668,8 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
             GST_TIME_ARGS (start_time));
         gst_buffer_replace (&pad->priv->buffer, buf);
         if (pad->priv->pending_vinfo.finfo) {
+          gst_caps_replace (&pad->priv->caps, pad->priv->pending_caps);
+          gst_caps_replace (&pad->priv->pending_caps, NULL);
           pad->info = pad->priv->pending_vinfo;
           need_reconfigure = TRUE;
           pad->priv->pending_vinfo.finfo = NULL;
@@ -1605,6 +1688,8 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
       } else {
         gst_buffer_replace (&pad->priv->buffer, buf);
         if (pad->priv->pending_vinfo.finfo) {
+          gst_caps_replace (&pad->priv->caps, pad->priv->pending_caps);
+          gst_caps_replace (&pad->priv->pending_caps, NULL);
           pad->info = pad->priv->pending_vinfo;
           need_reconfigure = TRUE;
           pad->priv->pending_vinfo.finfo = NULL;
@@ -1637,6 +1722,7 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
                   pad->priv->max_last_buffer_repeat) {
                 pad->priv->start_time = pad->priv->end_time = -1;
                 gst_buffer_replace (&pad->priv->buffer, NULL);
+                gst_caps_replace (&pad->priv->caps, NULL);
               }
             } else {
               pad->priv->start_time = pad->priv->end_time = -1;
@@ -1644,6 +1730,7 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
             need_more_data = TRUE;
           } else {
             gst_buffer_replace (&pad->priv->buffer, NULL);
+            gst_caps_replace (&pad->priv->caps, NULL);
             pad->priv->start_time = pad->priv->end_time = -1;
           }
         } else if (is_eos) {
@@ -1651,6 +1738,7 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
         }
       } else if (is_eos) {
         gst_buffer_replace (&pad->priv->buffer, NULL);
+        gst_caps_replace (&pad->priv->caps, NULL);
       } else if (pad->priv->start_time != -1) {
         /* When the current buffer didn't have a duration, but
          * max-last-buffer-repeat was set, we use start_time as
@@ -1662,6 +1750,7 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
                 pad->priv->max_last_buffer_repeat) {
               pad->priv->start_time = pad->priv->end_time = -1;
               gst_buffer_replace (&pad->priv->buffer, NULL);
+              gst_caps_replace (&pad->priv->caps, NULL);
             }
           }
         }
@@ -1917,7 +2006,7 @@ gst_video_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
   } else {
     flow_ret =
         gst_video_aggregator_fill_queues (vagg, output_start_running_time,
-        output_end_running_time);
+        output_end_running_time, timeout);
   }
 
   if (flow_ret == GST_AGGREGATOR_FLOW_NEED_DATA && !timeout) {
@@ -2313,6 +2402,8 @@ gst_video_aggregator_release_pad (GstElement * element, GstPad * pad)
     gst_video_aggregator_reset (vagg);
 
   gst_buffer_replace (&vaggpad->priv->buffer, NULL);
+  gst_caps_replace (&vaggpad->priv->caps, NULL);
+  gst_caps_replace (&vaggpad->priv->pending_caps, NULL);
 
   GST_ELEMENT_CLASS (gst_video_aggregator_parent_class)->release_pad
       (GST_ELEMENT (vagg), pad);
@@ -2483,10 +2574,7 @@ gst_video_aggregator_pad_sink_acceptcaps (GstPad * pad,
     GstVideoAggregator * vagg, GstCaps * caps)
 {
   gboolean ret;
-  GstCaps *modified_caps;
   GstCaps *accepted_caps;
-  GstCaps *template_caps;
-  gboolean had_current_caps = TRUE;
   gint i, n;
   GstStructure *s;
   GstAggregator *agg = GST_AGGREGATOR (vagg);
@@ -2495,12 +2583,8 @@ gst_video_aggregator_pad_sink_acceptcaps (GstPad * pad,
 
   accepted_caps = gst_pad_get_current_caps (GST_PAD (agg->srcpad));
 
-  template_caps = gst_pad_get_pad_template_caps (GST_PAD (agg->srcpad));
-
-  if (accepted_caps == NULL) {
-    accepted_caps = template_caps;
-    had_current_caps = FALSE;
-  }
+  if (accepted_caps == NULL)
+    accepted_caps = gst_pad_get_pad_template_caps (GST_PAD (agg->srcpad));
 
   accepted_caps = gst_caps_make_writable (accepted_caps);
 
@@ -2520,15 +2604,10 @@ gst_video_aggregator_pad_sink_acceptcaps (GstPad * pad,
     }
   }
 
-  modified_caps = gst_caps_intersect (accepted_caps, template_caps);
-
   ret = gst_caps_can_intersect (caps, accepted_caps);
   GST_DEBUG_OBJECT (pad, "%saccepted caps %" GST_PTR_FORMAT,
       (ret ? "" : "not "), caps);
   gst_caps_unref (accepted_caps);
-  gst_caps_unref (modified_caps);
-  if (had_current_caps)
-    gst_caps_unref (template_caps);
   return ret;
 }
 
@@ -2580,6 +2659,7 @@ gst_video_aggregator_finalize (GObject * o)
   GstVideoAggregator *vagg = GST_VIDEO_AGGREGATOR (o);
 
   g_mutex_clear (&vagg->priv->lock);
+  g_ptr_array_unref (vagg->priv->supported_formats);
 
   G_OBJECT_CLASS (gst_video_aggregator_parent_class)->finalize (o);
 }
@@ -2673,12 +2753,48 @@ static void
 gst_video_aggregator_init (GstVideoAggregator * vagg,
     GstVideoAggregatorClass * klass)
 {
-  vagg->priv = gst_video_aggregator_get_instance_private (vagg);
+  GstCaps *src_template;
+  GstPadTemplate *pad_template;
 
+  vagg->priv = gst_video_aggregator_get_instance_private (vagg);
   vagg->priv->current_caps = NULL;
 
   g_mutex_init (&vagg->priv->lock);
 
   /* initialize variables */
   gst_video_aggregator_reset (vagg);
+
+  /* Finding all supported formats */
+  vagg->priv->supported_formats = g_ptr_array_new ();
+  pad_template =
+      gst_element_class_get_pad_template (GST_ELEMENT_CLASS (klass), "src");
+  src_template = gst_pad_template_get_caps (pad_template);
+  for (gint i = 0; i < gst_caps_get_size (src_template); i++) {
+    const GValue *v =
+        gst_structure_get_value (gst_caps_get_structure (src_template, i),
+        "format");
+
+    if (G_VALUE_HOLDS_STRING (v)) {
+      GstVideoFormat f = gst_video_format_from_string (g_value_get_string (v));
+      GstVideoFormatInfo *format_info =
+          (GstVideoFormatInfo *) gst_video_format_get_info (f);
+      g_ptr_array_add (vagg->priv->supported_formats, format_info);
+      continue;
+    }
+
+    if (GST_VALUE_HOLDS_LIST (v)) {
+      gint j;
+
+      for (j = 0; j < gst_value_list_get_size (v); j++) {
+        const GValue *v1 = gst_value_list_get_value (v, j);
+        GstVideoFormat f =
+            gst_video_format_from_string (g_value_get_string (v1));
+        GstVideoFormatInfo *format_info =
+            (GstVideoFormatInfo *) gst_video_format_get_info (f);
+        g_ptr_array_add (vagg->priv->supported_formats, format_info);
+      }
+    }
+  }
+
+  gst_caps_unref (src_template);
 }
