@@ -21,39 +21,32 @@
 
 /**
  * SECTION:gstvideooverlaycomposition
+ * @title: GstVideoOverlayRectangle
  * @short_description: Video Buffer Overlay Compositions (Subtitles, Logos)
  *
- * <refsect2>
- * <para>
  * Functions to create and handle overlay compositions on video buffers.
- * </para>
- * <para>
+ *
  * An overlay composition describes one or more overlay rectangles to be
  * blended on top of a video buffer.
- * </para>
- * <para>
+ *
  * This API serves two main purposes:
- * <itemizedlist>
- * <listitem>
- * it can be used to attach overlay information (subtitles or logos)
- * to non-raw video buffers such as GL/VAAPI/VDPAU surfaces. The actual
- * blending of the overlay can then be done by e.g. the video sink that
- * processes these non-raw buffers.
- * </listitem>
- * <listitem>
- * it can also be used to blend overlay rectangles on top of raw video
- * buffers, thus consolidating blending functionality for raw video in
- * one place.
- * </listitem>
+ *
+ * * it can be used to attach overlay information (subtitles or logos)
+ *   to non-raw video buffers such as GL/VAAPI/VDPAU surfaces. The actual
+ *   blending of the overlay can then be done by e.g. the video sink that
+ *   processes these non-raw buffers.
+ *
+ * * it can also be used to blend overlay rectangles on top of raw video
+ *   buffers, thus consolidating blending functionality for raw video in
+ *   one place.
+ *
  * Together, this allows existing overlay elements to easily handle raw
  * and non-raw video as input in without major changes (once the overlays
- * have been put into a #GstOverlayComposition object anyway) - for raw
+ * have been put into a #GstVideoOverlayComposition object anyway) - for raw
  * video the overlay can just use the blending function to blend the data
  * on top of the video, and for surface buffers it can just attach them to
  * the buffer and let the sink render the overlays.
- * </itemizedlist>
- * </para>
- * </refsect2>
+ *
  */
 
 /* TODO:
@@ -260,7 +253,8 @@ gst_video_overlay_composition_meta_get_info (void)
 {
   static const GstMetaInfo *video_overlay_composition_meta_info = NULL;
 
-  if (g_once_init_enter (&video_overlay_composition_meta_info)) {
+  if (g_once_init_enter ((GstMetaInfo **) &
+          video_overlay_composition_meta_info)) {
     const GstMetaInfo *meta =
         gst_meta_register (GST_VIDEO_OVERLAY_COMPOSITION_META_API_TYPE,
         "GstVideoOverlayCompositionMeta",
@@ -269,7 +263,8 @@ gst_video_overlay_composition_meta_get_info (void)
         (GstMetaFreeFunction) gst_video_overlay_composition_meta_free,
         (GstMetaTransformFunction)
         gst_video_overlay_composition_meta_transform);
-    g_once_init_leave (&video_overlay_composition_meta_info, meta);
+    g_once_init_leave ((GstMetaInfo **) & video_overlay_composition_meta_info,
+        (GstMetaInfo *) meta);
   }
   return video_overlay_composition_meta_info;
 }
@@ -317,6 +312,8 @@ gst_video_overlay_composition_free (GstMiniObject * mini_obj)
   num = comp->num_rectangles;
 
   while (num > 0) {
+    gst_mini_object_remove_parent (GST_MINI_OBJECT_CAST (comp->rectangles[num -
+                1]), GST_MINI_OBJECT_CAST (comp));
     gst_video_overlay_rectangle_unref (comp->rectangles[num - 1]);
     --num;
   }
@@ -359,6 +356,8 @@ gst_video_overlay_composition_new (GstVideoOverlayRectangle * rectangle)
 
   comp->rectangles = g_new0 (GstVideoOverlayRectangle *, RECTANGLE_ARRAY_STEP);
   comp->rectangles[0] = gst_video_overlay_rectangle_ref (rectangle);
+  gst_mini_object_add_parent (GST_MINI_OBJECT_CAST (rectangle),
+      GST_MINI_OBJECT_CAST (comp));
   comp->num_rectangles = 1;
 
   comp->seq_num = gst_video_overlay_get_seqnum ();
@@ -387,7 +386,7 @@ gst_video_overlay_composition_add_rectangle (GstVideoOverlayComposition * comp,
 {
   g_return_if_fail (GST_IS_VIDEO_OVERLAY_COMPOSITION (comp));
   g_return_if_fail (GST_IS_VIDEO_OVERLAY_RECTANGLE (rectangle));
-  g_return_if_fail (GST_MINI_OBJECT_REFCOUNT_VALUE (comp) == 1);
+  g_return_if_fail (gst_mini_object_is_writable (GST_MINI_OBJECT_CAST (comp)));
 
   if (comp->num_rectangles % RECTANGLE_ARRAY_STEP == 0) {
     comp->rectangles =
@@ -397,6 +396,8 @@ gst_video_overlay_composition_add_rectangle (GstVideoOverlayComposition * comp,
 
   comp->rectangles[comp->num_rectangles] =
       gst_video_overlay_rectangle_ref (rectangle);
+  gst_mini_object_add_parent (GST_MINI_OBJECT_CAST (rectangle),
+      GST_MINI_OBJECT_CAST (comp));
   comp->num_rectangles += 1;
 
   comp->min_seq_num_used = MIN (comp->min_seq_num_used, rectangle->seq_num);
@@ -586,11 +587,12 @@ gst_video_overlay_composition_make_writable (GstVideoOverlayComposition * comp)
 
   g_return_val_if_fail (GST_IS_VIDEO_OVERLAY_COMPOSITION (comp), NULL);
 
-  if (GST_MINI_OBJECT_REFCOUNT_VALUE (comp) == 1) {
+  if (gst_mini_object_is_writable (GST_MINI_OBJECT_CAST (comp))) {
     guint n;
 
     for (n = 0; n < comp->num_rectangles; ++n) {
-      if (GST_MINI_OBJECT_REFCOUNT_VALUE (comp->rectangles[n]) != 1)
+      if (!gst_mini_object_is_writable (GST_MINI_OBJECT_CAST (comp->rectangles
+                  [n])))
         goto copy;
     }
     return comp;
@@ -633,6 +635,8 @@ gst_video_overlay_rectangle_free (GstMiniObject * mini_obj)
 {
   GstVideoOverlayRectangle *rect = (GstVideoOverlayRectangle *) mini_obj;
 
+  gst_mini_object_remove_parent (GST_MINI_OBJECT_CAST (rect->pixels),
+      GST_MINI_OBJECT_CAST (rect));
   gst_buffer_replace (&rect->pixels, NULL);
 
   while (rect->scaled_rectangles != NULL) {
@@ -734,10 +738,15 @@ gst_video_overlay_rectangle_new_raw (GstBuffer * pixels,
   g_mutex_init (&rect->lock);
 
   rect->pixels = gst_buffer_ref (pixels);
+  gst_mini_object_add_parent (GST_MINI_OBJECT_CAST (pixels),
+      GST_MINI_OBJECT_CAST (rect));
   rect->scaled_rectangles = NULL;
 
   gst_video_info_init (&rect->info);
-  gst_video_info_set_format (&rect->info, format, width, height);
+  if (!gst_video_info_set_format (&rect->info, format, width, height)) {
+    gst_mini_object_unref (GST_MINI_OBJECT_CAST (rect));
+    return NULL;
+  }
   if (flags & GST_VIDEO_OVERLAY_FORMAT_FLAG_PREMULTIPLIED_ALPHA)
     rect->info.flags |= GST_VIDEO_FLAG_PREMULTIPLIED_ALPHA;
 
@@ -818,7 +827,8 @@ gst_video_overlay_rectangle_set_render_rectangle (GstVideoOverlayRectangle *
     guint render_height)
 {
   g_return_if_fail (GST_IS_VIDEO_OVERLAY_RECTANGLE (rectangle));
-  g_return_if_fail (GST_MINI_OBJECT_REFCOUNT_VALUE (rectangle) == 1);
+  g_return_if_fail (gst_mini_object_is_writable (GST_MINI_OBJECT_CAST
+          (rectangle)));
 
   rectangle->x = render_x;
   rectangle->y = render_y;
@@ -831,12 +841,17 @@ static void
 gst_video_overlay_rectangle_premultiply_0 (GstVideoFrame * frame)
 {
   int i, j;
-  for (j = 0; j < GST_VIDEO_FRAME_HEIGHT (frame); ++j) {
+  int width = GST_VIDEO_FRAME_WIDTH (frame);
+  int height = GST_VIDEO_FRAME_HEIGHT (frame);
+  int stride = GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0);
+  guint8 *data = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
+
+  for (j = 0; j < height; ++j) {
     guint8 *line;
 
-    line = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
-    line += GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0) * j;
-    for (i = 0; i < GST_VIDEO_FRAME_WIDTH (frame); ++i) {
+    line = data;
+    line += stride * j;
+    for (i = 0; i < width; ++i) {
       int a = line[0];
       line[1] = line[1] * a / 255;
       line[2] = line[2] * a / 255;
@@ -850,12 +865,17 @@ static void
 gst_video_overlay_rectangle_premultiply_3 (GstVideoFrame * frame)
 {
   int i, j;
-  for (j = 0; j < GST_VIDEO_FRAME_HEIGHT (frame); ++j) {
+  int width = GST_VIDEO_FRAME_WIDTH (frame);
+  int height = GST_VIDEO_FRAME_HEIGHT (frame);
+  int stride = GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0);
+  guint8 *data = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
+
+  for (j = 0; j < height; ++j) {
     guint8 *line;
 
-    line = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
-    line += GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0) * j;
-    for (i = 0; i < GST_VIDEO_FRAME_WIDTH (frame); ++i) {
+    line = data;
+    line += stride * j;
+    for (i = 0; i < width; ++i) {
       int a = line[3];
       line[0] = line[0] * a / 255;
       line[1] = line[1] * a / 255;
@@ -889,12 +909,17 @@ static void
 gst_video_overlay_rectangle_unpremultiply_0 (GstVideoFrame * frame)
 {
   int i, j;
-  for (j = 0; j < GST_VIDEO_FRAME_HEIGHT (frame); ++j) {
+  int width = GST_VIDEO_FRAME_WIDTH (frame);
+  int height = GST_VIDEO_FRAME_HEIGHT (frame);
+  int stride = GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0);
+  guint8 *data = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
+
+  for (j = 0; j < height; ++j) {
     guint8 *line;
 
-    line = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
-    line += GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0) * j;
-    for (i = 0; i < GST_VIDEO_FRAME_WIDTH (frame); ++i) {
+    line = data;
+    line += stride * j;
+    for (i = 0; i < width; ++i) {
       int a = line[0];
       if (a) {
         line[1] = MIN ((line[1] * 255 + a / 2) / a, 255);
@@ -910,12 +935,17 @@ static void
 gst_video_overlay_rectangle_unpremultiply_3 (GstVideoFrame * frame)
 {
   int i, j;
-  for (j = 0; j < GST_VIDEO_FRAME_HEIGHT (frame); ++j) {
+  int width = GST_VIDEO_FRAME_WIDTH (frame);
+  int height = GST_VIDEO_FRAME_HEIGHT (frame);
+  int stride = GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0);
+  guint8 *data = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
+
+  for (j = 0; j < height; ++j) {
     guint8 *line;
 
-    line = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
-    line += GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0) * j;
-    for (i = 0; i < GST_VIDEO_FRAME_WIDTH (frame); ++i) {
+    line = data;
+    line += stride * j;
+    for (i = 0; i < width; ++i) {
       int a = line[3];
       if (a) {
         line[0] = MIN ((line[0] * 255 + a / 2) / a, 255);
@@ -1002,7 +1032,13 @@ gst_video_overlay_rectangle_apply_global_alpha (GstVideoOverlayRectangle * rect,
     gst_video_overlay_rectangle_extract_alpha (rect);
 
   src = rect->initial_alpha;
-  rect->pixels = gst_buffer_make_writable (rect->pixels);
+  if (!gst_buffer_is_writable (rect->pixels)) {
+    gst_mini_object_remove_parent (GST_MINI_OBJECT_CAST (rect->pixels),
+        GST_MINI_OBJECT_CAST (rect));
+    rect->pixels = gst_buffer_copy (rect->pixels);
+    gst_mini_object_add_parent (GST_MINI_OBJECT_CAST (rect->pixels),
+        GST_MINI_OBJECT_CAST (rect));
+  }
 
   gst_video_frame_map (&frame, &rect->info, rect->pixels, GST_MAP_READ);
   dst = GST_VIDEO_FRAME_PLANE_DATA (&frame, 0);
@@ -1057,7 +1093,10 @@ gst_video_overlay_rectangle_convert (GstVideoInfo * src, GstBuffer * src_buffer,
   height = GST_VIDEO_INFO_HEIGHT (src);
 
   gst_video_info_init (dest);
-  gst_video_info_set_format (dest, dest_format, width, height);
+  if (!gst_video_info_set_format (dest, dest_format, width, height)) {
+    g_warn_if_reached ();
+    return;
+  }
 
   *dest_buffer = gst_buffer_new_and_alloc (GST_VIDEO_INFO_SIZE (dest));
 
@@ -1527,6 +1566,8 @@ gst_video_overlay_rectangle_set_global_alpha (GstVideoOverlayRectangle *
     rectangle, gfloat global_alpha)
 {
   g_return_if_fail (GST_IS_VIDEO_OVERLAY_RECTANGLE (rectangle));
+  g_return_if_fail (gst_mini_object_is_writable (GST_MINI_OBJECT_CAST
+          (rectangle)));
   g_return_if_fail (global_alpha >= 0 && global_alpha <= 1);
 
   if (rectangle->global_alpha != global_alpha) {
@@ -1587,7 +1628,7 @@ gst_video_overlay_rectangle_copy (GstVideoOverlayRectangle * rectangle)
  * the per rectangle sequence number, which is misleading for renderers/
  * consumers, that handle global-alpha themselves. For them  the
  * pixel-data returned by gst_video_overlay_rectangle_get_pixels_*()
- * wont be different for different global-alpha values. In this case a
+ * won't be different for different global-alpha values. In this case a
  * renderer could also use the GstBuffer pointers as a hint for changed
  * pixel-data.
  *
