@@ -85,7 +85,7 @@
  * ## Two basic usage scenarios
  *
  * There are two basic usage scenarios: in the simplest case, the application
- * uses #playbin or #plasink or knows exactly what particular element is used
+ * uses #playbin or #playsink or knows exactly what particular element is used
  * for video output, which is usually the case when the application creates
  * the videosink to use (e.g. #xvimagesink, #ximagesink, etc.) itself; in this
  * case, the application can just create the videosink element, create and
@@ -118,10 +118,10 @@
  * windowing systems are not thread-safe at all and a lot of care would be
  * required to co-ordinate the toolkit and window system calls of the
  * different threads (Gtk+ users please note: prior to Gtk+ 2.18
- * GDK_WINDOW_XID() was just a simple structure access, so generally fine to do
+ * `GDK_WINDOW_XID` was just a simple structure access, so generally fine to do
  * within the bus sync handler; this macro was changed to a function call in
  * Gtk+ 2.18 and later, which is likely to cause problems when called from a
- * sync handler; see below for a better approach without GDK_WINDOW_XID()
+ * sync handler; see below for a better approach without `GDK_WINDOW_XID`
  * used in the callback).
  *
  * ## GstVideoOverlay and Gtk+
@@ -228,13 +228,13 @@
  * ## GstVideoOverlay and Qt
  *
  * |[
- * #include &lt;glib.h&gt;
- * #include &lt;gst/gst.h&gt;
- * #include &lt;gst/video/videooverlay.h&gt;
+ * #include <glib.h>;
+ * #include <gst/gst.h>;
+ * #include <gst/video/videooverlay.h>;
  *
- * #include &lt;QApplication&gt;
- * #include &lt;QTimer&gt;
- * #include &lt;QWidget&gt;
+ * #include <QApplication>;
+ * #include <QTimer>;
+ * #include <QWidget>;
  *
  * int main(int argc, char *argv[])
  * {
@@ -525,7 +525,7 @@ gst_is_video_overlay_prepare_window_handle_message (GstMessage * msg)
  * properties. This helper will install "render-rectangle" property into the
  * class.
  *
- * Since 1.14
+ * Since: 1.14
  */
 void
 gst_video_overlay_install_properties (GObjectClass * oclass, gint last_prop_id)
@@ -553,7 +553,7 @@ gst_video_overlay_install_properties (GObjectClass * oclass, gint last_prop_id)
  *
  * Returns: %TRUE if the @property_id matches the GstVideoOverlay property
  *
- * Since 1.14
+ * Since: 1.14
  */
 gboolean
 gst_video_overlay_set_property (GObject * object, gint last_prop_id,
@@ -591,7 +591,7 @@ wrong_format:
     g_value_init (&string, G_TYPE_STRING);
     g_value_transform (value, &string);
 
-    g_critical ("Badly formated rectangle, must contains four gint (got '%s')",
+    g_critical ("Badly formatted rectangle, must contains four gint (got '%s')",
         g_value_get_string (&string));
 
     g_value_unset (&string);
