@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __GST_ADDER_H__
@@ -29,18 +29,8 @@
 
 G_BEGIN_DECLS
 
-#define GST_TYPE_ADDER            (gst_adder_get_type())
-#define GST_ADDER(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_ADDER,GstAdder))
-#define GST_IS_ADDER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ADDER))
-#define GST_ADDER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass) ,GST_TYPE_ADDER,GstAdderClass))
-#define GST_IS_ADDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass) ,GST_TYPE_ADDER))
-#define GST_ADDER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj) ,GST_TYPE_ADDER,GstAdderClass))
-
-typedef struct _GstAdder             GstAdder;
-typedef struct _GstAdderClass        GstAdderClass;
-typedef struct _GstAdderInputChannel GstAdderInputChannel;
-
-typedef void (*GstAdderFunction) (gpointer out, gpointer in, guint size);
+#define GST_TYPE_ADDER (gst_adder_get_type())
+G_DECLARE_FINAL_TYPE (GstAdder, gst_adder, GST, ADDER, GstElement)
 
 /**
  * GstAdder:
@@ -58,35 +48,40 @@ struct _GstAdder {
   /* the next are valid for both int and float */
   GstAudioInfo    info;
 
-  /* function to add samples */
-  GstAdderFunction func;
-
   /* counters to keep track of timestamps */
   gint64          offset;
 
   /* sink event handling */
   GstSegment      segment;
   volatile gboolean new_segment_pending;
-  volatile gboolean wait_for_new_segment;
-  /* src event handling */
   volatile gboolean flush_stop_pending;
 
-  /* target caps */
+  /* current caps */
+  GstCaps *current_caps;
+
+  /* target caps (set via property) */
   GstCaps *filter_caps;
 
   /* Pending inline events */
   GList *pending_events;
   
-  gboolean in_setcaps;
+  gboolean send_stream_start;
+  gboolean send_caps;
 };
 
-struct _GstAdderClass {
-  GstElementClass parent_class;
-};
+#define GST_TYPE_ADDER_PAD (gst_adder_pad_get_type())
+G_DECLARE_FINAL_TYPE (GstAdderPad, gst_adder_pad, GST, ADDER_PAD, GstPad)
 
-GType    gst_adder_get_type (void);
+struct _GstAdderPad {
+  GstPad parent;
+
+  gdouble volume;
+  gint volume_i32;
+  gint volume_i16;
+  gint volume_i8;
+  gboolean mute;
+};
 
 G_END_DECLS
 
-
 #endif /* __GST_ADDER_H__ */
