@@ -620,6 +620,8 @@ gst_audio_interleave_class_init (GstAudioInterleaveClass * klass)
           "Channel positions from input",
           "Take channel positions from the input", TRUE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  gst_type_mark_as_plugin_api (GST_TYPE_AUDIO_INTERLEAVE_PAD, 0);
 }
 
 static void
@@ -895,7 +897,7 @@ gst_audio_interleave_child_proxy_init (gpointer g_iface, gpointer iface_data)
 {
   GstChildProxyInterface *iface = g_iface;
 
-  GST_INFO ("intializing child proxy interface");
+  GST_INFO ("initializing child proxy interface");
   iface->get_child_by_index =
       gst_audio_interleave_child_proxy_get_child_by_index;
   iface->get_children_count =
