@@ -38,7 +38,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_compositor_blend_debug);
 
 /* Below are the implementations of everything */
 
-/* A32 is for AYUV, ARGB and BGRA */
+/* A32 is for AYUV, VUYA, ARGB and BGRA */
 #define BLEND_A32(name, method, LOOP)		\
 static void \
 method##_ ##name (GstVideoFrame * srcframe, gint xpos, gint ypos, \
@@ -101,17 +101,26 @@ _overlay_loop_##name (guint8 * dest, const guint8 * src, gint src_height, \
   s_alpha = MIN (255, s_alpha); \
   switch (mode) { \
     case COMPOSITOR_BLEND_MODE_SOURCE:\
-      compositor_orc_source_##name (dest, dest_stride, src, src_stride, \
-        s_alpha, src_width, src_height); \
-        break;\
+      if (s_alpha == 255) { \
+        guint y; \
+        for (y = 0; y < src_height; y++) { \
+          memcpy (dest, src, 4 * src_width); \
+          dest += dest_stride; \
+          src += src_stride; \
+        } \
+      } else { \
+        compositor_orc_source_##name (dest, dest_stride, src, src_stride, \
+          s_alpha, src_width, src_height); \
+      } \
+      break;\
     case COMPOSITOR_BLEND_MODE_OVER:\
       compositor_orc_overlay_##name (dest, dest_stride, src, src_stride, \
         s_alpha, src_width, src_height); \
-        break;\
+      break;\
     case COMPOSITOR_BLEND_MODE_ADD:\
       compositor_orc_overlay_##name##_addition (dest, dest_stride, src, src_stride, \
         s_alpha, src_width, src_height); \
-        break;\
+      break;\
   }\
 }
 
@@ -124,15 +133,24 @@ _blend_loop_##name (guint8 * dest, const guint8 * src, gint src_height, \
   s_alpha = MIN (255, s_alpha); \
   switch (mode) { \
     case COMPOSITOR_BLEND_MODE_SOURCE:\
-      compositor_orc_source_##name (dest, dest_stride, src, src_stride, \
-        s_alpha, src_width, src_height); \
-        break;\
+      if (s_alpha == 255) { \
+        guint y; \
+        for (y = 0; y < src_height; y++) { \
+          memcpy (dest, src, 4 * src_width); \
+          dest += dest_stride; \
+          src += src_stride; \
+        } \
+      } else { \
+        compositor_orc_source_##name (dest, dest_stride, src, src_stride, \
+          s_alpha, src_width, src_height); \
+      } \
+      break;\
     case COMPOSITOR_BLEND_MODE_OVER:\
     case COMPOSITOR_BLEND_MODE_ADD:\
       /* both modes are the same for opaque background */ \
       compositor_orc_blend_##name (dest, dest_stride, src, src_stride, \
         s_alpha, src_width, src_height); \
-        break;\
+      break;\
   }\
 }
 
@@ -194,6 +212,7 @@ fill_checker_##name##_c (GstVideoFrame * frame) \
 A32_CHECKER_C (argb, TRUE, 0, 1, 2, 3);
 A32_CHECKER_C (bgra, TRUE, 3, 2, 1, 0);
 A32_CHECKER_C (ayuv, FALSE, 0, 1, 2, 3);
+A32_CHECKER_C (vuya, FALSE, 3, 2, 1, 0);
 
 #define YUV_TO_R(Y,U,V) (CLAMP (1.164 * (Y - 16) + 1.596 * (V - 128), 0, 255))
 #define YUV_TO_G(Y,U,V) (CLAMP (1.164 * (Y - 16) - 0.813 * (V - 128) - 0.391 * (U - 128), 0, 255))
@@ -231,6 +250,7 @@ A32_COLOR (bgra, TRUE, 0, 8, 16, 24);
 A32_COLOR (abgr, TRUE, 24, 0, 8, 16);
 A32_COLOR (rgba, TRUE, 0, 24, 16, 8);
 A32_COLOR (ayuv, FALSE, 24, 16, 8, 0);
+A32_COLOR (vuya, FALSE, 0, 8, 16, 24);
 
 /* Y444, Y42B, I420, YV12, Y41B */
 #define PLANAR_YUV_BLEND(format_name,format_enum,x_round,y_round,MEMCPY,BLENDLOOP) \
@@ -354,7 +374,7 @@ blend_##format_name (GstVideoFrame * srcframe, gint xpos, gint ypos, \
   comp_xpos = (xpos == 0) ? 0 : GST_VIDEO_FORMAT_INFO_SCALE_WIDTH (info, 1, xpos); \
   comp_ypos = (ypos == 0) ? 0 : GST_VIDEO_FORMAT_INFO_SCALE_HEIGHT (info, 1, ypos); \
   comp_xoffset = (xoffset == 0) ? 0 : GST_VIDEO_FORMAT_INFO_SCALE_WIDTH (info, 1, xoffset); \
-  comp_yoffset = (yoffset == 0) ? 0 : GST_VIDEO_FORMAT_INFO_SCALE_HEIGHT (info, 1, yoffset); \
+  comp_yoffset = (yoffset == 0) ? 0 : yoffset >> info->h_sub[1]; \
   _blend_##format_name (b_src + comp_xoffset + comp_yoffset * src_comp_rowstride, \
       b_dest + comp_xpos + comp_ypos * dest_comp_rowstride, \
       src_comp_rowstride, \
@@ -370,7 +390,7 @@ blend_##format_name (GstVideoFrame * srcframe, gint xpos, gint ypos, \
   comp_xpos = (xpos == 0) ? 0 : GST_VIDEO_FORMAT_INFO_SCALE_WIDTH (info, 2, xpos); \
   comp_ypos = (ypos == 0) ? 0 : GST_VIDEO_FORMAT_INFO_SCALE_HEIGHT (info, 2, ypos); \
   comp_xoffset = (xoffset == 0) ? 0 : GST_VIDEO_FORMAT_INFO_SCALE_WIDTH (info, 2, xoffset); \
-  comp_yoffset = (yoffset == 0) ? 0 : GST_VIDEO_FORMAT_INFO_SCALE_HEIGHT (info, 2, yoffset); \
+  comp_yoffset = (yoffset == 0) ? 0 : yoffset >> info->h_sub[2]; \
   _blend_##format_name (b_src + comp_xoffset + comp_yoffset * src_comp_rowstride, \
       b_dest + comp_xpos + comp_ypos * dest_comp_rowstride, \
       src_comp_rowstride, \
@@ -1023,6 +1043,7 @@ FillCheckerFunction gst_compositor_fill_checker_argb;
 FillCheckerFunction gst_compositor_fill_checker_bgra;
 /* ABGR is equal to ARGB, RGBA is equal to BGRA */
 FillCheckerFunction gst_compositor_fill_checker_ayuv;
+FillCheckerFunction gst_compositor_fill_checker_vuya;
 FillCheckerFunction gst_compositor_fill_checker_y444;
 FillCheckerFunction gst_compositor_fill_checker_y42b;
 FillCheckerFunction gst_compositor_fill_checker_i420;
@@ -1044,6 +1065,7 @@ FillColorFunction gst_compositor_fill_color_bgra;
 FillColorFunction gst_compositor_fill_color_abgr;
 FillColorFunction gst_compositor_fill_color_rgba;
 FillColorFunction gst_compositor_fill_color_ayuv;
+FillColorFunction gst_compositor_fill_color_vuya;
 FillColorFunction gst_compositor_fill_color_y444;
 FillColorFunction gst_compositor_fill_color_y42b;
 FillColorFunction gst_compositor_fill_color_i420;
@@ -1084,6 +1106,7 @@ gst_compositor_init_blend (void)
   gst_compositor_fill_checker_argb = GST_DEBUG_FUNCPTR (fill_checker_argb_c);
   gst_compositor_fill_checker_bgra = GST_DEBUG_FUNCPTR (fill_checker_bgra_c);
   gst_compositor_fill_checker_ayuv = GST_DEBUG_FUNCPTR (fill_checker_ayuv_c);
+  gst_compositor_fill_checker_vuya = GST_DEBUG_FUNCPTR (fill_checker_vuya_c);
   gst_compositor_fill_checker_i420 = GST_DEBUG_FUNCPTR (fill_checker_i420);
   gst_compositor_fill_checker_nv12 = GST_DEBUG_FUNCPTR (fill_checker_nv12);
   gst_compositor_fill_checker_nv21 = GST_DEBUG_FUNCPTR (fill_checker_nv21);
@@ -1101,6 +1124,7 @@ gst_compositor_init_blend (void)
   gst_compositor_fill_color_abgr = GST_DEBUG_FUNCPTR (fill_color_abgr);
   gst_compositor_fill_color_rgba = GST_DEBUG_FUNCPTR (fill_color_rgba);
   gst_compositor_fill_color_ayuv = GST_DEBUG_FUNCPTR (fill_color_ayuv);
+  gst_compositor_fill_color_vuya = GST_DEBUG_FUNCPTR (fill_color_vuya);
   gst_compositor_fill_color_i420 = GST_DEBUG_FUNCPTR (fill_color_i420);
   gst_compositor_fill_color_yv12 = GST_DEBUG_FUNCPTR (fill_color_yv12);
   gst_compositor_fill_color_nv12 = GST_DEBUG_FUNCPTR (fill_color_nv12);
