@@ -24,7 +24,7 @@
  * SECTION:element-compositor
  * @title: compositor
  *
- * Compositor can accept AYUV, ARGB and BGRA video streams. For each of the requested
+ * Compositor can accept AYUV, VUYA, ARGB and BGRA video streams. For each of the requested
  * sink pads it will compare the incoming geometry and framerate to define the
  * output parameters. Indeed output video frames will have the geometry of the
  * biggest incoming video stream and the framerate of the fastest incoming one.
@@ -65,7 +65,7 @@
  *   compositor name=comp ! videoconvert ! ximagesink \
  *   videotestsrc !  \
  *   video/x-raw, framerate=\(fraction\)5/1, width=320, height=240 ! comp.
- * ]| A pipeline to demostrate bgra comping. (This does not demonstrate alpha blending).
+ * ]| A pipeline to demonstrate bgra comping. (This does not demonstrate alpha blending).
  * |[
  * gst-launch-1.0 videotestsrc pattern=1 ! \
  *   video/x-raw,format =I420, framerate=\(fraction\)10/1, width=100, height=100 ! \
@@ -103,7 +103,7 @@
 GST_DEBUG_CATEGORY_STATIC (gst_compositor_debug);
 #define GST_CAT_DEFAULT gst_compositor_debug
 
-#define FORMATS " { AYUV, BGRA, ARGB, RGBA, ABGR, Y444, Y42B, YUY2, UYVY, "\
+#define FORMATS " { AYUV, VUYA, BGRA, ARGB, RGBA, ABGR, Y444, Y42B, YUY2, UYVY, "\
                 "   YVYU, I420, YV12, NV12, NV21, Y41B, RGB, BGR, xRGB, xBGR, "\
                 "   RGBx, BGRx } "
 
@@ -116,7 +116,7 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink_%u",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE (FORMATS))
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL))
     );
 
 static void gst_compositor_child_proxy_init (gpointer g_iface,
@@ -142,6 +142,29 @@ gst_compositor_operator_get_type (void)
   return compositor_operator_type;
 }
 
+#define GST_TYPE_COMPOSITOR_BACKGROUND (gst_compositor_background_get_type())
+static GType
+gst_compositor_background_get_type (void)
+{
+  static GType compositor_background_type = 0;
+
+  static const GEnumValue compositor_background[] = {
+    {COMPOSITOR_BACKGROUND_CHECKER, "Checker pattern", "checker"},
+    {COMPOSITOR_BACKGROUND_BLACK, "Black", "black"},
+    {COMPOSITOR_BACKGROUND_WHITE, "White", "white"},
+    {COMPOSITOR_BACKGROUND_TRANSPARENT,
+        "Transparent Background to enable further compositing", "transparent"},
+    {0, NULL, NULL},
+  };
+
+  if (!compositor_background_type) {
+    compositor_background_type =
+        g_enum_register_static ("GstCompositorBackground",
+        compositor_background);
+  }
+  return compositor_background_type;
+}
+
 #define DEFAULT_PAD_XPOS   0
 #define DEFAULT_PAD_YPOS   0
 #define DEFAULT_PAD_WIDTH  0
@@ -315,7 +338,7 @@ clamp_rectangle (gint x, gint y, gint w, gint h, gint outer_width,
 /* Call this with the lock taken */
 static gboolean
 _pad_obscures_rectangle (GstVideoAggregator * vagg, GstVideoAggregatorPad * pad,
-    const GstVideoRectangle rect, gboolean rect_transparent)
+    const GstVideoRectangle rect)
 {
   GstVideoRectangle pad_rect;
   GstCompositorPad *cpad = GST_COMPOSITOR_PAD (pad);
@@ -324,12 +347,11 @@ _pad_obscures_rectangle (GstVideoAggregator * vagg, GstVideoAggregatorPad * pad,
   if (!gst_video_aggregator_pad_has_current_buffer (pad))
     return FALSE;
 
-  /* Can't obscure if it's transparent and if the format has an alpha component
-   * we'd have to inspect every pixel to know if the frame is opaque, so assume
-   * it doesn't obscure. As a bonus, if the rectangle is fully transparent, we
-   * can also obscure it if we have alpha components on the pad */
-  if (!rect_transparent &&
-      (cpad->alpha != 1.0 || GST_VIDEO_INFO_HAS_ALPHA (&pad->info)))
+  /* Can't obscure if we introduce alpha or if the format has an alpha
+   * component as we'd have to inspect every pixel to know if the frame is
+   * opaque, so assume it doesn't obscure
+   */
+  if (cpad->alpha != 1.0 || GST_VIDEO_INFO_HAS_ALPHA (&pad->info))
     return FALSE;
 
   pad_rect.x = cpad->xpos;
@@ -394,9 +416,12 @@ gst_compositor_pad_prepare_frame (GstVideoAggregatorPad * pad,
   /* Check if this frame is obscured by a higher-zorder frame
    * TODO: Also skip a frame if it's obscured by a combination of
    * higher-zorder frames */
-  l = g_list_find (GST_ELEMENT (vagg)->sinkpads, pad)->next;
+  l = g_list_find (GST_ELEMENT (vagg)->sinkpads, pad);
+  /* The pad might've just been removed */
+  if (l)
+    l = l->next;
   for (; l; l = l->next) {
-    if (_pad_obscures_rectangle (vagg, l->data, frame_rect, FALSE)) {
+    if (_pad_obscures_rectangle (vagg, l->data, frame_rect)) {
       frame_obscured = TRUE;
       break;
     }
@@ -521,29 +546,6 @@ enum
   PROP_BACKGROUND,
 };
 
-#define GST_TYPE_COMPOSITOR_BACKGROUND (gst_compositor_background_get_type())
-static GType
-gst_compositor_background_get_type (void)
-{
-  static GType compositor_background_type = 0;
-
-  static const GEnumValue compositor_background[] = {
-    {COMPOSITOR_BACKGROUND_CHECKER, "Checker pattern", "checker"},
-    {COMPOSITOR_BACKGROUND_BLACK, "Black", "black"},
-    {COMPOSITOR_BACKGROUND_WHITE, "White", "white"},
-    {COMPOSITOR_BACKGROUND_TRANSPARENT,
-        "Transparent Background to enable further compositing", "transparent"},
-    {0, NULL, NULL},
-  };
-
-  if (!compositor_background_type) {
-    compositor_background_type =
-        g_enum_register_static ("GstCompositorBackground",
-        compositor_background);
-  }
-  return compositor_background_type;
-}
-
 static void
 gst_compositor_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec)
@@ -599,6 +601,13 @@ set_functions (GstCompositor * self, GstVideoInfo * info)
       self->fill_color = gst_compositor_fill_color_ayuv;
       ret = TRUE;
       break;
+    case GST_VIDEO_FORMAT_VUYA:
+      self->blend = gst_compositor_blend_vuya;
+      self->overlay = gst_compositor_overlay_vuya;
+      self->fill_checker = gst_compositor_fill_checker_vuya;
+      self->fill_color = gst_compositor_fill_color_vuya;
+      ret = TRUE;
+      break;
     case GST_VIDEO_FORMAT_ARGB:
       self->blend = gst_compositor_blend_argb;
       self->overlay = gst_compositor_overlay_argb;
@@ -840,7 +849,7 @@ _negotiated_caps (GstAggregator * agg, GstCaps * caps)
 }
 
 static gboolean
-_should_draw_background (GstVideoAggregator * vagg, gboolean bg_transparent)
+_should_draw_background (GstVideoAggregator * vagg)
 {
   GstVideoRectangle bg_rect;
   gboolean draw = TRUE;
@@ -854,7 +863,7 @@ _should_draw_background (GstVideoAggregator * vagg, gboolean bg_transparent)
   /* Check if the background is completely obscured by a pad
    * TODO: Also skip if it's obscured by a combination of pads */
   for (l = GST_ELEMENT (vagg)->sinkpads; l; l = l->next) {
-    if (_pad_obscures_rectangle (vagg, l->data, bg_rect, bg_transparent)) {
+    if (_pad_obscures_rectangle (vagg, l->data, bg_rect)) {
       draw = FALSE;
       break;
     }
@@ -874,8 +883,7 @@ _draw_background (GstVideoAggregator * vagg, GstVideoFrame * outframe,
    * don't bother drawing the background at all. We can also always use the
    * 'blend' BlendFunction in that case because it only changes if we have to
    * overlay on top of a transparent background. */
-  if (!_should_draw_background (vagg,
-          comp->background == COMPOSITOR_BACKGROUND_TRANSPARENT))
+  if (!_should_draw_background (vagg))
     return FALSE;
 
   switch (comp->background) {
@@ -1108,6 +1116,10 @@ gst_compositor_class_init (GstCompositorClass * klass)
       "Filter/Editor/Video/Compositor",
       "Composite multiple video streams", "Wim Taymans <wim@fluendo.com>, "
       "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
+
+  gst_type_mark_as_plugin_api (GST_TYPE_COMPOSITOR_PAD, 0);
+  gst_type_mark_as_plugin_api (GST_TYPE_COMPOSITOR_OPERATOR, 0);
+  gst_type_mark_as_plugin_api (GST_TYPE_COMPOSITOR_BACKGROUND, 0);
 }
 
 static void
