@@ -198,7 +198,8 @@ struct _StreamGroup
   GstEncodeBin *ebin;
   GstEncodingProfile *profile;
   GstPad *ghostpad;             /* Sink ghostpad */
-  GstElement *inqueue;          /* Queue just after the ghostpad */
+  GstElement *identity;         /* Identity just after the ghostpad */
+  GstElement *inqueue;          /* Queue just after the identity */
   GstElement *splitter;
   GList *converters;            /* List of conversion GstElement */
   GstElement *capsfilter;       /* profile->restriction (if non-NULL/ANY) */
@@ -208,6 +209,8 @@ struct _StreamGroup
   GstElement *combiner;
   GstElement *parser;
   GstElement *smartencoder;
+  GstElement *smart_capsfilter;
+  gulong smart_capsfilter_sid;
   GstElement *outfilter;        /* Output capsfilter (streamprofile.format) */
   gulong outputfilter_caps_sid;
   GstElement *formatter;
@@ -399,8 +402,7 @@ gst_encode_bin_class_init (GstEncodeBinClass * klass)
   gst_encode_bin_signals[SIGNAL_REQUEST_PAD] =
       g_signal_new ("request-pad", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstEncodeBinClass,
-          request_pad), NULL, NULL, g_cclosure_marshal_generic,
-      GST_TYPE_PAD, 1, GST_TYPE_CAPS);
+          request_pad), NULL, NULL, NULL, GST_TYPE_PAD, 1, GST_TYPE_CAPS);
 
   /**
    * GstEncodeBin::request-profile-pad
@@ -417,8 +419,8 @@ gst_encode_bin_class_init (GstEncodeBinClass * klass)
   gst_encode_bin_signals[SIGNAL_REQUEST_PROFILE_PAD] =
       g_signal_new ("request-profile-pad", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstEncodeBinClass,
-          request_profile_pad), NULL, NULL, g_cclosure_marshal_generic,
-      GST_TYPE_PAD, 1, G_TYPE_STRING);
+          request_profile_pad), NULL, NULL, NULL, GST_TYPE_PAD, 1,
+      G_TYPE_STRING);
 
   klass->request_pad = gst_encode_bin_request_pad_signal;
   klass->request_profile_pad = gst_encode_bin_request_profile_pad_signal;
@@ -445,6 +447,8 @@ gst_encode_bin_class_init (GstEncodeBinClass * klass)
       "Generic/Bin/Encoder",
       "Convenience encoding/muxing element",
       "Edward Hervey <edward.hervey@collabora.co.uk>");
+
+  gst_type_mark_as_plugin_api (GST_TYPE_ENCODEBIN_FLAGS, 0);
 }
 
 static void
@@ -546,8 +550,14 @@ gst_encode_bin_set_property (GObject * object, guint prop_id,
       ebin->tolerance = g_value_get_uint64 (value);
       break;
     case PROP_AVOID_REENCODING:
+    {
+      gboolean avoided_reencoding = ebin->avoid_reencoding;
       ebin->avoid_reencoding = g_value_get_boolean (value);
+      if (ebin->avoid_reencoding != avoided_reencoding && ebin->profile)
+        gst_encode_bin_set_profile (ebin, gst_object_ref (ebin->profile));
+
       break;
+    }
     case PROP_FLAGS:
       ebin->flags = g_value_get_flags (value);
       break;
@@ -866,14 +876,25 @@ no_stream_group:
 
 /* Create a parser for the given stream profile */
 static inline GstElement *
-_get_parser (GstEncodeBin * ebin, GstEncodingProfile * sprof)
+_get_parser (GstEncodeBin * ebin, GstEncodingProfile * sprof,
+    GstElement * encoder)
 {
   GList *parsers1, *parsers, *tmp;
   GstElement *parser = NULL;
   GstElementFactory *parserfact = NULL;
-  GstCaps *format;
+  GstCaps *format = NULL;
 
-  format = gst_encoding_profile_get_format (sprof);
+  if (encoder) {
+    GstPadTemplate *template = gst_element_get_pad_template (encoder, "src");
+
+    if (template)
+      format = gst_pad_template_get_caps (template);
+  }
+
+  if (!format || gst_caps_is_any (format)) {
+    gst_clear_caps (&format);
+    format = gst_encoding_profile_get_format (sprof);
+  }
 
   GST_DEBUG ("Getting list of parsers for format %" GST_PTR_FORMAT, format);
 
@@ -1120,10 +1141,17 @@ _profile_restriction_caps_cb (GstEncodingProfile * profile,
 
 static void
 _capsfilter_force_format (GstPad * pad,
-    GParamSpec * arg G_GNUC_UNUSED, gulong * signal_id)
+    GParamSpec * arg G_GNUC_UNUSED, StreamGroup * sgroup)
 {
   GstCaps *caps;
   GstStructure *structure;
+  GstElement *parent =
+      GST_ELEMENT_CAST (gst_object_get_parent (GST_OBJECT (pad)));
+
+  if (!parent) {
+    GST_DEBUG_OBJECT (pad, "Doesn't have a parent anymore");
+    return;
+  }
 
   g_object_get (pad, "caps", &caps, NULL);
   caps = gst_caps_copy (caps);
@@ -1131,10 +1159,36 @@ _capsfilter_force_format (GstPad * pad,
   structure = gst_caps_get_structure (caps, 0);
   gst_structure_remove_field (structure, "streamheader");
   GST_INFO_OBJECT (pad, "Forcing caps to %" GST_PTR_FORMAT, caps);
-  g_object_set (GST_OBJECT_PARENT (pad), "caps", caps, NULL);
-  g_signal_handler_disconnect (pad, *signal_id);
-  *signal_id = 0;
+  if (parent == sgroup->outfilter || parent == sgroup->smart_capsfilter) {
+    /* outfilter and the smart encoder internal capsfilter need to always be
+     * in sync so the caps match between the two */
+    if (sgroup->smart_capsfilter) {
+      gst_structure_remove_field (structure, "codec_data");
+      /* The smart encoder handles codec_data itself */
+      g_object_set (sgroup->smart_capsfilter, "caps", caps, NULL);
+
+      g_signal_handler_disconnect (sgroup->smart_capsfilter->sinkpads->data,
+          sgroup->smart_capsfilter_sid);
+      sgroup->smart_capsfilter_sid = 0;
+    }
+
+    if (sgroup->outfilter) {
+      GstCaps *tmpcaps = gst_caps_copy (caps);
+      g_object_set (sgroup->outfilter, "caps", tmpcaps, NULL);
+      gst_caps_unref (tmpcaps);
+      g_signal_handler_disconnect (sgroup->outfilter->sinkpads->data,
+          sgroup->outputfilter_caps_sid);
+      sgroup->outputfilter_caps_sid = 0;
+    }
+  } else if (parent == sgroup->capsfilter) {
+    g_object_set (parent, "caps", caps, NULL);
+    g_signal_handler_disconnect (pad, sgroup->inputfilter_caps_sid);
+  } else {
+    g_assert_not_reached ();
+  }
+
   gst_caps_unref (caps);
+  gst_object_unref (parent);
 }
 
 static void
@@ -1147,8 +1201,7 @@ _set_group_caps_format (StreamGroup * sgroup, GstEncodingProfile * prof,
     if (!sgroup->outputfilter_caps_sid) {
       sgroup->outputfilter_caps_sid =
           g_signal_connect (sgroup->outfilter->sinkpads->data,
-          "notify::caps", G_CALLBACK (_capsfilter_force_format),
-          &sgroup->outputfilter_caps_sid);
+          "notify::caps", G_CALLBACK (_capsfilter_force_format), sgroup);
     }
   }
 }
@@ -1196,6 +1249,112 @@ _set_up_fake_encoder_pad_probe (GstEncodeBin * ebin, StreamGroup * sgroup)
   gst_object_unref (pad);
 }
 
+static GstElement *
+setup_smart_encoder (GstEncodeBin * ebin, GstEncodingProfile * sprof,
+    StreamGroup * sgroup)
+{
+  GstElement *encoder = NULL, *parser = NULL;
+  GstElement *reencoder_bin = NULL;
+  GstElement *sinkelement, *convert = NULL;
+  GstElement *smartencoder = g_object_new (GST_TYPE_SMART_ENCODER, NULL);
+  GstPad *srcpad = gst_element_get_static_pad (smartencoder, "src");
+  GstCaps *format = gst_encoding_profile_get_format (sprof);
+  GstCaps *tmpcaps = gst_pad_query_caps (srcpad, NULL);
+  const gboolean native_video =
+      ! !(ebin->flags & GST_ENCODEBIN_FLAG_NO_VIDEO_CONVERSION);
+
+  /* Check if stream format is compatible */
+  if (!gst_caps_can_intersect (tmpcaps, format)) {
+    GST_DEBUG_OBJECT (ebin,
+        "We don't have a smart encoder for the stream format: %" GST_PTR_FORMAT,
+        format);
+    goto err;
+  }
+
+  sinkelement = encoder = _get_encoder (ebin, sprof);
+  if (!encoder) {
+    GST_INFO_OBJECT (ebin, "No encoder found... not using smart rendering");
+    goto err;
+  }
+
+  parser = _get_parser (ebin, sprof, encoder);
+  sgroup->smart_capsfilter = gst_element_factory_make ("capsfilter", NULL);
+  reencoder_bin = gst_bin_new (NULL);
+  g_object_set (sgroup->smart_capsfilter, "caps", format, NULL);
+
+  gst_bin_add_many (GST_BIN (reencoder_bin),
+      gst_object_ref (encoder),
+      parser ? gst_object_ref (parser) : sgroup->smart_capsfilter,
+      parser ? gst_object_ref (sgroup->smart_capsfilter) : NULL, NULL);
+  if (!native_video) {
+    convert = gst_element_factory_make ("videoconvert", NULL);
+    if (!convert) {
+      GST_ERROR_OBJECT (ebin, "`videoconvert` element missing");
+      goto err;
+    }
+
+    gst_bin_add (GST_BIN (reencoder_bin), gst_object_ref (convert));
+    if (!gst_element_link (convert, sinkelement)) {
+      GST_ERROR_OBJECT (ebin, "Can not link `videoconvert` to %" GST_PTR_FORMAT,
+          sinkelement);
+      goto err;
+    }
+    sinkelement = convert;
+  }
+
+  if (!gst_element_link_many (encoder,
+          parser ? parser : sgroup->smart_capsfilter,
+          parser ? sgroup->smart_capsfilter : NULL, NULL)) {
+    GST_ERROR_OBJECT (ebin, "Can not link smart encoding elements");
+    goto err;
+  }
+
+  if (!gst_element_add_pad (reencoder_bin,
+          gst_ghost_pad_new ("sink", sinkelement->sinkpads->data))) {
+    GST_ERROR_OBJECT (ebin, "Can add smart encoding bin `srcpad`");
+    goto err;
+  }
+
+  if (!gst_element_add_pad (reencoder_bin,
+          gst_ghost_pad_new ("src", sgroup->smart_capsfilter->srcpads->data))) {
+    GST_ERROR_OBJECT (ebin, "Could not ghost smart encoder bin"
+        " srcpad, not being smart.");
+    goto err;
+  }
+
+  if (!gst_encoding_profile_get_allow_dynamic_output (sprof)) {
+    /* Enforce no dynamic output in the smart encoder */
+    if (!sgroup->smart_capsfilter_sid) {
+      sgroup->smart_capsfilter_sid =
+          g_signal_connect (sgroup->smart_capsfilter->sinkpads->data,
+          "notify::caps", G_CALLBACK (_capsfilter_force_format), sgroup);
+    }
+  }
+
+  if (!gst_smart_encoder_set_encoder (GST_SMART_ENCODER (smartencoder),
+          format, reencoder_bin)) {
+    reencoder_bin = NULL;       /* We do not own the ref anymore */
+    GST_ERROR_OBJECT (ebin, "Could not set encoder to the smart encoder,"
+        " disabling smartness");
+    goto err;
+  }
+
+done:
+  gst_caps_unref (tmpcaps);
+  gst_caps_unref (format);
+  gst_object_unref (srcpad);
+  gst_clear_object (&encoder);
+  gst_clear_object (&parser);
+  gst_clear_object (&convert);
+
+  return smartencoder;
+
+err:
+  gst_clear_object (&smartencoder);
+  gst_clear_object (&reencoder_bin);
+  goto done;
+}
+
 /* FIXME : Add handling of streams that don't require conversion elements */
 /*
  * Create the elements, StreamGroup, add the sink pad, link it to the muxer
@@ -1302,9 +1461,8 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
     goto outfilter_link_failure;
   last = sgroup->outfilter;
 
-
-  sgroup->parser = _get_parser (ebin, sprof);
-
+  sgroup->encoder = _get_encoder (ebin, sprof);
+  sgroup->parser = _get_parser (ebin, sgroup->profile, sgroup->encoder);
   if (sgroup->parser != NULL) {
     GST_DEBUG ("Got a parser %s", GST_ELEMENT_NAME (sgroup->parser));
     gst_bin_add (GST_BIN (ebin), sgroup->parser);
@@ -1329,6 +1487,19 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
   gst_bin_add (GST_BIN (ebin), sgroup->splitter);
   tosync = g_list_append (tosync, sgroup->splitter);
 
+  if (gst_encoding_profile_get_single_segment (sprof)) {
+
+    if (!ebin->avoid_reencoding) {
+      sgroup->identity = gst_element_factory_make ("identity", NULL);
+      g_object_set (sgroup->identity, "single-segment", TRUE, NULL);
+      gst_bin_add (GST_BIN (ebin), sgroup->identity);
+      tosync = g_list_append (tosync, sgroup->identity);
+    } else {
+      GST_INFO_OBJECT (ebin, "Single segment is not supported when avoiding"
+          " to reencode!");
+    }
+  }
+
   /* Input queue
    * FIXME : figure out what max-size to use for the input queue */
   sgroup->inqueue = gst_element_factory_make ("queue", NULL);
@@ -1339,11 +1510,11 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
 
   gst_bin_add (GST_BIN (ebin), sgroup->inqueue);
   tosync = g_list_append (tosync, sgroup->inqueue);
-  if (G_UNLIKELY (!fast_element_link (sgroup->inqueue, sgroup->splitter)))
-    goto splitter_link_failure;
 
-  /* Expose input queue sink pad as ghostpad */
-  sinkpad = gst_element_get_static_pad (sgroup->inqueue, "sink");
+  /* Expose input queue or identity sink pad as ghostpad */
+  sinkpad =
+      gst_element_get_static_pad (sgroup->identity ? sgroup->
+      identity : sgroup->inqueue, "sink");
   if (sinkpadname == NULL) {
     gchar *pname =
         g_strdup_printf ("%s_%u", gst_encoding_profile_get_type_nick (sprof),
@@ -1355,6 +1526,13 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
     sgroup->ghostpad = gst_ghost_pad_new (sinkpadname, sinkpad);
   gst_object_unref (sinkpad);
 
+  if (sgroup->identity
+      && G_UNLIKELY (!fast_element_link (sgroup->identity, sgroup->inqueue)))
+    goto queue_link_failure;
+
+  if (G_UNLIKELY (!fast_element_link (sgroup->inqueue, sgroup->splitter)))
+    goto splitter_link_failure;
+
 
   /* Path 1 : Already-encoded data */
   sinkpad =
@@ -1364,26 +1542,16 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
     goto no_combiner_sinkpad;
 
   if (ebin->avoid_reencoding) {
-    GstCaps *tmpcaps;
-
     GST_DEBUG ("Asked to use Smart Encoder");
-    sgroup->smartencoder = g_object_new (GST_TYPE_SMART_ENCODER, NULL);
-
-    /* Check if stream format is compatible */
-    srcpad = gst_element_get_static_pad (sgroup->smartencoder, "src");
-    tmpcaps = gst_pad_query_caps (srcpad, NULL);
-    if (!gst_caps_can_intersect (tmpcaps, format)) {
-      GST_DEBUG ("We don't have a smart encoder for the stream format");
-      gst_object_unref (sgroup->smartencoder);
-      sgroup->smartencoder = NULL;
-    } else {
+    sgroup->smartencoder = setup_smart_encoder (ebin, sprof, sgroup);
+    if (sgroup->smartencoder) {
       gst_bin_add ((GstBin *) ebin, sgroup->smartencoder);
+      srcpad = gst_element_get_static_pad (sgroup->smartencoder, "src");
       fast_pad_link (srcpad, sinkpad);
+      gst_object_unref (srcpad);
       tosync = g_list_append (tosync, sgroup->smartencoder);
       sinkpad = gst_element_get_static_pad (sgroup->smartencoder, "sink");
     }
-    gst_caps_unref (tmpcaps);
-    gst_object_unref (srcpad);
   }
 
   srcpad =
@@ -1403,8 +1571,7 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
 
   /* 1. Create the encoder */
   GST_LOG ("Adding encoder");
-  sgroup->encoder = _get_encoder (ebin, sprof);
-  if (sgroup->encoder != NULL) {
+  if (sgroup->encoder) {
     gst_bin_add ((GstBin *) ebin, sgroup->encoder);
     tosync = g_list_append (tosync, sgroup->encoder);
 
@@ -1446,8 +1613,7 @@ _create_stream_group (GstEncodeBin * ebin, GstEncodingProfile * sprof,
     if (!sgroup->inputfilter_caps_sid) {
       sgroup->inputfilter_caps_sid =
           g_signal_connect (sgroup->capsfilter->sinkpads->data,
-          "notify::caps", G_CALLBACK (_capsfilter_force_format),
-          &sgroup->inputfilter_caps_sid);
+          "notify::caps", G_CALLBACK (_capsfilter_force_format), sgroup);
     }
   }
 
@@ -1670,6 +1836,10 @@ splitter_link_failure:
   GST_ERROR_OBJECT (ebin, "Failure linking to the splitter");
   goto cleanup;
 
+queue_link_failure:
+  GST_ERROR_OBJECT (ebin, "Failure linking to the inqueue");
+  goto cleanup;
+
 combiner_link_failure:
   GST_ERROR_OBJECT (ebin, "Failure linking to the combiner");
   goto cleanup;
@@ -2110,6 +2280,7 @@ stream_group_free (GstEncodeBin * ebin, StreamGroup * sgroup)
   }
   if (sgroup->smartencoder)
     gst_element_set_state (sgroup->smartencoder, GST_STATE_NULL);
+  gst_clear_object (&sgroup->smart_capsfilter);
 
   if (sgroup->capsfilter) {
     gst_element_set_state (sgroup->capsfilter, GST_STATE_NULL);
@@ -2118,11 +2289,6 @@ stream_group_free (GstEncodeBin * ebin, StreamGroup * sgroup)
     else
       gst_element_unlink (sgroup->capsfilter, sgroup->fakesink);
 
-    if (sgroup->inputfilter_caps_sid) {
-      g_signal_handler_disconnect (sgroup->capsfilter->sinkpads->data,
-          sgroup->inputfilter_caps_sid);
-      sgroup->inputfilter_caps_sid = 0;
-    }
     gst_bin_remove ((GstBin *) ebin, sgroup->capsfilter);
   }
 
