@@ -45,7 +45,7 @@
  * decodebin3 differs from the previous decodebin (decodebin2) in important ways:
  *
  * * supports publication and selection of stream information via
- * GstStreamCollection messages and #GST_EVENT_SELECT_STREAM events.
+ * GstStreamCollection messages and #GST_EVENT_SELECT_STREAMS events.
  *
  * * dynamically switches stream connections internally, and
  * reuses decoder elements when stream selections change, so that in
@@ -53,18 +53,18 @@
  * and only creates new elements when streams change and an existing decoder
  * is not capable of handling the new format.
  *
- * * supports multiple input pads for the parallel decoding of auxilliary streams
+ * * supports multiple input pads for the parallel decoding of auxiliary streams
  * not muxed with the primary stream.
  *
  * * does not handle network stream buffering. decodebin3 expects that network stream
  * buffering is handled upstream, before data is passed to it.
  *
- * <emphasis>decodebin3 is still experimental API and a technology preview.
- * Its behaviour and exposed API is subject to change.</emphasis>
+ * > decodebin3 is still experimental API and a technology preview.
+ * > Its behaviour and exposed API is subject to change.
  *
  */
 
-/**
+/*
  * Global design
  *
  * 1) From sink pad to elementary streams (GstParseBin)
@@ -216,7 +216,7 @@ struct _GstDecodebin3
   /* counter for input */
   guint32 input_counter;
   /* Current stream group_id (default : GST_GROUP_ID_INVALID) */
-  /* FIXME : Needs to be resetted appropriately (when upstream changes ?) */
+  /* FIXME : Needs to be reset appropriately (when upstream changes ?) */
   guint32 current_group_id;
   /* End of variables protected by input_lock */
 
@@ -561,25 +561,28 @@ gst_decodebin3_class_init (GstDecodebin3Class * klass)
    * This signal is emitted whenever @decodebin needs to decide whether
    * to expose a @stream of a given @collection.
    *
+   * Note that the prefered way to select streams is to listen to
+   * GST_MESSAGE_STREAM_COLLECTION on the bus and send a
+   * GST_EVENT_SELECT_STREAMS with the streams the user wants.
+   *
    * Returns: 1 if the stream should be selected, 0 if it shouldn't be selected.
    * A value of -1 (default) lets @decodebin decide what to do with the stream.
    * */
   gst_decodebin3_signals[SIGNAL_SELECT_STREAM] =
       g_signal_new ("select-stream", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodebin3Class, select_stream),
-      _gst_int_accumulator, NULL, g_cclosure_marshal_generic,
+      _gst_int_accumulator, NULL, NULL,
       G_TYPE_INT, 2, GST_TYPE_STREAM_COLLECTION, GST_TYPE_STREAM);
 
   /**
    * GstDecodebin3::about-to-finish:
    *
    * This signal is emitted when the data for the selected URI is
-   * entirely buffered and it is safe to specify anothe URI.
+   * entirely buffered and it is safe to specify another URI.
    */
   gst_decodebin3_signals[SIGNAL_ABOUT_TO_FINISH] =
       g_signal_new ("about-to-finish", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_generic, G_TYPE_NONE,
-      0, G_TYPE_NONE);
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 0, G_TYPE_NONE);
 
 
   element_class->request_new_pad =
@@ -753,7 +756,7 @@ parsebin_drained_cb (GstElement * parsebin, DecodebinInput * input)
   gboolean all_drained;
   GList *tmp;
 
-  GST_WARNING_OBJECT (dbin, "input %p drained", input);
+  GST_INFO_OBJECT (dbin, "input %p drained", input);
   input->drained = TRUE;
 
   all_drained = dbin->main_input->drained;
@@ -764,7 +767,7 @@ parsebin_drained_cb (GstElement * parsebin, DecodebinInput * input)
   }
 
   if (all_drained) {
-    GST_WARNING_OBJECT (dbin, "All inputs drained. Posting about-to-finish");
+    GST_INFO_OBJECT (dbin, "All inputs drained. Posting about-to-finish");
     g_signal_emit (dbin, gst_decodebin3_signals[SIGNAL_ABOUT_TO_FINISH], 0,
         NULL);
   }
@@ -898,6 +901,7 @@ gst_decodebin3_input_pad_unlink (GstPad * pad, GstObject * parent)
       if (dbin->collection)
         gst_object_unref (dbin->collection);
       dbin->collection = collection;
+      dbin->select_streams_seqnum = GST_SEQNUM_INVALID;
 
       msg =
           gst_message_new_stream_collection ((GstObject *) dbin,
@@ -1075,6 +1079,7 @@ update_requested_selection (GstDecodebin3 * dbin)
 {
   guint i, nb;
   GList *tmp = NULL;
+  gboolean all_user_selected = TRUE;
   GstStreamType used_types = 0;
   GstStreamCollection *collection;
 
@@ -1109,6 +1114,9 @@ update_requested_selection (GstDecodebin3 * dbin)
         gst_decodebin3_signals[SIGNAL_SELECT_STREAM], 0, collection, stream,
         &request);
     GST_DEBUG_OBJECT (dbin, "stream %s , request:%d", sid, request);
+
+    if (request == -1)
+      all_user_selected = FALSE;
     if (request == 1 || (request == -1
             && (stream_in_list (dbin->requested_selection, sid)
                 || stream_in_list (dbin->active_selection, sid)))) {
@@ -1125,16 +1133,19 @@ update_requested_selection (GstDecodebin3 * dbin)
     }
   }
 
-  /* 4. If not, match one stream of each type */
-  for (i = 0; i < nb; i++) {
-    GstStream *stream = gst_stream_collection_get_stream (collection, i);
-    GstStreamType curtype = gst_stream_get_stream_type (stream);
-    if (!(used_types & curtype)) {
-      const gchar *sid = gst_stream_get_stream_id (stream);
-      GST_DEBUG_OBJECT (dbin, "Selecting stream '%s' of type %s",
-          sid, gst_stream_type_get_name (curtype));
-      tmp = g_list_append (tmp, (gchar *) sid);
-      used_types |= curtype;
+  /* 4. If the user didn't explicitly selected all streams, match one stream of each type */
+  if (!all_user_selected && dbin->select_streams_seqnum == GST_SEQNUM_INVALID) {
+    for (i = 0; i < nb; i++) {
+      GstStream *stream = gst_stream_collection_get_stream (collection, i);
+      GstStreamType curtype = gst_stream_get_stream_type (stream);
+      if (!(used_types & curtype)) {
+        const gchar *sid = gst_stream_get_stream_id (stream);
+        GST_DEBUG_OBJECT (dbin,
+            "Automatically selecting stream '%s' of type %s", sid,
+            gst_stream_type_get_name (curtype));
+        tmp = g_list_append (tmp, (gchar *) sid);
+        used_types |= curtype;
+      }
     }
   }
 
@@ -1416,6 +1427,7 @@ handle_stream_collection (GstDecodebin3 * dbin,
     /* dbin->pending_collection = */
     /*     g_list_append (dbin->pending_collection, collection); */
   }
+  dbin->select_streams_seqnum = GST_SEQNUM_INVALID;
   SELECTION_UNLOCK (dbin);
 }
 
@@ -1441,7 +1453,7 @@ gst_decodebin3_handle_message (GstBin * bin, GstMessage * message)
       }
 
       SELECTION_LOCK (dbin);
-      if (dbin->collection && collection != dbin->collection) {
+      if (dbin->collection) {
         /* Replace collection message, we most likely aggregated it */
         GstMessage *new_msg;
         new_msg =
@@ -2197,6 +2209,15 @@ reconfigure_output_stream (DecodebinOutputStream * output,
 
     gst_bin_remove ((GstBin *) dbin, output->decoder);
     output->decoder = NULL;
+  } else if (output->linked) {
+    /* Otherwise if we have no decoder yet but the output is linked make
+     * sure that the ghost pad is really unlinked in case no decoder was
+     * needed previously */
+    if (!gst_ghost_pad_set_target ((GstGhostPad *) output->src_pad, NULL)) {
+      GST_ERROR_OBJECT (dbin, "Could not release ghost pad");
+      gst_caps_unref (new_caps);
+      goto cleanup;
+    }
   }
 
   gst_caps_unref (new_caps);
@@ -2250,6 +2271,20 @@ reconfigure_output_stream (DecodebinOutputStream * output,
     goto cleanup;
   }
   if (output->src_exposed == FALSE) {
+    GstEvent *stream_start;
+
+    stream_start = gst_pad_get_sticky_event (slot->src_pad,
+        GST_EVENT_STREAM_START, 0);
+
+    /* Ensure GstStream is accesiable from pad-added callback */
+    if (stream_start) {
+      gst_pad_store_sticky_event (output->src_pad, stream_start);
+      gst_event_unref (stream_start);
+    } else {
+      GST_WARNING_OBJECT (slot->src_pad,
+          "Pad has no stored stream-start event");
+    }
+
     output->src_exposed = TRUE;
     gst_element_add_pad (GST_ELEMENT_CAST (dbin), output->src_pad);
   }
@@ -2900,6 +2935,8 @@ gst_decodebin3_change_state (GstElement * element, GstStateChange transition)
       dbin->slots = NULL;
       dbin->current_group_id = GST_GROUP_ID_INVALID;
       /* Free inputs */
+      /* Reset the main input group id since it will get a new id on a new stream */
+      dbin->main_input->group_id = GST_GROUP_ID_INVALID;
     }
       break;
     default:
