@@ -21,31 +21,32 @@
 
 /**
  * SECTION:element-multifdsink
+ * @title: multifdsink
  * @see_also: tcpserversink
  *
  * This plugin writes incoming data to a set of file descriptors. The
- * file descriptors can be added to multifdsink by emitting the #GstMultiFdSink::add signal. 
+ * file descriptors can be added to multifdsink by emitting the #GstMultiFdSink::add signal.
  * For each descriptor added, the #GstMultiFdSink::client-added signal will be called.
  *
  * The multifdsink element needs to be set into READY, PAUSED or PLAYING state
  * before operations such as adding clients are possible.
  *
- * As of version 0.10.8, a client can also be added with the #GstMultiFdSink::add-full signal
- * that allows for more control over what and how much data a client 
+ * A client can also be added with the #GstMultiFdSink::add-full signal
+ * that allows for more control over what and how much data a client
  * initially receives.
  *
  * Clients can be removed from multifdsink by emitting the #GstMultiFdSink::remove signal. For
  * each descriptor removed, the #GstMultiFdSink::client-removed signal will be called. The
  * #GstMultiFdSink::client-removed signal can also be fired when multifdsink decides that a
  * client is not active anymore or, depending on the value of the
- * #GstMultiFdSink:recover-policy property, if the client is reading too slowly.
+ * #GstMultiHandleSink:recover-policy property, if the client is reading too slowly.
  * In all cases, multifdsink will never close a file descriptor itself.
  * The user of multifdsink is responsible for closing all file descriptors.
  * This can for example be done in response to the #GstMultiFdSink::client-fd-removed signal.
  * Note that multifdsink still has a reference to the file descriptor when the
  * #GstMultiFdSink::client-removed signal is emitted, so that "get-stats" can be performed on
  * the descriptor; it is therefore not safe to close the file descriptor in
- * the #GstMultiFdSink::client-removed signal handler, and you should use the 
+ * the #GstMultiFdSink::client-removed signal handler, and you should use the
  * #GstMultiFdSink::client-fd-removed signal to safely close the fd.
  *
  * Multifdsink internally keeps a queue of the incoming buffers and uses a
@@ -53,52 +54,50 @@
  * client write can block the pipeline and that clients can read with different
  * speeds.
  *
- * When adding a client to multifdsink, the #GstMultiFdSink:sync-method property will define
- * which buffer in the queued buffers will be sent first to the client. Clients 
- * can be sent the most recent buffer (which might not be decodable by the 
- * client if it is not a keyframe), the next keyframe received in 
+ * When adding a client to multifdsink, the #GstMultiHandleSink:sync-method property will define
+ * which buffer in the queued buffers will be sent first to the client. Clients
+ * can be sent the most recent buffer (which might not be decodable by the
+ * client if it is not a keyframe), the next keyframe received in
  * multifdsink (which can take some time depending on the keyframe rate), or the
- * last received keyframe (which will cause a simple burst-on-connect). 
+ * last received keyframe (which will cause a simple burst-on-connect).
  * Multifdsink will always keep at least one keyframe in its internal buffers
  * when the sync-mode is set to latest-keyframe.
  *
- * As of version 0.10.8, there are additional values for the #GstMultiFdSink:sync-method 
+ * There are additional values for the #GstMultiHandleSink:sync-method
  * property to allow finer control over burst-on-connect behaviour. By selecting
  * the 'burst' method a minimum burst size can be chosen, 'burst-keyframe'
- * additionally requires that the burst begin with a keyframe, and 
+ * additionally requires that the burst begin with a keyframe, and
  * 'burst-with-keyframe' attempts to burst beginning with a keyframe, but will
  * prefer a minimum burst size even if it requires not starting with a keyframe.
  *
  * Multifdsink can be instructed to keep at least a minimum amount of data
- * expressed in time or byte units in its internal queues with the 
- * #GstMultiFdSink:time-min and #GstMultiFdSink:bytes-min properties respectively.
- * These properties are useful if the application adds clients with the 
+ * expressed in time or byte units in its internal queues with the
+ * #GstMultiHandleSink:time-min and #GstMultiHandleSink:bytes-min properties respectively.
+ * These properties are useful if the application adds clients with the
  * #GstMultiFdSink::add-full signal to make sure that a burst connect can
- * actually be honored. 
+ * actually be honored.
  *
  * When streaming data, clients are allowed to read at a different rate than
  * the rate at which multifdsink receives data. If the client is reading too
  * fast, no data will be send to the client until multifdsink receives more
- * data. If the client, however, reads too slowly, data for that client will be 
- * queued up in multifdsink. Two properties control the amount of data 
- * (buffers) that is queued in multifdsink: #GstMultiFdSink:buffers-max and 
- * #GstMultiFdSink:buffers-soft-max. A client that falls behind by
- * #GstMultiFdSink:buffers-max is removed from multifdsink forcibly.
+ * data. If the client, however, reads too slowly, data for that client will be
+ * queued up in multifdsink. Two properties control the amount of data
+ * (buffers) that is queued in multifdsink: #GstMultiHandleSink:buffers-max and
+ * #GstMultiHandleSink:buffers-soft-max. A client that falls behind by
+ * #GstMultiHandleSink:buffers-max is removed from multifdsink forcibly.
  *
- * A client with a lag of at least #GstMultiFdSink:buffers-soft-max enters the recovery
- * procedure which is controlled with the #GstMultiFdSink:recover-policy property.
+ * A client with a lag of at least #GstMultiHandleSink:buffers-soft-max enters the recovery
+ * procedure which is controlled with the #GstMultiHandleSink:recover-policy property.
  * A recover policy of NONE will do nothing, RESYNC_LATEST will send the most recently
  * received buffer as the next buffer for the client, RESYNC_SOFT_LIMIT
  * positions the client to the soft limit in the buffer queue and
  * RESYNC_KEYFRAME positions the client at the most recent keyframe in the
  * buffer queue.
  *
- * multifdsink will by default synchronize on the clock before serving the 
- * buffers to the clients. This behaviour can be disabled by setting the sync 
+ * multifdsink will by default synchronize on the clock before serving the
+ * buffers to the clients. This behaviour can be disabled by setting the sync
  * property to FALSE. Multifdsink will by default not do QoS and will never
  * drop late buffers.
- *
- * Last reviewed on 2006-09-12 (0.10.10)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -120,10 +119,6 @@
 #include <sys/stat.h>
 #include <netinet/in.h>
 
-#ifdef HAVE_FIONREAD_IN_SYS_FILIO
-#include <sys/filio.h>
-#endif
-
 #include "gstmultifdsink.h"
 
 #define NOT_IMPLEMENTED 0
@@ -155,8 +150,7 @@ enum
 enum
 {
   PROP_0,
-  PROP_HANDLE_READ,
-  PROP_LAST
+  PROP_HANDLE_READ
 };
 
 static void gst_multi_fd_sink_stop_pre (GstMultiHandleSink * mhsink);
@@ -221,8 +215,6 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
    * GstMultiFdSink::handle-read
    *
    * Handle read requests from clients and discard the data.
-   *
-   * Since: 0.10.23
    */
   g_object_class_install_property (gobject_class, PROP_HANDLE_READ,
       g_param_spec_boolean ("handle-read", "Handle Read",
@@ -240,7 +232,7 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
       g_signal_new ("add", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
       G_STRUCT_OFFSET (GstMultiFdSinkClass, add), NULL, NULL,
-      g_cclosure_marshal_generic, G_TYPE_NONE, 1, G_TYPE_INT);
+      NULL, G_TYPE_NONE, 1, G_TYPE_INT);
   /**
    * GstMultiFdSink::add-full:
    * @gstmultifdsink:  the multifdsink element to emit this signal on
@@ -260,9 +252,8 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
       g_signal_new ("add-full", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
       G_STRUCT_OFFSET (GstMultiFdSinkClass, add_full), NULL, NULL,
-      g_cclosure_marshal_generic, G_TYPE_NONE, 6,
-      G_TYPE_INT, GST_TYPE_SYNC_METHOD, GST_TYPE_FORMAT, G_TYPE_UINT64,
-      GST_TYPE_FORMAT, G_TYPE_UINT64);
+      NULL, G_TYPE_NONE, 6, G_TYPE_INT, GST_TYPE_SYNC_METHOD, GST_TYPE_FORMAT,
+      G_TYPE_UINT64, GST_TYPE_FORMAT, G_TYPE_UINT64);
   /**
    * GstMultiFdSink::remove:
    * @gstmultifdsink: the multifdsink element to emit this signal on
@@ -274,7 +265,7 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
       g_signal_new ("remove", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
       G_STRUCT_OFFSET (GstMultiFdSinkClass, remove), NULL, NULL,
-      g_cclosure_marshal_generic, G_TYPE_NONE, 1, G_TYPE_INT);
+      NULL, G_TYPE_NONE, 1, G_TYPE_INT);
   /**
    * GstMultiFdSink::remove-flush:
    * @gstmultifdsink: the multifdsink element to emit this signal on
@@ -287,29 +278,33 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
       g_signal_new ("remove-flush", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
       G_STRUCT_OFFSET (GstMultiFdSinkClass, remove_flush), NULL, NULL,
-      g_cclosure_marshal_generic, G_TYPE_NONE, 1, G_TYPE_INT);
+      NULL, G_TYPE_NONE, 1, G_TYPE_INT);
 
   /**
    * GstMultiFdSink::get-stats:
    * @gstmultifdsink: the multifdsink element to emit this signal on
    * @fd:             the file descriptor to get stats of from multifdsink
    *
-   * Get statistics about @fd. This function returns a GValueArray to ease
+   * Get statistics about @fd. This function returns a #GstStructure to ease
    * automatic wrapping for bindings.
    *
-   * Returns: a GValueArray with the statistics. The array contains guint64
-   *     values that represent respectively: total number of bytes sent, time
-   *     when the client was added, time when the client was
-   *     disconnected/removed, time the client is/was active, last activity
-   *     time (in epoch seconds), number of buffers dropped.
-   *     All times are expressed in nanoseconds (GstClockTime).
-   *     The array can be 0-length if the client was not found.
+   * Returns: a #GstStructure with the statistics. The structures
+   *     contains guint64 values that represent respectively: total
+   *     number of bytes sent (bytes-sent), time when the client was
+   *     added (connect-time), time when the client was
+   *     disconnected/removed (disconnect-time), time the client
+   *     is/was active (connect-duration), last activity time (in
+   *     epoch seconds) (last-activity-time), number of buffers
+   *     dropped (buffers-dropped), the timestamp of the first buffer
+   *     (first-buffer-ts) and of the last buffer (last-buffer-ts).
+   *     All times are expressed in nanoseconds (GstClockTime).  The
+   *     structure can be empty if the client was not found.
    */
   gst_multi_fd_sink_signals[SIGNAL_GET_STATS] =
       g_signal_new ("get-stats", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
       G_STRUCT_OFFSET (GstMultiFdSinkClass, get_stats), NULL, NULL,
-      g_cclosure_marshal_generic, GST_TYPE_STRUCTURE, 1, G_TYPE_INT);
+      NULL, GST_TYPE_STRUCTURE, 1, G_TYPE_INT);
 
   /**
    * GstMultiFdSink::client-added:
@@ -322,8 +317,7 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
    */
   gst_multi_fd_sink_signals[SIGNAL_CLIENT_ADDED] =
       g_signal_new ("client-added", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_generic, G_TYPE_NONE,
-      1, G_TYPE_INT);
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_INT);
   /**
    * GstMultiFdSink::client-removed:
    * @gstmultifdsink: the multifdsink element that emitted this signal
@@ -336,12 +330,12 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
    *
    * @gstmultifdsink still holds a handle to @fd so it is possible to call
    * the get-stats signal from this callback. For the same reason it is
-   * not safe to close() and reuse @fd in this callback.
+   * not safe to `close()` and reuse @fd in this callback.
    */
   gst_multi_fd_sink_signals[SIGNAL_CLIENT_REMOVED] =
       g_signal_new ("client-removed", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_generic,
-      G_TYPE_NONE, 2, G_TYPE_INT, GST_TYPE_CLIENT_STATUS);
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 2, G_TYPE_INT,
+      GST_TYPE_CLIENT_STATUS);
   /**
    * GstMultiFdSink::client-fd-removed:
    * @gstmultifdsink: the multifdsink element that emitted this signal
@@ -353,14 +347,11 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
    *
    * In this callback, @gstmultifdsink has removed all the information
    * associated with @fd and it is therefore not possible to call get-stats
-   * with @fd. It is however safe to close() and reuse @fd in the callback.
-   *
-   * Since: 0.10.7
+   * with @fd. It is however safe to `close()` and reuse @fd in the callback.
    */
   gst_multi_fd_sink_signals[SIGNAL_CLIENT_FD_REMOVED] =
       g_signal_new ("client-fd-removed", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_generic,
-      G_TYPE_NONE, 1, G_TYPE_INT);
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_INT);
 
   gst_element_class_set_static_metadata (gstelement_class,
       "Multi filedescriptor sink", "Sink/Network",
@@ -576,7 +567,7 @@ gst_multi_fd_sink_hash_changed (GstMultiHandleSink * mhsink)
 
 /* handle a read on a client fd,
  * which either indicates a close or should be ignored
- * returns FALSE if some error occured or the client closed. */
+ * returns FALSE if some error occurred or the client closed. */
 static gboolean
 gst_multi_fd_sink_handle_client_read (GstMultiFdSink * sink,
     GstTCPClient * client)
@@ -669,7 +660,7 @@ ioctl_failed:
  * When the sending returns a partial buffer we stop sending more data as
  * the next send operation could block.
  *
- * This functions returns FALSE if some error occured.
+ * This functions returns FALSE if some error occurred.
  */
 static gboolean
 gst_multi_fd_sink_handle_client_write (GstMultiFdSink * sink,
@@ -678,7 +669,6 @@ gst_multi_fd_sink_handle_client_write (GstMultiFdSink * sink,
   gboolean more;
   gboolean flushing;
   GstClockTime now;
-  GTimeVal nowtv;
   GstMultiHandleSink *mhsink = GST_MULTI_HANDLE_SINK (sink);
   GstMultiHandleSinkClass *mhsinkclass =
       GST_MULTI_HANDLE_SINK_GET_CLASS (mhsink);
@@ -691,8 +681,7 @@ gst_multi_fd_sink_handle_client_write (GstMultiFdSink * sink,
   do {
     gint maxsize;
 
-    g_get_current_time (&nowtv);
-    now = GST_TIMEVAL_TO_TIME (nowtv);
+    now = g_get_real_time () * GST_USECOND;
 
     if (!mhclient->sending) {
       /* client is not working on a buffer */
@@ -910,10 +899,8 @@ gst_multi_fd_sink_handle_clients (GstMultiFdSink * sink)
      * and will not disconnect inactive client in the streaming thread. */
     if (G_UNLIKELY (result == 0)) {
       GstClockTime now;
-      GTimeVal nowtv;
 
-      g_get_current_time (&nowtv);
-      now = GST_TIMEVAL_TO_TIME (nowtv);
+      now = g_get_real_time () * GST_USECOND;
 
       CLIENTS_LOCK (mhsink);
       for (clients = mhsink->clients; clients; clients = next) {
@@ -959,7 +946,7 @@ gst_multi_fd_sink_handle_clients (GstMultiFdSink * sink)
 
           res = fcntl (fd, F_GETFL, &flags);
           if (res == -1) {
-            GST_WARNING_OBJECT (sink, "fnctl failed for %d, removing: %s (%d)",
+            GST_WARNING_OBJECT (sink, "fcntl failed for %d, removing: %s (%d)",
                 fd, g_strerror (errno), errno);
             if (errno == EBADF) {
               mhclient->status = GST_CLIENT_STATUS_ERROR;
