@@ -52,8 +52,6 @@ GST_DEBUG_CATEGORY (videoconvert_debug);
 #define GST_CAT_DEFAULT videoconvert_debug
 GST_DEBUG_CATEGORY_STATIC (CAT_PERFORMANCE);
 
-GType gst_video_convert_get_type (void);
-
 static GQuark _colorspace_quark;
 
 #define gst_video_convert_parent_class parent_class
@@ -149,7 +147,7 @@ gst_video_convert_caps_remove_format_info (GstCaps * caps)
 }
 
 /*
- * This is an incomplete matrix of in formats and a score for the prefered output
+ * This is an incomplete matrix of in formats and a score for the preferred output
  * format.
  *
  *         out: RGB24   RGB16  ARGB  AYUV  YUV444  YUV422 YUV420 YUV411 YUV410  PAL  GRAY
@@ -166,9 +164,9 @@ gst_video_convert_caps_remove_format_info (GstCaps * caps)
  * PAL            1      3       2     6     4       6      7      8      9      0    10
  * GRAY           1      4       3     2     1       5      6      7      8      9    0
  *
- * PAL or GRAY are never prefered, if we can we would convert to PAL instead
+ * PAL or GRAY are never preferred, if we can we would convert to PAL instead
  * of GRAY, though
- * less subsampling is prefered and if any, preferably horizontal
+ * less subsampling is preferred and if any, preferably horizontal
  * We would like to keep the alpha, even if we would need to to colorspace conversion
  * or lose depth.
  */
@@ -432,6 +430,9 @@ gst_video_convert_set_info (GstVideoFilter * filter,
     GstVideoInfo * out_info)
 {
   GstVideoConvert *space;
+  GstBaseTransformClass *gstbasetransform_class =
+      GST_BASE_TRANSFORM_GET_CLASS (filter);
+  GstVideoInfo tmp_info;
 
   space = GST_VIDEO_CONVERT_CAST (filter);
 
@@ -453,6 +454,21 @@ gst_video_convert_set_info (GstVideoFilter * filter,
   if (in_info->interlace_mode != out_info->interlace_mode)
     goto format_mismatch;
 
+  /* if the only thing different in the caps is the transfer function, and
+   * we're converting between equivalent transfer functions, do passthrough */
+  tmp_info = *in_info;
+  tmp_info.colorimetry.transfer = out_info->colorimetry.transfer;
+  if (gst_video_info_is_equal (&tmp_info, out_info)) {
+    if (gst_video_transfer_function_is_equivalent (in_info->
+            colorimetry.transfer, in_info->finfo->bits,
+            out_info->colorimetry.transfer, out_info->finfo->bits)) {
+      gstbasetransform_class->passthrough_on_same_caps = FALSE;
+      gst_base_transform_set_passthrough (GST_BASE_TRANSFORM (filter), TRUE);
+      return TRUE;
+    }
+  }
+  gstbasetransform_class->passthrough_on_same_caps = TRUE;
+  gst_base_transform_set_passthrough (GST_BASE_TRANSFORM (filter), FALSE);
 
   space->convert = gst_video_converter_new (in_info, out_info,
       gst_structure_new ("GstVideoConvertConfig",
