@@ -29,16 +29,9 @@
 
 G_BEGIN_DECLS
 
-#define GST_TYPE_VIDEO_TEST_SRC \
-  (gst_video_test_src_get_type())
-#define GST_VIDEO_TEST_SRC(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_VIDEO_TEST_SRC,GstVideoTestSrc))
-#define GST_VIDEO_TEST_SRC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VIDEO_TEST_SRC,GstVideoTestSrcClass))
-#define GST_IS_VIDEO_TEST_SRC(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VIDEO_TEST_SRC))
-#define GST_IS_VIDEO_TEST_SRC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VIDEO_TEST_SRC))
+#define GST_TYPE_VIDEO_TEST_SRC (gst_video_test_src_get_type())
+G_DECLARE_FINAL_TYPE (GstVideoTestSrc, gst_video_test_src, GST, VIDEO_TEST_SRC,
+    GstPushSrc)
 
 /**
  * GstVideoTestSrcPattern:
@@ -116,8 +109,18 @@ typedef enum {
   GST_VIDEO_TEST_SRC_COLORS
 } GstVideoTestSrcPattern;
 
-typedef struct _GstVideoTestSrc GstVideoTestSrc;
-typedef struct _GstVideoTestSrcClass GstVideoTestSrcClass;
+typedef enum {
+  GST_VIDEO_TEST_SRC_FRAMES,
+  GST_VIDEO_TEST_SRC_WALL_TIME,
+  GST_VIDEO_TEST_SRC_RUNNING_TIME
+} GstVideoTestSrcAnimationMode;
+
+
+typedef enum {
+  GST_VIDEO_TEST_SRC_WAVY,
+  GST_VIDEO_TEST_SRC_SWEEP,
+  GST_VIDEO_TEST_SRC_HSWEEP
+} GstVideoTestSrcMotionType;
 
 /**
  * GstVideoTestSrc:
@@ -133,7 +136,7 @@ struct _GstVideoTestSrc {
   GstVideoTestSrcPattern pattern_type;
 
   /* video state */
-  GstVideoInfo info;
+  GstVideoInfo info; /* protected by the object or stream lock */
   GstVideoChromaResample *subsample;
   gboolean bayer;
   gint x_invert;
@@ -174,7 +177,15 @@ struct _GstVideoTestSrc {
   gint horizontal_offset;
   gint horizontal_speed;
 
-  void (*make_image) (GstVideoTestSrc *v, GstVideoFrame *frame);
+  /* smpte & snow */
+  guint random_state;
+
+  /* Ball motion */
+  GstVideoTestSrcAnimationMode animation_mode;
+  GstVideoTestSrcMotionType motion_type;
+  gboolean flip;
+
+  void (*make_image) (GstVideoTestSrc *v, GstClockTime pts, GstVideoFrame *frame);
 
   /* temporary AYUV/ARGB scanline */
   guint8 *tmpline_u8;
@@ -187,12 +198,6 @@ struct _GstVideoTestSrc {
   gpointer *lines;
 };
 
-struct _GstVideoTestSrcClass {
-  GstPushSrcClass parent_class;
-};
-
-GType gst_video_test_src_get_type (void);
-
 G_END_DECLS
 
 #endif /* __GST_VIDEO_TEST_SRC_H__ */
